@@ -4,11 +4,12 @@
 # frescobaldiui.rc :
 i18n("&Insert")
 i18n("&Document")
+i18n("&LilyPond")
+i18n("&Sessions")
 i18n("&Pitch")
 i18n("&Lyrics")
 i18n("&Rhythm")
 i18n("&Bar Lines")
-i18n("&Generated Files")
 i18n("&Source Document")
 i18n("&Extra Tools")
 
