@@ -17,12 +17,14 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 A wizard to create empty staff paper with LilyPond
 """
-import os, sip
+import sip
 
-from PyQt4.QtCore import QObject, QSize, Qt, QVariant, SIGNAL
+from PyQt4.QtCore import QSize, Qt
 from PyQt4.QtGui import (
     QCheckBox, QComboBox, QGridLayout, QGroupBox, QHBoxLayout, QIcon, QLabel,
     QPixmap, QPushButton, QSpinBox, QStackedWidget, QToolButton, QTreeWidget,
@@ -30,16 +32,16 @@ from PyQt4.QtGui import (
 
 from PyKDE4.kdecore import KGlobal, KUrl, i18n
 from PyKDE4.kdeui import (
-    KDialog, KGuiItem, KIcon, KMessageBox, KPushButton, KStandardGuiItem)
+    KDialog, KIcon, KMessageBox, KPushButton, KStandardGuiItem)
 from PyKDE4.kio import KFileDialog, KIO
 
 import ly, ly.indent
-from kateshell.app import lazymethod
-from frescobaldi_app.mainapp import SymbolManager
-from frescobaldi_app.version import defaultVersion
-from frescobaldi_app.widgets import StackFader, ClefSelector
+from kateshell.app import cacheresult
+from kateshell.widgets import StackFader
+from frescobaldi_app.mainapp import SymbolManager, lilyPondVersion
+from frescobaldi_app.widgets import ClefSelector
 from frescobaldi_app.runlily import BackgroundJob, LilyPreviewDialog
-from frescobaldi_app.actions import openPDF, printPDF
+from frescobaldi_app.actions import openPDF, printPDF, getPrinter
 
 
 class Dialog(KDialog):
@@ -134,10 +136,8 @@ class Dialog(KDialog):
         settings.layout().addWidget(self.pageNumbers, 2, 0, 1, 2)
         settings.layout().addWidget(l, 3, 0, Qt.AlignRight)
         settings.layout().addWidget(self.pageNumStart, 3, 1)
-        QObject.connect(self.barLines, SIGNAL("toggled(bool)"),
-            self.barsPerLine.setEnabled)
-        QObject.connect(self.pageNumbers, SIGNAL("toggled(bool)"),
-            self.pageNumStart.setEnabled)
+        self.barLines.toggled.connect(self.barsPerLine.setEnabled)
+        self.pageNumbers.toggled.connect(self.pageNumStart.setEnabled)
         
         # types
         self.typeWidgets = [
@@ -150,13 +150,13 @@ class Dialog(KDialog):
         for widget in self.typeWidgets:
             self.stack.addWidget(widget)
             self.typeChooser.addItem(widget.name())
-        QObject.connect(self.typeChooser, SIGNAL("currentIndexChanged(int)"),
-            lambda index: self.stack.setCurrentWidget(self.typeWidgets[index]))
+        self.typeChooser.currentIndexChanged.connect(lambda index:
+            self.stack.setCurrentWidget(self.typeWidgets[index]))
 
         self.actors = [
-            OpenPDF,
-            SavePDF,
             PrintPDF,
+            SavePDF,
+            OpenPDF,
             CopyToEditor,
             ]
         for actor in self.actors:
@@ -166,8 +166,8 @@ class Dialog(KDialog):
         # cleanup on exit
         mainwin.aboutToClose.connect(self.cleanup)
         # buttons
-        QObject.connect(self, SIGNAL("resetClicked()"), self.default)
-        QObject.connect(self, SIGNAL("tryClicked()"), self.showPreview)
+        self.resetClicked.connect(self.default)
+        self.tryClicked.connect(self.showPreview)
         self.setInitialSize(QSize(400, 240))
         self.default()
         self.loadSettings()
@@ -196,19 +196,23 @@ class Dialog(KDialog):
             widget.default()
 
     def loadSettings(self):
-        # only remember option to remove default FRESCOBALDI.ORG tagline
-        self.removeTagline.setChecked(config().readEntry(
-            "remove tagline", QVariant(False)).toBool())
+        conf = config()
+        self.removeTagline.setChecked(conf.readEntry("remove tagline", False))
+        action = conf.readEntry("action", "PrintPDF")
+        for index, actor in enumerate(self.actors):
+            if actor.__name__ == action:
+                self.actionChooser.setCurrentIndex(index)
     
     def saveSettings(self):
-        # only remember option to remove default FRESCOBALDI.ORG tagline
-        config().writeEntry(
-            "remove tagline", QVariant(self.removeTagline.isChecked()))
+        conf = config()
+        conf.writeEntry("remove tagline", self.removeTagline.isChecked())
+        action = self.actors[self.actionChooser.currentIndex()].__name__
+        conf.writeEntry("action", action)
     
     def showPreview(self):
         self.previewDialog().showPreview(self.ly())
 
-    @lazymethod
+    @cacheresult
     def previewDialog(self):
         return PreviewDialog(self)
 
@@ -218,17 +222,17 @@ class Dialog(KDialog):
         """
         staff = self.stack.currentWidget()
         output = []
-        version = defaultVersion()
+        version = lilyPondVersion()
         if version:
-            output.append('\\version "%s"\n' % version)
-        output.append('#(set-global-staff-size %d)\n' % self.staffSize.value())
+            output.append('\\version "{0}"\n'.format(version))
+        output.append('#(set-global-staff-size {0})\n'.format(self.staffSize.value()))
         # paper section
         output.append('\\paper {')
         if self.paperSize.currentIndex() > 0:
-            output.append('#(set-paper-size "%s")' % ly.paperSizes[self.paperSize.currentIndex()-1])
+            output.append('#(set-paper-size "{0}")'.format(ly.paperSizes[self.paperSize.currentIndex()-1]))
         if self.pageNumbers.isChecked():
             output.append('top-margin = 10\\mm')
-            output.append('first-page-number = #%d' % self.pageNumStart.value())
+            output.append('first-page-number = #{0}'.format(self.pageNumStart.value()))
             output.append('oddHeaderMarkup = \\markup \\fill-line {')
             output.append('\\strut')
             output.append("\\fromproperty #'page:page-number-string")
@@ -243,10 +247,9 @@ class Dialog(KDialog):
         else:
             output.append('bottom-margin = 10\\mm')
             output.append('oddFooterMarkup = \\markup \\abs-fontsize #6 \\fill-line {')
-            tagline = config().readEntry("tagline", QVariant(
-                '\\with-url #"http://www.frescobaldi.org/" FRESCOBALDI.ORG'
-                )).toString()
-            output.append('\\sans { %s }' % tagline)
+            tagline = config().readEntry("tagline",
+                '\\with-url #"http://www.frescobaldi.org/" FRESCOBALDI.ORG')
+            output.append('\\sans {{ {0} }}'.format(tagline))
             output.append('\\strut')
             output.append('}')
         output.append('evenFooterMarkup = ##f')
@@ -254,9 +257,9 @@ class Dialog(KDialog):
         output.append('ragged-right = ##f')
         output.append('}\n')
         # music expression
-        output.append('music = \\repeat unfold %d { %% pages' % self.pageCount.value())
-        output.append('\\repeat unfold %d { %% systems' % staff.systemCount())
-        output.append('\\repeat unfold %d { %% bars' % (
+        output.append('music = \\repeat unfold {0} {{ % pages'.format(self.pageCount.value()))
+        output.append('\\repeat unfold {0} {{ % systems'.format(staff.systemCount()))
+        output.append('\\repeat unfold {0} {{ % bars'.format(
             self.barLines.isChecked() and self.barsPerLine.value() or 1))
         output.extend(('s1', '\\noBreak', '}', '\\break', '\\noPageBreak', '}', '\\pageBreak', '}\n'))
 
@@ -331,36 +334,17 @@ class SavePDF(BlankPaperJob):
         BlankPaperJob.__init__(self, dialog)
         self.sourcePDF = None
         self.targetPDF = None
-        dlg = KFileDialog(KUrl(), '*.pdf|%s\n*|%s' % (
+        dlg = KFileDialog(KUrl(), '*.pdf|{0}\n*|{1}'.format(
             i18n("PDF Files"), i18n("All Files")), dialog)
         dlg.setOperationMode(KFileDialog.Saving)
         dlg.setCaption(i18n("Save PDF"))
-        
-        # because KEncodingFileDialog.setConfirmOverwrite() is not available
-        # in KDE 4.1, we try to use it, but fall back to our old own
-        # implementation.
-        checkOverwrite = False
-        try:
-            dlg.setConfirmOverwrite(True)
-        except AttributeError:
-            checkOverwrite = True
-        
+        dlg.setConfirmOverwrite(True)
         dlg.setSelection('staffpaper.pdf')
         if dlg.exec_():
-            url = dlg.selectedUrl()
-            # now confirm overwrite if the dialog didn't handle that (KDE < 4.2)
-            if not (checkOverwrite and url.isLocalFile()
-                    and os.path.exists(unicode(url.path()))
-                    and KMessageBox.warningContinueCancel(dialog,
-                        i18n("A file named \"%1\" already exists. "
-                            "Are you sure you want to overwrite it?",
-                            url.fileName()),
-                        i18n("Overwrite File?"), KGuiItem(i18n("&Overwrite"))) ==
-                        KMessageBox.Cancel):
-                self.targetPDF = url
-                self.savePDF()
-                return
-        self.cleanup()
+            self.targetPDF = dlg.selectedUrl()
+            self.savePDF()
+        else:
+            self.cleanup()
         
     def handlePDF(self, fileName):
         self.sourcePDF = fileName
@@ -377,10 +361,27 @@ class SavePDF(BlankPaperJob):
 class PrintPDF(BlankPaperJob):
     @staticmethod
     def name():
-        return i18n("Directly print on default printer")
+        return i18n("Print...")
 
+    def __init__(self, dialog):
+        BlankPaperJob.__init__(self, dialog)
+        self.printer = None
+        self.fileName = None
+        printer = getPrinter(dialog, i18n("Print staff paper"))
+        if printer:
+            self.printer = printer
+            self.printPDF()
+        else:
+            self.cleanup()
+            
     def handlePDF(self, fileName):
-        printPDF(fileName, self.dialog)
+        self.fileName = fileName
+        self.printPDF()
+        
+    def printPDF(self):
+        """ Prints if the printer and the PDF are both there."""
+        if self.printer and self.fileName:
+            printPDF(self.fileName, self.dialog, self.printer)
 
 
 class CopyToEditor(object):
@@ -458,7 +459,7 @@ class LayoutContexts(object):
         for name, lines in self._contexts.iteritems():
             if lines:
                 result.append('\\context {')
-                result.append('\\%s' % name)
+                result.append('\\' + name)
                 result.extend(lines)
                 result.append('}')
         return result
@@ -507,7 +508,7 @@ class SingleStaff(StaffBase):
             return ['\\new TabStaff { \\music }']
         else:
             if self.clef.clef():
-                return ['\\new Staff { \\clef "%s" \\music }' % self.clef.clef()]
+                return ['\\new Staff {{ \\clef "{0}" \\music }}'.format(self.clef.clef())]
             else:
                 layout.add('Staff', '\\remove "Clef_engraver"')
                 return ['\\new Staff { \\music }']
@@ -534,11 +535,15 @@ class PianoStaff(StaffBase):
         layout.setSpanBarContexts(['PianoStaff'])
         if not self.clefs.isChecked():
             layout.add('Staff', '\\remove "Clef_engraver"')
+        if lilyPondVersion() < (2, 13, 4):
+            spacing = "#'minimum-Y-extent = #'(-6 . 3)"
+        else:
+            spacing = "#'next-staff-spacing = #'((space . 10))"
         return ['\\new PianoStaff <<',
-            '\\new Staff { \\clef treble \\music }',
             '\\new Staff \\with {',
-            "\\override VerticalAxisGroup #'minimum-Y-extent = #'(-3 . 6)",
-            '} { \\clef bass \\music }',
+            '\\override VerticalAxisGroup ' + spacing,
+            '} { \\clef treble \\music }',
+            '\\new Staff { \\clef bass \\music }',
             '>>']
 
 
@@ -551,13 +556,21 @@ class OrganStaff(PianoStaff):
         self.clefs.setChecked(True)
     
     def music(self, layout):
-        PianoStaff.music(self, layout)
+        layout.setSpanBarContexts(['GrandStaff'])
+        if not self.clefs.isChecked():
+            layout.add('Staff', '\\remove "Clef_engraver"')
+        if lilyPondVersion() < (2, 13, 4):
+            spacing = "#'minimum-Y-extent = #'(-6 . 3)"
+        else:
+            spacing = "#'next-staff-spacing = #'((space . 12) (stretchability . 1))"
+            layout.add('Staff', "\\override VerticalAxisGroup "
+                "#'next-staff-spacing = #'((space . 10) (stretchability . 1))")
         return ['<<',
-            '\\new PianoStaff <<',
-            '\\new Staff { \\clef treble \\music }',
+            '\\new GrandStaff <<',
             '\\new Staff \\with {',
-            "\\override VerticalAxisGroup #'minimum-Y-extent = #'(-5 . 6)",
-            '} { \\clef bass \\music }',
+            '\\override VerticalAxisGroup ' + spacing,
+            '} { \\clef treble \\music }',
+            '\\new Staff { \\clef bass \\music }',
             '>>',
             '\\new Staff { \\clef bass \\music }',
             '>>']
@@ -598,8 +611,7 @@ class ChoirStaff(StaffBase):
             "choir score, first set the number of staves per system to 4. "
             "Then enter \"SATB\" (without the quotes) here."))
         
-        QObject.connect(self.staffCount, SIGNAL("valueChanged(int)"),
-            self.slotStaffCountChanged)
+        self.staffCount.valueChanged.connect(self.slotStaffCountChanged)
         self.slotStaffCountChanged(2)    
         
     def name(self):
@@ -627,7 +639,7 @@ class ChoirStaff(StaffBase):
         self.systems.setValue(systemCount)
 
     def music(self, layout):
-        clefs = unicode(self.clefs.currentText()).upper()
+        clefs = self.clefs.currentText().upper()
         length = self.staffCount.value()
         if clefs and set(clefs) <= set("SATB"):
             # pad to staff count if necessary
@@ -635,17 +647,18 @@ class ChoirStaff(StaffBase):
         else:
             clefs = [None] * length
             layout.add("Staff", '\\remove "Clef_engraver"')
-        layout.add("Staff",
-            "\\override VerticalAxisGroup #'minimum-Y-extent = #'(-6 . 4)")
+        if lilyPondVersion() < (2, 13, 4):
+            layout.add("Staff",
+                "\\override VerticalAxisGroup #'minimum-Y-extent = #'(-6 . 4)")
         music = ['\\new ChoirStaff <<']
         for clef in clefs:
-            music.append('\\new Staff {%s \\music }' % {
+            music.append('\\new Staff {{{0} \\music }}'.format({
                 'S': ' \\clef treble',
                 'A': ' \\clef treble',
                 'T': ' \\clef "treble_8"',
                 'B': ' \\clef bass',
                 None: '',
-                }[clef])
+                }[clef]))
         music.append('>>')
         return music
 
@@ -676,23 +689,23 @@ class CustomStaff(SymbolManager, QWidget):
         operations.setSpacing(0)
         operations.setContentsMargins(0, 0, 0, 0)
         removeButton = KPushButton(KStandardGuiItem.remove())
+        removeButton.clicked.connect(self.removeSelectedItems)
         operations.addWidget(removeButton)
         upButton = QToolButton()
+        upButton.clicked.connect(self.moveSelectedItemsUp)
         upButton.setIcon(KIcon("go-up"))
         operations.addWidget(upButton)
         downButton = QToolButton()
+        downButton.clicked.connect(self.moveSelectedItemsDown)
         downButton.setIcon(KIcon("go-down"))
         operations.addWidget(downButton)
-        QObject.connect(removeButton, SIGNAL("clicked()"), self.removeSelectedItems)
-        QObject.connect(upButton, SIGNAL("clicked()"), self.moveSelectedItemsUp)
-        QObject.connect(downButton, SIGNAL("clicked()"), self.moveSelectedItemsDown)
         newStaves.setSpacing(0)
         newStaves.setContentsMargins(0, 0, 0, 0)
         
         self.tree.setIconSize(QSize(32, 32))
         self.tree.setDragDropMode(QTreeWidget.InternalMove)
         self.tree.headerItem().setHidden(True)
-        QObject.connect(self.tree, SIGNAL("itemSelectionChanged()"), self.slotSelectionChanged)
+        self.tree.itemSelectionChanged.connect(self.slotSelectionChanged)
         
         for staffType in (
             BracketItem,
@@ -700,9 +713,8 @@ class CustomStaff(SymbolManager, QWidget):
             StaffItem,
             ):
             b = QPushButton(staffType.name())
+            b.clicked.connect((lambda t: lambda: self.createItem(t))(staffType))
             b.setIconSize(QSize(40, 40))
-            QObject.connect(b, SIGNAL("clicked()"),
-                lambda t=staffType: self.createItem(t))
             self.addSymbol(b, staffType.symbol())
             newStaves.addWidget(b)
         
@@ -829,46 +841,41 @@ class StaffItem(Item):
         l.setBuddy(self.clef)
         w.layout().addWidget(l, 0, 0, Qt.AlignRight)
         w.layout().addWidget(self.clef, 0, 1)
-        QObject.connect(self.clef, SIGNAL("currentIndexChanged(int)"), self.clefChanged)
-        self.spaceAbove = QSpinBox()
-        self.spaceAbove.setRange(0, 25)
-        self.spaceAbove.setValue(5)
+        self.clef.currentIndexChanged.connect(self.clefChanged)
         self.spaceBelow = QSpinBox()
         self.spaceBelow.setRange(0, 25)
         self.spaceBelow.setValue(5)
-        w.layout().addWidget(self.spaceAbove, 1, 1)
         w.layout().addWidget(self.spaceBelow, 2, 1)
-        l = QLabel(i18n("Space above:"))
-        l.setBuddy(self.spaceAbove)
-        w.layout().addWidget(l, 1, 0, Qt.AlignRight)
         l = QLabel(i18n("Space below:"))
         l.setBuddy(self.spaceBelow)
         w.layout().addWidget(l, 2, 0, Qt.AlignRight)
         return w
         
     def clefChanged(self, index):
-        self.setIcon(0, self.staff.symbolIcon('clef_%s' % (self.clef.clef() or 'none')))
+        self.setIcon(0, self.staff.symbolIcon('clef_' + (self.clef.clef() or 'none')))
 
     def staffCount(self):
         return 1
     
     def music(self, layout):
         clef = self.clef.clef()
-        if clef == 'tab':
-            staff = 'TabStaff'
-        else:
-            staff = 'Staff'
+        staff = 'TabStaff' if clef == 'tab' else 'Staff'
         layout.addStaffContext(staff)
-        music = ['\\new %s \\with {' % staff]
-        music.append(
-            "\\override VerticalAxisGroup #'minimum-Y-extent = #'(-%d . %d)" %
-            (self.spaceBelow.value(), self.spaceAbove.value()))
+        music = ['\\new {0} \\with {{'.format(staff)]
+        if lilyPondVersion() < (2, 13, 4):
+            music.append(
+                "\\override VerticalAxisGroup #'minimum-Y-extent = #'(-{0} . 5)".format(
+                    self.spaceBelow.value()))
+        else:
+            music.append(
+                "\\override VerticalAxisGroup #'next-staff-spacing = #'((space . {0}))".format(
+                    self.spaceBelow.value() + 5))
         if not clef:
             music.append('\\remove "Clef_engraver"')
         if not clef or clef == 'tab':
             music.append('} { \\music }')
         else:
-            music.append('} { \\clef "%s" \\music }' % clef)
+            music.append('}} {{ \\clef "{0}" \\music }}'.format(clef))
         return music
 
 
@@ -927,9 +934,9 @@ class BracketItem(Item):
         if self.squareBracket.isChecked():
             withMusic.append("systemStartDelimiter = #'SystemStartSquare")
         if withMusic:
-            music = ['\\new %s \\with {' % staff] + withMusic + ['} <<']
+            music = ['\\new {0} \\with {{'.format(staff)] + withMusic + ['} <<']
         else:
-            music = ['\\new %s <<' % staff]
+            music = ['\\new {0} <<'.format(staff)]
         music.extend(self.childMusic(layout))
         music.append('>>')
         return music
