@@ -17,19 +17,22 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 LilyPond auto completion
 """
 
 import re
 
-from PyQt4.QtCore import QModelIndex, Qt, QVariant
+from PyQt4.QtCore import QModelIndex, Qt
 from PyQt4.QtGui import QBrush, QColor, QTextFormat
 from PyKDE4.kdecore import KGlobal
 from PyKDE4.ktexteditor import KTextEditor
 
-import ly, ly.font, ly.tokenize, ly.version, ly.words, ly.colors
+import ly.tokenize, ly.version, ly.words, ly.colors
 import frescobaldi_app.version
+from frescobaldi_app.mainapp import lilyPondCommand
 
 
 class CompletionHelper(object):
@@ -39,18 +42,13 @@ class CompletionHelper(object):
     # roles on the Name column
     roles = {
         KTextEditor.CodeCompletionModel.CompletionRole:
-            QVariant(
-                KTextEditor.CodeCompletionModel.FirstProperty |
-                KTextEditor.CodeCompletionModel.Public |
-                KTextEditor.CodeCompletionModel.LastProperty ),
-        KTextEditor.CodeCompletionModel.ScopeIndex:
-            QVariant(0),
-        KTextEditor.CodeCompletionModel.MatchQuality:
-            QVariant(10),
-        KTextEditor.CodeCompletionModel.HighlightingMethod:
-            QVariant(QVariant.Invalid),
-        KTextEditor.CodeCompletionModel.InheritanceDepth:
-            QVariant(0),
+            KTextEditor.CodeCompletionModel.FirstProperty |
+            KTextEditor.CodeCompletionModel.Public |
+            KTextEditor.CodeCompletionModel.LastProperty,
+        KTextEditor.CodeCompletionModel.ScopeIndex: 0,
+        KTextEditor.CodeCompletionModel.MatchQuality: 10,
+        KTextEditor.CodeCompletionModel.HighlightingMethod: None,
+        KTextEditor.CodeCompletionModel.InheritanceDepth: 0,
     }
     
     def __init__(self, model, resultList=None):
@@ -77,12 +75,11 @@ class CompletionHelper(object):
     def data(self, index, role):
         if index.column() == KTextEditor.CodeCompletionModel.Name:
             if role == Qt.DisplayRole:
-                return QVariant(self.resultList[index.row()])
+                return self.resultList[index.row()]
             try:
                 return self.roles[role]
             except KeyError:
                 pass
-        return QVariant()
     
     def executeCompletionItem(self, doc, word, row):
         pass
@@ -106,7 +103,7 @@ class VarCompletions(CompletionHelper):
     """
     def executeCompletionItem(self, doc, word, row):
         text = self.resultList[row]
-        line = unicode(doc.line(word.end().line()))[word.end().column():]
+        line = doc.line(word.end().line())[word.end().column():]
         if not line.lstrip().startswith('='):
             text += ' = '
         doc.replaceText(word, text)
@@ -120,19 +117,19 @@ class ColorCompletions(CompletionHelper):
     roles = CompletionHelper.roles.copy()
     roles.update({
         KTextEditor.CodeCompletionModel.HighlightingMethod:
-            QVariant(KTextEditor.CodeCompletionModel.CustomHighlighting)
+            KTextEditor.CodeCompletionModel.CustomHighlighting
     })
     
     def data(self, index, role):
         if index.column() == KTextEditor.CodeCompletionModel.Name:
             name, (r, g, b) = self.resultList[index.row()]
             if role == Qt.DisplayRole:
-                return QVariant(name)
+                return name
             elif role == KTextEditor.CodeCompletionModel.CustomHighlight:
                 format = QTextFormat()
                 color = QColor.fromRgbF(r, g, b)
                 format.setBackground(QBrush(color))
-                return QVariant([0, len(name), format])
+                return [0, len(name), format]
         return super(ColorCompletions, self).data(index, role)
 
 
@@ -144,7 +141,7 @@ class ExpansionCompletions(CompletionHelper):
         self.mgr = model.doc.app.mainwin.expandManager()
         self.expansions = self.mgr.expansionsList()
         descriptions = [self.mgr.description(name) for name in self.expansions]
-        result = ['%s (%s)' % r for r in zip(self.expansions, descriptions)]
+        result = ['{0} ({1})'.format(e, d) for e, d in zip(self.expansions, descriptions)]
         super(ExpansionCompletions, self).__init__(model, result)
 
     def executeCompletionItem(self, doc, word, row):
@@ -171,7 +168,8 @@ def findMatches(model, view, word, invocationType):
     """
     doc = view.document()
     line, col = word.start().line(), word.start().column()
-    text = unicode(doc.line(line))[:col]
+    text = doc.line(line)
+    text = text[:col] if text else "" # cause KatePart returns None i.s.o. ""
     
     # determine what the user tries to type
     # very specific situations:
@@ -181,6 +179,8 @@ def findMatches(model, view, word, invocationType):
         return ly.words.midi_instruments
     if re.search(r'\\musicglyph\s*#"$', text):
         return musicglyph_names()
+    if re.search(r'\bmarkFormatter\s*=\s*#$', text):
+        return ly.words.mark_formatters
     if re.search(r'\\key\s+[a-z]+\s*\\$', text):
         return ly.words.modes
     if re.search(r'\\(un)?set\b\s*$', text):
@@ -212,8 +212,8 @@ def findMatches(model, view, word, invocationType):
     if re.search(r"#'break-visibility\s*=\s*#$", text):
         return ly.words.break_visibility
     # parse to get current context
-    fragment = unicode(doc.text(KTextEditor.Range(
-        KTextEditor.Cursor(0, 0), word.start())))
+    fragment = doc.text(KTextEditor.Range(KTextEditor.Cursor(0, 0),
+        word.start())) or "" # KatePart returns None instead of empty string
     tokenizer = ly.tokenize.Tokenizer()
     token = None # in case the next loop does not run at all
     for token in tokenizer.tokens(fragment):
@@ -230,7 +230,7 @@ def findMatches(model, view, word, invocationType):
                 return ly.words.markupcommands
         commands = (ly.words.keywords + ly.words.keywords_completion
             + ly.words.musiccommands + ly.words.musiccommands_completion
-            + lilypondVersion())
+            + lilyPondVersion())
         if tokenizer.parser().token == "\\context":
             return commands + ly.words.contexts
         else:
@@ -262,29 +262,19 @@ def findMatches(model, view, word, invocationType):
         if tokenizer.parser().token in ("\\context", "\\with"):
             return VarCompletions(model, ly.words.contextproperties)
     
-    if tokenizer.depth() <= (1, 0) or not text.strip():
-         # only on toplevel or on empty line
+    if not text.strip():
+         # only on an empty line: show the expansions
          return ExpansionCompletions(model)
 
 
-# lazy-load and cache some data
-@ly.lazy
+# load some (cached) data
 def musicglyph_names():
-    datadir = ly.version.datadir(command("lilypond"))
-    if datadir:
-        font = ly.font.emmentaler20(datadir)
-        if font:
-            return tuple(font.glyphs())
+    font = ly.version.LilyPondInstance(lilyPondCommand()).fontInfo("emmentaler-20")
+    if font:
+        return tuple(font.glyphs())
     return ()
 
-@ly.lazy
-def lilypondVersion():
+def lilyPondVersion():
     ver = frescobaldi_app.version.defaultVersion()
-    return ver and ('version "%s"' % ver,) or ()
+    return ('version "{0}"'.format(ver),) if ver else ()
 
-# utility functions
-def config(group):
-    return KGlobal.config().group(group)
-
-def command(cmd):
-    return unicode(config("commands").readEntry(cmd, QVariant(cmd)).toString())
