@@ -17,12 +17,14 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 Advanced manipulations on LilyPond documents.
 """
 
 import os, re, weakref
-from rational import Rational
+from fractions import Fraction
 
 from PyQt4 import QtCore, QtGui
 
@@ -31,8 +33,10 @@ from PyKDE4.kdeui import KDialog, KIcon, KMessageBox
 from PyKDE4.ktexteditor import KTextEditor
 
 import ly.rx, ly.pitch, ly.parse, ly.tokenize, ly.tools, ly.version
-from kateshell.app import lazymethod
-from frescobaldi_app.widgets import promptText
+from kateshell.app import cacheresult
+from kateshell.widgets import promptText
+from kateshell.mainwindow import addAccelerators
+from frescobaldi_app.mainapp import lilyPondCommand
 
 
 class DocumentManipulator(object):
@@ -51,8 +55,8 @@ class DocumentManipulator(object):
             a.setCheckable(True)
             if lang == currentLang:
                 a.setChecked(True)
-            QtCore.QObject.connect(a, QtCore.SIGNAL("triggered()"),
-                lambda lang=lang: self.changeLanguage(lang))
+            a.triggered.connect((lambda lang: lambda: self.changeLanguage(lang))(lang))
+        addAccelerators(menu.actions())
     
     def changeLanguage(self, lang):
         """
@@ -70,17 +74,16 @@ class DocumentManipulator(object):
             return
         
         # Apply the changes.
-        self.doc.doc.startEditing()
-        changes.applyToCursor(EditCursor(self.doc.doc))
-        if not start and not includeCommandChanged:
-            self.addLineToTop('\\include "%s.ly"' % lang)
-        self.doc.doc.endEditing()
+        with self.doc.editContext():
+            changes.applyToCursor(EditCursor(self.doc.doc))
+            if not start and not includeCommandChanged:
+                self.addLineToTop('\\include "{0}.ly"'.format(lang))
         if start and not includeCommandChanged:
             KMessageBox.information(self.doc.app.mainwin,
-                '<p>%s</p><p><tt>\\include "%s.ly"</tt></p>' %
-                (i18n("The pitch language of the selected text has been "
-                        "updated, but you need to manually add the following "
-                        "command to your document:"), lang),
+                '<p>{0}</p><p><tt>\\include "{1}.ly"</tt></p>'.format(
+                i18n("The pitch language of the selected text has been "
+                     "updated, but you need to manually add the following "
+                     "command to your document:"), lang),
                 i18n("Pitch Name Language"))
 
     def addLineToTop(self, text):
@@ -163,10 +166,10 @@ class DocumentManipulator(object):
         
         text = self.doc.selectionText().strip()
         if '\n' in text:
-            result = "%s =%s {\n%s\n}\n" % (name, mode, text)
+            result = "{0} ={1} {{\n{2}\n}}\n".format(name, mode, text)
             result = self.doc.indent(result)
         else:
-            result = "%s =%s { %s }\n" % (name, mode, text)
+            result = "{0} ={1} {{ {2} }}\n".format(name, mode, text)
             
         if not isblank(self.doc.line(insertLine)):
             result += '\n'
@@ -174,17 +177,16 @@ class DocumentManipulator(object):
             result = '\n' + result
         
         # add space if necessary
-        variable = "\\%s" % name
+        variable = "\\" + name
         end = selRange.end()
         if not isblank(self.doc.line(end.line())[end.column():end.column()+1]):
             variable += " "
 
         # do it:
         cursor = KTextEditor.Cursor(insertLine, 0)
-        self.doc.doc.startEditing()
-        self.doc.replaceSelectionWith(variable, keepSelection=False)
-        self.doc.doc.insertText(cursor, result)
-        self.doc.doc.endEditing()
+        with self.doc.editContext():
+            self.doc.replaceSelectionWith(variable, keepSelection=False)
+            self.doc.doc.insertText(cursor, result)
         
     def repeatLastExpression(self):
         """
@@ -196,8 +198,8 @@ class DocumentManipulator(object):
         while lineNum > 0 and isblank(self.doc.line(lineNum)):
             lineNum -= 1
             
-        text = unicode(self.doc.doc.text(
-            KTextEditor.Range(KTextEditor.Cursor(lineNum, 0), curPos)))
+        text = self.doc.doc.text(
+            KTextEditor.Range(KTextEditor.Cursor(lineNum, 0), curPos))
         matchObj = None
         for m in ly.rx.chord.finditer(text):
             if m.group('chord'):
@@ -240,7 +242,7 @@ class DocumentManipulator(object):
         col = curPos.column()
         if col > 0 and not isblank(self.doc.line()[col-1]):
             result = " " + result
-        self.doc.view.insertText(result + " ")
+        self.doc.view.insertText(result)
     
     def selectLines(self):
         """
@@ -344,7 +346,7 @@ class DocumentManipulator(object):
         # adjust end:
         text = self.doc.line(end.line())
         col = end.column()
-        if re.match("%s|%s" % (ly.rx.step, ly.rx.rest), text[col:]):
+        if re.match(ly.rx.step + "|" + ly.rx.rest, text[col:]):
             for m in ly.rx.chord_rest.finditer(text):
                 if (m.group('chord')
                     and m.start('chord') <= col <= m.end('chord')):
@@ -385,13 +387,12 @@ class DocumentManipulator(object):
         
         # We don't just replace the text, because that would destroy smart
         # point and click. We only replace the indents.
-        self.doc.doc.startEditing()
-        for old, new in zip(oldindents, newindents):
-            if old != new:
-                self.doc.doc.replaceText(
-                    KTextEditor.Range(startline, 0, startline, len(old)), new)
-            startline += 1
-        self.doc.doc.endEditing()
+        with self.doc.editContext():
+            for old, new in zip(oldindents, newindents):
+                if old != new:
+                    self.doc.doc.replaceText(
+                        KTextEditor.Range(startline, 0, startline, len(old)), new)
+                startline += 1
         self.doc.view.removeSelection()
 
     def populateContextMenu(self, menu):
@@ -450,8 +451,7 @@ class DocumentManipulator(object):
             if m.start() <= col <= m.end():
                 fileName = m.group(1)
                 a = menu.addAction(KIcon("document-open"), i18n("Open %1", fileName))
-                QtCore.QObject.connect(a, QtCore.SIGNAL("triggered()"),
-                    lambda: self.openIncludeFile(fileName))
+                a.triggered.connect(lambda: self.openIncludeFile(fileName))
                 return
         
         # Rhythm submenu
@@ -478,8 +478,11 @@ class DocumentManipulator(object):
         
         # Hyphenate Lyrics
         if selection and isinstance(tokenizer.parser(), tokenizer.LyricModeParser):
+            menu.addSeparator()
             menu.addAction(
                 self.doc.app.mainwin.actionCollection().action("lyrics_hyphen"))
+            menu.addAction(
+                self.doc.app.mainwin.actionCollection().action("lyrics_copy_dehyphen"))
     
     def openIncludeFile(self, fileName):
         """
@@ -494,7 +497,7 @@ class DocumentManipulator(object):
             localdir = self.doc.app.defaultDirectory() or os.getcwd()
         url = os.path.normpath(os.path.join(localdir, fileName))
         if not os.path.exists(url):
-            datadir = ly.version.datadir(lilypondCommand())
+            datadir = ly.version.LilyPondInstance(lilyPondCommand()).datadir()
             if datadir and os.path.exists(os.path.join(datadir, 'ly', fileName)):
                 url = os.path.join(datadir, 'ly', fileName)
         self.doc.app.openUrl(url).setActive()
@@ -507,24 +510,24 @@ class DocumentManipulator(object):
         """
         selection = self.doc.selectionText()
         if selection:
-            repl = double and u'\u201C%s\u201D' or u'\u2018%s\u2019'
-            self.doc.replaceSelectionWith(repl % selection, keepSelection=False)
+            repl = double and '\u201C{0}\u201D' or '\u2018{0}\u2019'
+            self.doc.replaceSelectionWith(repl.format(selection), keepSelection=False)
         else:
             cursor = self.doc.view.cursorPosition()
             line, col = cursor.line(), cursor.column()
             right = col > 0 and self.doc.line(line)[col-1] not in '" \t'
             self.doc.view.insertText({
-                (False, False): u'\u2018',     # LEFT SINGLE QUOTATION MARK
-                (False, True ): u'\u2019',     # RIGHT SINGLE QUOTATION MARK
-                (True,  False): u'\u201C',     # LEFT DOUBLE QUOTATION MARK
-                (True,  True ): u'\u201D',     # RIGHT DOUBLE QUOTATION MARK
+                (False, False): '\u2018',     # LEFT SINGLE QUOTATION MARK
+                (False, True ): '\u2019',     # RIGHT SINGLE QUOTATION MARK
+                (True,  False): '\u201C',     # LEFT DOUBLE QUOTATION MARK
+                (True,  True ): '\u201D',     # RIGHT DOUBLE QUOTATION MARK
                 }[(double, right)])
 
     def insertBarLine(self, bar):
         """
         Insert a \\bar ".." command with the given type.
         """
-        self.doc.view.insertText('\\bar "%s"' % bar)
+        self.doc.view.insertText('\\bar "{0}"'.format(bar))
         
     def insertTemplate(self, text, cursor=None, remove=None, doIndent=True):
         """
@@ -579,10 +582,9 @@ class DocumentManipulator(object):
                     cur.walk(text[pos:m.end('full')])
                     pos = m.end('full')
                     insertions.append(cur.kteCursor())
-            self.doc.doc.startEditing()
-            for i in reversed(insertions):
-                self.doc.doc.insertText(i, art)
-            self.doc.doc.endEditing()
+            with self.doc.editContext():
+                for i in reversed(insertions):
+                    self.doc.doc.insertText(i, art)
             self.doc.view.removeSelection()
         else:
             self.adjustCursorToChords()
@@ -601,13 +603,13 @@ class DocumentManipulator(object):
         space1, sel, space2 = re.compile(
             r'^(\s*)(.*?)(\s*)$', re.DOTALL).match(text).groups()
         if alwaysMultiLine or '\n' in text:
-            result = "%s\n%s\n%s" % (before, sel, after)
+            result = "{0}\n{1}\n{2}".format(before, sel, after)
             # indent the result corresponding with the first selection line.
             selRange = self.doc.view.selectionRange()
             indentDepth = self.doc.currentIndent(selRange.start(), False)
             result = self.doc.indent(result, indentDepth).lstrip()
         else:
-            result = "%s %s %s" % (before, sel, after)
+            result = "{0} {1} {2}".format(before, sel, after)
         # re-add the space at start and end of selection
         return ''.join((space1, result, space2))
         
@@ -622,30 +624,29 @@ class DocumentManipulator(object):
         if selRange.start().position() == (0, 0):
             return
         text = self.doc.selectionText()
-        self.doc.doc.startEditing()
-        atStart = cursor.position() == selRange.start().position()
-        # Determine current depth (we could be in a long \book block)
-        tokenizer = ly.tokenize.Tokenizer()
-        for token in tokenizer.tokens(self.doc.textToCursor(selRange.start())):
-            pass
-        self.doc.doc.removeText(selRange)
-        insert = KTextEditor.Cursor(0, 0)
-        for r in reversed(list(self.findBlankLines(tokenizer.depth()))):
-            if r.end().position() < selRange.start().position():
-                insert.setLine(r.end().line())
-                break
-        cursor = Cursor(insert)
-        if not text.endswith('\n'):
-            text += '\n'
-        cursor.walk(text)
-        self.doc.doc.insertText(insert, text)
+        with self.doc.editContext():
+            atStart = cursor.position() == selRange.start().position()
+            # Determine current depth (we could be in a long \book block)
+            tokenizer = ly.tokenize.Tokenizer()
+            for token in tokenizer.tokens(self.doc.textToCursor(selRange.start())):
+                pass
+            self.doc.doc.removeText(selRange)
+            insert = KTextEditor.Cursor(0, 0)
+            for r in reversed(list(self.findBlankLines(tokenizer.depth()))):
+                if r.end().position() < selRange.start().position():
+                    insert.setLine(r.end().line())
+                    break
+            cursor = Cursor(insert)
+            if not text.endswith('\n'):
+                text += '\n'
+            cursor.walk(text)
+            self.doc.doc.insertText(insert, text)
         selRange = KTextEditor.Range(insert, cursor.kteCursor())
         self.doc.view.setSelection(selRange)
         if atStart:
             self.doc.view.setCursorPosition(selRange.start())
         else:
             self.doc.view.setCursorPosition(selRange.end())
-        self.doc.doc.endEditing()
     
     def moveSelectionDown(self):
         """
@@ -659,34 +660,33 @@ class DocumentManipulator(object):
         if selRange.end().position() == docRange.end().position():
             return
         text = self.doc.selectionText()
-        self.doc.doc.startEditing()
-        atStart = cursor.position() == selRange.start().position()
-        # Determine current depth (we could be in a long \book block)
-        tokenizer = ly.tokenize.Tokenizer()
-        for token in tokenizer.tokens(self.doc.textToCursor(selRange.start())):
-            pass
-        self.doc.doc.removeText(selRange)
-        for r in self.findBlankLines(tokenizer.depth()):
-            if r.start().position() > selRange.start().position():
-                insert = KTextEditor.Cursor(r.end().line(), 0)
-                break
-        else:
-            docRange = self.doc.doc.documentRange()
-            self.doc.doc.insertText(docRange.end(), '\n')
-            docRange = self.doc.doc.documentRange()
-            insert = docRange.end()
-        cursor = Cursor(insert)
-        if not text.endswith('\n'):
-            text += '\n'
-        cursor.walk(text)
-        self.doc.doc.insertText(insert, text)
+        with self.doc.editContext():
+            atStart = cursor.position() == selRange.start().position()
+            # Determine current depth (we could be in a long \book block)
+            tokenizer = ly.tokenize.Tokenizer()
+            for token in tokenizer.tokens(self.doc.textToCursor(selRange.start())):
+                pass
+            self.doc.doc.removeText(selRange)
+            for r in self.findBlankLines(tokenizer.depth()):
+                if r.start().position() > selRange.start().position():
+                    insert = KTextEditor.Cursor(r.end().line(), 0)
+                    break
+            else:
+                docRange = self.doc.doc.documentRange()
+                self.doc.doc.insertText(docRange.end(), '\n')
+                docRange = self.doc.doc.documentRange()
+                insert = docRange.end()
+            cursor = Cursor(insert)
+            if not text.endswith('\n'):
+                text += '\n'
+            cursor.walk(text)
+            self.doc.doc.insertText(insert, text)
         selRange = KTextEditor.Range(insert, cursor.kteCursor())
         self.doc.view.setSelection(selRange)
         if atStart:
             self.doc.view.setCursorPosition(selRange.start())
         else:
             self.doc.view.setCursorPosition(selRange.end())
-        self.doc.doc.endEditing()
 
     def convertRelativeToAbsolute(self):
         """
@@ -737,7 +737,7 @@ class DocumentManipulator(object):
                 "that are not available in the pitch language \"%1\".",
                 language))
         
-    @lazymethod
+    @cacheresult
     def transposeDialog(self):
         return TransposeDialog(self.doc.view)
 
@@ -778,7 +778,7 @@ class TransposeDialog(KDialog):
             self.fromPitch.clear()
             for octave in (",", "", "'"):
                 for note in range(7):
-                    for alter in Rational(-1, 2), 0, Rational(1, 2):
+                    for alter in Fraction(-1, 2), 0, Fraction(1, 2):
                         self.fromPitch.insertItem(0,
                             ly.pitch.pitchWriter[language](note, alter) + octave)
             fromIndex != -1 and self.fromPitch.setCurrentIndex(fromIndex)
@@ -802,7 +802,7 @@ class TransposeDialog(KDialog):
         return KDialog.exec_(self)
     
     def pitchFrom(self, combobox):
-        t = unicode(combobox.currentText())
+        t = combobox.currentText()
         p = ly.pitch.Pitch()
         p.octave = t.count("'") - t.count(",")
         result = ly.pitch.pitchReader[self.language](
@@ -881,15 +881,17 @@ class EditCursor(ly.tokenize.Cursor):
     """
     Translates changes to a Python string in a ly.tokenize.ChangeList
     to changes to a KTextEditor.Document and applies them.
+    Can be used as a context manager, in which case it folds all edits
+    in one undo action.
     """
     def __init__(self, doc):
         super(EditCursor, self).__init__()
         self.doc = doc
     
-    def startEditing(self):
+    def __enter__(self):
         self.doc.startEditing()
         
-    def endEditing(self):
+    def __exit__(self, *args):
         self.doc.endEditing()
         
     def insertText(self, text):
@@ -908,8 +910,3 @@ class EditCursor(ly.tokenize.Cursor):
 def isblank(text):
     """ Returns True if text is None, empty or only contains spaces. """
     return not text or text.isspace()
-
-def lilypondCommand():
-    """ Returns the user's preferred lilypond command. """
-    return unicode(KGlobal.config().group("commands").readEntry("lilypond",
-        QtCore.QVariant("lilypond")).toString())
