@@ -17,41 +17,43 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 Expand Manager, manages expansions.
 """
 import re
 
-from PyQt4.QtCore import QObject, QString, QTimer, Qt, QVariant, SIGNAL
+from PyQt4.QtCore import Qt
 from PyQt4.QtGui import (
-    QFont, QKeySequence, QSplitter, QTextEdit, QTreeWidget, QTreeWidgetItem,
-    QVBoxLayout)
+    QFont, QSplitter, QTextEdit, QTreeWidget, QTreeWidgetItem, QVBoxLayout)
 
 from PyKDE4.kdecore import KConfig, KGlobal, i18n
 from PyKDE4.kdeui import (
-    KDialog, KKeySequenceWidget, KMessageBox, KShortcut, KStandardGuiItem,
+    KDialog, KKeySequenceWidget, KMessageBox, KStandardGuiItem,
     KTreeWidgetSearchLine, KVBox)
 from PyKDE4.ktexteditor import KTextEditor
 
 import ly.parse, ly.pitch
 
-from kateshell.app import lazymethod
+from kateshell.app import cacheresult
+from kateshell.shortcut import ShortcutClient
 from frescobaldi_app.highlight import LilyPondHighlighter
 
 
-class ExpandManager(object):
+class ExpandManager(ShortcutClient):
     def __init__(self, mainwin):
         self.mainwin = mainwin
-        self.shortcuts = mainwin.expansionShortcuts
+        ShortcutClient.__init__(self, mainwin.expansionShortcuts)
         self.expansions = KConfig("expansions", KConfig.NoGlobals, "appdata")
         # delete shortcut actions that do not exist here anymore
-        self.shortcuts.shakeHands(self.expansionsList())
+        self.shakeHands(self.expansionsList())
         
     def actionTriggered(self, name):
         return self.doExpand(name)
         
-    def populateAction(self, action):
-        action.setText(self.description(action.objectName()))
+    def populateAction(self, name, action):
+        action.setText(self.description(name))
         
     def expand(self):
         """
@@ -75,7 +77,7 @@ class ExpandManager(object):
         # open dialog and let the user choose
         self.expansionDialog().show()
         
-    @lazymethod
+    @cacheresult
     def expansionDialog(self):
         return ExpansionDialog(self)
 
@@ -94,7 +96,7 @@ class ExpandManager(object):
         """
         Return the description for the expansion name.
         """
-        return unicode(self.expansions.group(name).readEntry("Name", QVariant("")).toString())
+        return self.expansions.group(name).readEntry("Name", "")
     
     def doExpand(self, expansion, remove=None):
         """
@@ -104,7 +106,7 @@ class ExpandManager(object):
         doc = self.mainwin.currentDocument()
         
         group = self.expansions.group(expansion)
-        text = unicode(group.readEntry("Text", QVariant("")).toString())
+        text = group.readEntry("Text", "")
         
         # where to insert the text:
         cursor = remove and remove.start() or doc.view.cursorPosition()
@@ -150,7 +152,7 @@ class ExpansionDialog(KDialog):
             KDialog.Ok | KDialog.Close | KDialog.User1 | KDialog.User2 ))
         self.setButtonGuiItem(KDialog.User1, KStandardGuiItem.remove())
         self.setButtonGuiItem(KDialog.User2, KStandardGuiItem.add())
-        QObject.connect(self, SIGNAL("closeClicked()"), self.reject)
+        self.closeClicked.connect(self.reject)
         self.setDefaultButton(KDialog.Ok)
         self.setHelp("expand")
         
@@ -203,15 +205,14 @@ class ExpansionDialog(KDialog):
             
         edit.setWhatsThis(
             "<html><head><style type='text/css'>"
-            "td.short { font-family: monospace; font-weight: bold; }"
+            "td.short {{ font-family: monospace; font-weight: bold; }}"
             "</style></head><body>"
-            "<p>%s</p><table border=0 width=300 cellspacing=2><tbody>"
-            "<tr><td class=short align=center>(|)</td><td>%s</td></tr>"
-            "<tr><td class=short align=center>@</td><td>%s</td></tr>"
-            "</tbody></table></body></html>"
-            % (i18n(
-                "This is the text associated with the selected shortcut. "
-                "Some characters have special meaning:"),
+            "<p>{0}</p><table border=0 width=300 cellspacing=2><tbody>"
+            "<tr><td class=short align=center>(|)</td><td>{1}</td></tr>"
+            "<tr><td class=short align=center>@</td><td>{2}</td></tr>"
+            "</tbody></table></body></html>".format(
+            i18n("This is the text associated with the selected shortcut. "
+                 "Some characters have special meaning:"),
             i18n("Place the cursor on this spot."),
             i18n("Translate the following pitch."),
             ))
@@ -225,32 +226,28 @@ class ExpansionDialog(KDialog):
         
         # load the expansions
         for name in sorted(self.manager.expansionsList()):
-            self.createItem(name, self.manager.description(name),
-            self.manager.shortcuts.shortcut(name))
+            self.createItem(name, self.manager.description(name))
 
         tree.sortByColumn(1, Qt.AscendingOrder)
         tree.setSortingEnabled(True)
         tree.resizeColumnToContents(1)
         
-        QObject.connect(self, SIGNAL("user1Clicked()"), self.removeItem)
-        QObject.connect(self, SIGNAL("user2Clicked()"), self.addItem)
-        QObject.connect(edit, SIGNAL("textChanged()"), self.editChanged)
-        QObject.connect(search, SIGNAL("textChanged(QString)"), self.checkMatch)
-        QObject.connect(tree, SIGNAL("itemSelectionChanged()"), self.updateSelection)
-        QObject.connect(tree, SIGNAL("itemChanged(QTreeWidgetItem*, int)"),
-            self.itemChanged, Qt.QueuedConnection)
-        QObject.connect(key, SIGNAL("keySequenceChanged (QKeySequence)"),
-            self.keySequenceChanged)
+        self.user1Clicked.connect(self.removeItem)
+        self.user2Clicked.connect(self.addItem)
+        edit.textChanged.connect(self.editChanged)
+        search.textChanged.connect(self.checkMatch)
+        tree.itemSelectionChanged.connect(self.updateSelection)
+        tree.itemChanged.connect(self.itemChanged, Qt.QueuedConnection)
+        key.keySequenceChanged.connect(self.keySequenceChanged)
     
-    def createItem(self, name, description, key=None):
-        """ Create a new item, if key is given it should be a KShortcut. """
+    def createItem(self, name, description):
+        """ Create a new item. """
         item = QTreeWidgetItem(self.treeWidget)
         item.groupName = name
         item.setFont(0, QFont("monospace"))
         item.setText(0, name)
         item.setText(1, description)
-        if key:
-            item.setText(2, key.toList()[0].toString())
+        item.setText(2, self.manager.shortcutText(name))
         item.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEditable | Qt.ItemIsEnabled)
         return item
     
@@ -264,11 +261,11 @@ class ExpansionDialog(KDialog):
         name = "new"
         while self.manager.expansionExists(name):
             num += 1
-            name = "new%d" % num
+            name = "new{0}".format(num)
         description = i18n("New Item")
         if num:
-            description += " %d" % num
-        self.manager.expansions.group(name).writeEntry("Name", QVariant(unicode(description)))
+            description += " {0}".format(num)
+        self.manager.expansions.group(name).writeEntry("Name", description)
         self.searchLine.clear() # otherwise strange things happen...
         item = self.createItem(name, description)
         if self.edit.item is None:
@@ -290,7 +287,7 @@ class ExpansionDialog(KDialog):
             index = self.treeWidget.indexOfTopLevelItem(item)
             setIndex = index + 1 < self.treeWidget.topLevelItemCount()
             self.manager.expansions.deleteGroup(item.groupName)
-            self.manager.shortcuts.removeShortcut(item.groupName)
+            self.manager.removeShortcut(item.groupName)
             self.treeWidget.takeTopLevelItem(index)
             if setIndex:
                 self.setCurrentItem(self.treeWidget.topLevelItem(index))
@@ -326,12 +323,12 @@ class ExpansionDialog(KDialog):
         if items:
             name = items[0].text(0)
             group = self.manager.expansions.group(name)
-            self.edit.setPlainText(group.readEntry("Text", QVariant("")).toString())
+            self.edit.setPlainText(group.readEntry("Text", ""))
             self.edit.item = items[0]
             self.edit.dirty = False
             # key shortcut widget
             self.key.setEnabled(True)
-            self.loadShortcut(name)
+            self.manager.keyLoadShortcut(self.key, name)
         else:
             self.edit.item = None
             self.edit.clear()
@@ -349,7 +346,7 @@ class ExpansionDialog(KDialog):
                     "Please use a different name."))
                 item.setText(0, item.groupName)
                 self.treeWidget.editItem(item, 0)
-            elif not re.match(r"\w+$", unicode(item.text(0))):
+            elif not re.match(r"\w+$", item.text(0)):
                 KMessageBox.error(self.manager.mainwin, i18n(
                     "Please only use letters, numbers and the underscore "
                     "character in the expansion name."))
@@ -359,32 +356,29 @@ class ExpansionDialog(KDialog):
                 # apply the changed mnemonic
                 old, new = item.groupName, item.text(0)
                 group = self.manager.expansions.group(new)
-                group.writeEntry("Name", QVariant(unicode(item.text(1))))
-                group.writeEntry("Text", QVariant(unicode(self.manager.expansions.group(
-                    old).readEntry("Text", QVariant("")).toString())))
+                group.writeEntry("Name", item.text(1))
+                group.writeEntry("Text", self.manager.expansions.group(old).readEntry("Text", ""))
                 self.manager.expansions.deleteGroup(old)
                 # move the shortcut
-                s = self.manager.shortcuts
-                if s.shortcut(old):
-                    s.setShortcut(new, s.shortcut(old))
-                s.removeShortcut(old)
+                if self.manager.shortcut(old):
+                    self.manager.setShortcut(new, self.manager.shortcut(old))
+                self.manager.removeShortcut(old)
                 item.groupName = item.text(0)
                 self.treeWidget.scrollToItem(item)
         elif column == 1:
             group = self.manager.expansions.group(item.text(0))
             if item.text(1):
-                group.writeEntry("Name", QVariant(unicode(item.text(1))))
+                group.writeEntry("Name", item.text(1))
                 self.treeWidget.scrollToItem(item)
                 self.treeWidget.resizeColumnToContents(1)
             else:
                 KMessageBox.error(self.manager.mainwin, i18n(
                     "Please don't leave the description empty."))
-                item.setText(1, group.readEntry("Name", QVariant("")).toString())
+                item.setText(1, group.readEntry("Name", ""))
                 self.treeWidget.editItem(item, 1)
         elif column == 2:
             # User should not edit textual representation of shortcut
-            key = self.manager.shortcuts.shortcut(item.text(0))
-            item.setText(2, key and key.toList()[0].toString() or '')
+            item.setText(2, self.manager.shortcutText(item.text(0)))
     
     def editChanged(self):
         """ Marks our edit view as changed. """
@@ -394,47 +388,33 @@ class ExpansionDialog(KDialog):
         """ (Internal use) save the edit if it has changed. """
         if self.edit.dirty and self.edit.item:
             self.manager.expansions.group(self.edit.item.text(0)).writeEntry(
-                "Text", QVariant(unicode(self.edit.toPlainText())))
+                "Text", self.edit.toPlainText())
             self.edit.dirty = False
     
     def keySequenceChanged(self, seq):
         """ Called when the user has changed the keyboard shortcut. """
         item = self.currentItem()
         if item:
-            self.key.applyStealShortcut()
-            self.manager.shortcuts.setShortcut(
-                item.text(0), KShortcut(seq))
+            self.manager.keySaveShortcut(self.key, item.text(0), seq)
             item.setText(2, seq.toString())
             self.updateShortcuts()
         
-    def loadShortcut(self, name):
-        """ Sets the shortcut button to the shortcut for the given name. """
-        key = self.manager.shortcuts.shortcut(name)
-        block = self.key.blockSignals(True)
-        self.key.setKeySequence(key and key.toList()[0] or QKeySequence())
-        self.key.blockSignals(block)
-        
     def updateShortcuts(self):
         """
         Checks if shortcuts have disappeared by stealing them from other
         keyboard shortcut dialogs.  And initialize the shortcut button to
         check for collisions.
         """
-        names = self.manager.shortcuts.shortcuts()
+        names = self.manager.shortcuts()
         for item in self.items():
             if item.text(2) and item.text(0) not in names:
                 item.setText(2, '')
             elif item.text(0) in names:
-                key = self.manager.shortcuts.shortcut(item.text(0))
-                item.setText(2, key and key.toList()[0].toString() or '')
+                item.setText(2, self.manager.shortcutText(item.text(0)))
         item = self.currentItem()
         if item:
-            self.loadShortcut(item.text(0))
-        self.key.setCheckActionCollections([
-            self.manager.mainwin.actionCollection(),
-            self.manager.mainwin.view().actionCollection(),
-            self.manager.shortcuts.actionCollection(),
-            ])
+            self.manager.keyLoadShortcut(self.key, item.text(0))
+        self.manager.keySetCheckActionCollections(self.key)
     
     def show(self):
         self.updateShortcuts()
@@ -464,7 +444,7 @@ class ExpandHighlighter(LilyPondHighlighter):
         def repl(m):
             matches.append((m.start(), len(m.group())))
             return ' ' * len(m.group())
-        text = re.compile(r"\(\|\)|@").sub(repl, unicode(text))
+        text = re.compile(r"\(\|\)|@").sub(repl, text)
         super(ExpandHighlighter, self).highlightBlock(text)
         for start, count in matches:
             self.setFormat(start, count, self.formats['special'])
