@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 Basic LilyPond syntax highlighter for QTextEdit.
 """
@@ -65,7 +67,6 @@ class LilyPondHighlighter(QSyntaxHighlighter):
         self.state = []
 
     def highlightBlock(self, text):
-        text = unicode(text)
         tokenizer = ly.tokenize.Tokenizer()
         previous = self.previousBlockState()
         if 0 <= previous < len(self.state):
