@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 A browser tool for the LilyPond documentation.
 """
@@ -24,7 +26,7 @@ A browser tool for the LilyPond documentation.
 import glob, os, re, sip
 import HTMLParser
 
-from PyQt4.QtCore import QEvent, QObject, Qt, QUrl, QVariant, SIGNAL
+from PyQt4.QtCore import QEvent, QObject, Qt, QUrl, SIGNAL
 from PyQt4.QtGui import QGridLayout, QStackedWidget, QToolBar, QWidget
 from PyQt4.QtWebKit import QWebPage, QWebView
 
@@ -54,8 +56,8 @@ def docHomeUrl():
     Returns the configured or found url (as a string) where the LilyPond
     documentation is to be found.
     """
-    url = config().readEntry("lilypond documentation", QVariant('')).toString()
-    return unicode(url or findLocalDocIndex() or "http://lilypond.org/doc")
+    url = config().readEntry("lilypond documentation", "")
+    return url or findLocalDocIndex() or "http://lilypond.org/doc"
 
 def findLocalDocIndex():
     """
@@ -116,7 +118,7 @@ class LilyDoc(QWidget):
             ):
             self.linkActions[name] = a = self.toolBar.addAction(KIcon(icon), title)
             a.setEnabled(False)
-            QObject.connect(a, SIGNAL("triggered()"), lambda name=name: self.slotRellink(name))
+            a.triggered.connect((lambda name: lambda: self.slotRellink(name))(name))
         
         # search text entry
         self.search = KLineEdit()
@@ -125,27 +127,25 @@ class LilyDoc(QWidget):
         self.search.setClickMessage(i18n("Search..."))
         
         # signals
-        QObject.connect(self.back, SIGNAL("triggered()"), self.slotBack)
-        QObject.connect(self.forward, SIGNAL("triggered()"), self.slotForward)
-        QObject.connect(self.home, SIGNAL("triggered()"), self.slotHome)
-        QObject.connect(self.textLarger, SIGNAL("triggered()"), self.slotLarger)
-        QObject.connect(self.textSmaller, SIGNAL("triggered()"), self.slotSmaller)
+        self.back.triggered.connect(self.slotBack)
+        self.forward.triggered.connect(self.slotForward)
+        self.home.triggered.connect(self.slotHome)
+        self.textLarger.triggered.connect(self.slotLarger)
+        self.textSmaller.triggered.connect(self.slotSmaller)
         
-        QObject.connect(self.view, SIGNAL("urlChanged(QUrl)"), self.updateActions)
-        QObject.connect(self.view, SIGNAL("loadFinished(bool)"), self.slotLoadFinished)
+        self.view.urlChanged.connect(self.updateActions)
+        self.view.loadFinished.connect(self.slotLoadFinished)
         self.view.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
-        QObject.connect(self.view.page(), SIGNAL("linkClicked(QUrl)"), self.openUrl)
+        self.view.page().linkClicked.connect(self.openUrl)
         self.view.page().setForwardUnsupportedContent(True)
-        QObject.connect(self.view.page(), SIGNAL("unsupportedContent(QNetworkReply*)"),
-            self.slotUnsupported)
+        self.view.page().unsupportedContent.connect(self.slotUnsupported)
         
-        QObject.connect(self.search, SIGNAL("textEdited(QString)"), self.slotSearch)
-        QObject.connect(self.search, SIGNAL("returnPressed()"), self.slotSearch)
+        self.search.textEdited.connect(self.slotSearch)
+        self.search.returnPressed.connect(self.slotSearch)
         
         # context menu:
         self.view.setContextMenuPolicy(Qt.CustomContextMenu)
-        QObject.connect(self.view, SIGNAL("customContextMenuRequested(QPoint)"),
-            self.slotShowContextMenu)
+        self.view.customContextMenuRequested.connect(self.slotShowContextMenu)
         
         # load initial view.
         self.editFontSize = 0
@@ -175,7 +175,7 @@ class LilyDoc(QWidget):
             self.doc.setMode('LilyPond')
             self.doc.setEncoding('UTF-8')
             self.doc.setReadWrite(False)
-            QObject.connect(self.doc, SIGNAL("completed()"), self.slotCompleted)
+            self.doc.completed.connect(self.slotCompleted)
         self.edit = self.doc.createView(self)
         # remember the view font size
         if self.editFontSize != 0:
@@ -188,14 +188,14 @@ class LilyDoc(QWidget):
         # make backspace go out
         a = self.edit.actionCollection().action("backspace")
         if a:
-            QObject.connect(a, SIGNAL("triggered()"), self.slotBack)
+            a.triggered.connect(self.slotBack)
         
     def openUrl(self, url):
         # handle .ly urls and load them read-only in KatePart
         if self.edit:
             sip.delete(self.edit)
             self.edit = None
-        if url.path().endsWith('.ly'):
+        if url.path().endswith('.ly'):
             self.createEditView()
             self.stack.addWidget(self.edit)
             self.doc.openUrl(KUrl(url))
@@ -261,7 +261,7 @@ class LilyDoc(QWidget):
         # Called when the HTML doc has loaded.
         # Parse HTML and display link rel='' buttons
         if success:
-            html = unicode(self.view.page().mainFrame().toHtml())
+            html = self.view.page().mainFrame().toHtml()
             self.rellinks = RellinksParser(html, self.view.url()).links()
             self.updateLinkActions()
     
@@ -313,8 +313,7 @@ class LilyDoc(QWidget):
             menu.addAction(a)
             menu.addSeparator()
             a = menu.addAction(KIcon("window-new"), i18n("Open Link in &New Window"))
-            QObject.connect(a, SIGNAL("triggered()"),
-                lambda url=hit.linkUrl(): self.slotNewWindow(url))
+            a.triggered.connect((lambda url: lambda: self.slotNewWindow(url))(hit.linkUrl()))
         else:
             if hit.isContentSelected():
                 a = self.view.pageAction(QWebPage.Copy)
@@ -323,8 +322,7 @@ class LilyDoc(QWidget):
                 menu.addAction(a)
                 menu.addSeparator()
             a = menu.addAction(KIcon("window-new"), i18n("Open Document in &New Window"))
-            QObject.connect(a, SIGNAL("triggered()"),
-                lambda url=self.view.url(): self.slotNewWindow(url))
+            a.triggered.connect((lambda url: lambda: self.slotNewWindow(url))(self.view.url()))
         if len(menu.actions()):
             menu.exec_(self.view.mapToGlobal(pos))
     
@@ -492,7 +490,10 @@ class IndexParser(HtmlParser):
         self._tableTag = None
         self.items = {}
         self.initLine()
-        self.feed(html)
+        try:
+            self.feed(html)
+        except HTMLParser.HTMLParseError:
+            pass
     
     def initLine(self):
         self._anchors = []
@@ -539,14 +540,57 @@ class NotationReferenceIndexParser(IndexParser):
 class LearningManualIndexParser(IndexParser):
     indexClass = 'index-cp'
     
+class InternalsReferenceChapterParser(HtmlParser):
+    """Parses a chapter of the LilyPond Internals Reference.
+    
+    Makes lists of Contexts or Grobs, etc.
+    
+    """
+    def __init__(self, html):
+        HtmlParser.__init__(self)
+        self._tableTag = None
+        self._parsing = False
+        self._anchor = None
+        self._title = ""
+        self.items = {}
+        try:
+            self.feed(html)
+        except HTMLParser.HTMLParseError:
+            pass
+    
+    def handle_starttag(self, tag, attrs):
+        attrs = dict(attrs)
+        if not self._parsing:
+            if tag in ('ul', 'table') and attrs.get('class') == "menu":
+                self._tableTag = tag
+                self._parsing = True
+        elif tag == 'a' and 'href' in attrs:
+            self._anchor = attrs['href']
+            
+    def handle_data(self, data):
+        if self._anchor is not None:
+            self._title += data
+            
+    def handle_endtag(self, tag):
+        if not self._parsing:
+            return
+        elif tag == self._tableTag:
+            self._parsing = False
+        elif tag == 'a':
+            title = self._title.split()[-1]
+            self.items[title] = self._anchor
+            self._title = ""
+            self._anchor = None
+
 
 class HtmlLoader(object):
     """
     Tries to load an url using KIO and follows HTTP and HTML redirects.
     Sends the done(self) Python signal if done.
     """
+    done = Signal()
+    
     def __init__(self, url):
-        self.done = Signal()
         self._html = None
         self.startJob(url)
     
@@ -554,9 +598,9 @@ class HtmlLoader(object):
         self._url = KUrl(url)
         self._data = ''
         self._job = KIO.get(KUrl(url), KIO.NoReload, KIO.HideProgressInfo)
-        QObject.connect(self._job, SIGNAL("data(KIO::Job*, QByteArray)"), self.slotData)
-        QObject.connect(self._job, SIGNAL("redirection(KIO::Job*, KUrl)"), self.slotRedirection)
-        QObject.connect(self._job, SIGNAL("result(KJob*)"), self.slotResult)
+        self._job.data.connect(self.slotData)
+        self._job.redirection.connect(self.slotRedirection)
+        QObject.connect(self._job, SIGNAL("result(KJob*)"), self.slotResult) # new-style connection not available?
         self._job.start()
     
     def slotData(self, job, data):
@@ -597,9 +641,10 @@ class HtmlMultiLoader(object):
     Loads the first URL of a list that succeeds.
     Calls done() when done with the correct Loader or with None.
     """
+    done = Signal()
+    
     def __init__(self, urls):
         self._urls = urls
-        self.done = Signal()
         self.loadNext()
         
     def loadNext(self):
@@ -628,9 +673,10 @@ class Index(object):
     
     urls = []
     
+    loadFinished = Signal()
+    
     def __init__(self, loader, tool):
         self.loaded = None
-        self.loadFinished = Signal()
         self._loader = loader
         self.tool = tool
         loader.done.connect(self._initialLoaderDone)
@@ -699,6 +745,13 @@ class Index(object):
         a.setSeparator(True)
         menu.insertAction(self.loadingAction, a)
         
+    def addMenuTitle(self, menu, title):
+        """
+        Add a title. Use this instead of menu.addTitle, because
+        different indexes may add entries asynchroneously to the same menu.
+        """
+        menu.addTitle(title, self.loadingAction)
+        
     def addMenuUrl(self, menu, title, url):
         """
         Adds an action to the menu with an url relative
@@ -706,10 +759,8 @@ class Index(object):
         
         Only call this if the loading was successful!
         """
-        a = KAction(title, menu)
-        menu.insertAction(self.loadingAction, a) 
-        QObject.connect(a, SIGNAL("triggered()"),
-            lambda: self.tool.openUrl(self.url.resolved(KUrl(url))))
+        menu.insertAction(self.loadingAction, KAction(title, menu,
+            triggered=lambda: self.tool.openUrl(self.url.resolved(KUrl(url)))))
 
 
 class NotationReferenceIndex(Index):
@@ -719,14 +770,17 @@ class NotationReferenceIndex(Index):
         # from 2.13 on the url scheme changed slightly
         'notation/LilyPond-command-index',
         'notation/LilyPond-command-index.html',
+        # new website layout uses lower case:
+        'notation/lilypond-command-index',
+        'notation/lilypond-command-index.html',
+        # in case the manuals.html page is used as index:
+        '../notation/lilypond-command-index',
+        '../notation/lilypond-command-index.html',
         )
     
     def parse(self, html):
-        try:
-            self.items = NotationReferenceIndexParser(html).items
-            return True
-        except HTMLParser.HTMLParseError:
-            return False
+        self.items = NotationReferenceIndexParser(html).items
+        return bool(self.items)
     
     def menuTitle(self):
         return i18n("Notation Reference")
@@ -759,14 +813,17 @@ class LearningManualIndex(Index):
         # from 2.13 on the url scheme changed slightly
         'learning/LilyPond-index',
         'learning/LilyPond-index.html',
+        # new website layout uses lower case:
+        'learning/lilypond-index',
+        'learning/lilypond-index.html',
+        # in case the manuals.html page is used as index:
+        '../learning/lilypond-index',
+        '../learning/lilypond-index.html',
         )
     
     def parse(self, html):
-        try:
-            self.items = LearningManualIndexParser(html).items
-            return True
-        except HTMLParser.HTMLParseError:
-            return False
+        self.items = LearningManualIndexParser(html).items
+        return bool(self.items)
     
     def menuTitle(self):
         return i18n("Learning Manual")
@@ -791,26 +848,110 @@ class LearningManualIndex(Index):
                 break
         self.addMenuUrl(menu, i18n("Learning Manual Index"), self.url)
         
-            
+
+class InternalsReferenceIndex(Index):
+    def parse(self, html):
+        self.items = InternalsReferenceChapterParser(html).items
+        return bool(self.items)
+    
+
+class InternalsReferenceContextsIndex(InternalsReferenceIndex):
+    urls = (
+        'user/lilypond-internals/Contexts',
+        'user/lilypond-internals/Contexts.html',
+        # from 2.13 on the url scheme changed slightly
+        'internals/Contexts',
+        'internals/Contexts.html',
+        # new website layout uses lower case:
+        'internals/contexts',
+        'internals/contexts.html',
+        # in case the manuals.html page is used as index:
+        '../internals/contexts',
+        '../internals/contexts.html',
+        )
+        
+    def addMenuActions(self, menu, text, column):
+        for m in re.finditer(r"[A-Z][A-Za-z]+", text):
+            if m.start() <= column <= m.end() and m.group() in self.items:
+                self.addMenuTitle(menu, i18n("Internals Reference"))
+                self.addMenuUrl(menu, i18n("The %1 context", m.group()), self.items[m.group()])
+                break
+
+
+class InternalsReferenceGrobsIndex(InternalsReferenceIndex):
+    urls = (
+        'user/lilypond-internals/All-layout-objects',
+        'user/lilypond-internals/All-layout-objects.html',
+        # from 2.13 on the url scheme changed slightly
+        'internals/All-layout-objects',
+        'internals/All-layout-objects.html',
+        # new website layout uses lower case:
+        'internals/all-layout-objects',
+        'internals/all-layout-objects.html',
+        # in case the manuals.html page is used as index:
+        '../internals/all-layout-objects',
+        '../internals/all-layout-objects.html',
+        )
+        
+    def addMenuActions(self, menu, text, column):
+        for m in re.finditer(r"[A-Z][A-Za-z]+", text):
+            if m.start() <= column <= m.end() and m.group() in self.items:
+                self.addMenuTitle(menu, i18n("Internals Reference"))
+                self.addMenuUrl(menu, i18n("The %1 layout object", m.group()), self.items[m.group()])
+                break
+
+
+class InternalsReferenceEngraversIndex(InternalsReferenceIndex):
+    urls = (
+        'user/lilypond-internals/Engravers',
+        'user/lilypond-internals/Engravers.html',
+        # 2.12
+        'user/lilypond-internals/Engravers-and-Performers',
+        'user/lilypond-internals/Engravers-and-Performers.html',
+        # from 2.13 on the url scheme changed slightly
+        'internals/Engravers-and-Performers',
+        'internals/Engravers-and-Performers.html',
+        # new website layout uses lower case:
+        'internals/engravers-and-performers',
+        'internals/engravers-and-performers.html',
+        # in case the manuals.html page is used as index:
+        '../internals/engravers-and-performers',
+        '../internals/engravers-and-performers.html',
+        )
+        
+    def addMenuActions(self, menu, text, column):
+        for m in re.finditer(r"[A-Z][A-Za-z]*(_[A-Za-z]+)+", text):
+            if m.start() <= column <= m.end() and m.group() in self.items:
+                self.addMenuTitle(menu, i18n("Internals Reference"))
+                self.addMenuUrl(menu, m.group(), self.items[m.group()])
+                break
+
+
 class DocFinder(object):
     """
     Find and pre-parse LilyPond documentation.
     """
     def __init__(self, tool):
         loader = HtmlLoader(KUrl(docHomeUrl()))
-        self.commandIndex = NotationReferenceIndex(loader, tool)
-        self.learningIndex = LearningManualIndex(loader, tool)
-            
+        self.indices = [
+            NotationReferenceIndex(loader, tool),        
+            LearningManualIndex(loader, tool),
+            InternalsReferenceContextsIndex(loader, tool),
+            InternalsReferenceGrobsIndex(loader, tool),
+            InternalsReferenceEngraversIndex(loader, tool),
+            ]
+        
     def addHelpMenu(self, contextMenu, text, column):
-        if (self.commandIndex.loaded is False
-            and self.learningIndex.loaded is False):
+        for index in self.indices:
+            if index.loaded is not False:
+                break
+        else:
             return # no docs available
         menu = KMenu(i18n("LilyPond &Help"), contextMenu)
         menu.setIcon(KIcon("lilydoc"))
         contextMenu.addMenu(menu)
-        self.learningIndex.addMenuActionsWhenLoaded(menu, text, column)
-        self.commandIndex.addMenuActionsWhenLoaded(menu, text, column)
-
+        for index in self.indices:
+            index.addMenuActionsWhenLoaded(menu, text, column)
 
 
 # Easily get our global config
