@@ -17,37 +17,56 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """ LilyPond Quick Insert Toolbox """
 
 import re
 
-from PyQt4.QtCore import QObject, QSize, SIGNAL
+from PyQt4.QtCore import QSize, Qt
 from PyQt4.QtGui import (
     QCheckBox, QComboBox, QGridLayout, QLabel, QToolBox, QToolButton, QWidget)
 from PyKDE4.kdecore import i18n
-from PyKDE4.kdeui import KHBox
+from PyKDE4.kdeui import KIcon, KHBox, KMenu
 
 import ly.articulation
+from kateshell.shortcut import UserShortcutDispatcher, ShortcutDispatcherClient
 from frescobaldi_app.mainapp import SymbolManager
 
 
-class ToolBox(SymbolManager, QToolBox):
+class QuickInsertPanel(SymbolManager, UserShortcutDispatcher, QToolBox):
+    """
+    The Quick Insert Panel manages its own actionCollection of shortcuts in
+    the QuickInsertShortcuts instance of the mainwindow.
+    
+    If the user presses a keyboard shortcut, the Quick Insert Panel is loaded,
+    and the action is dispatched to the tool it originated from.
+    
+    The actions are named with this format: 'tool:name'. So the correct
+    tool can always be found.
+    """
     def __init__(self, tool):
         QToolBox.__init__(self)
         SymbolManager.__init__(self)
+        UserShortcutDispatcher.__init__(self, tool.mainwin.quickInsertShortcuts)
         self.mainwin = tool.mainwin
         Articulations(self)
         self.setMinimumWidth(self.sizeHint().width())
-
-class Lqi(QWidget):
+        
+        
+class Lqi(ShortcutDispatcherClient, QWidget):
     """ Abstract base class for LilyPond Quick Insert tools """
 
-    def __init__(self, toolbox, label, icon="", tooltip=""):
+    def __init__(self, toolbox, name, title, icon="", symbol="", tooltip=""):
         QWidget.__init__(self, toolbox)
+        ShortcutDispatcherClient.__init__(self, toolbox, name)
+        self.toolbox = toolbox
         self.mainwin = toolbox.mainwin
-        i = toolbox.addItem(self, label)
+        i = toolbox.addItem(self, title)
         if icon:
-            toolbox.addItemSymbol(toolbox, i, icon)
+            toolbox.setItemIcon(i, KIcon(icon))
+        elif symbol:
+            toolbox.addItemSymbol(toolbox, i, symbol)
         if tooltip:
             toolbox.setItemToolTip(i, tooltip)
 
@@ -59,15 +78,16 @@ class Articulations(Lqi):
     If text (music) is selected, the articulation will be added to all notes.
     """
     def __init__(self, toolbox):
-        Lqi.__init__(self, toolbox, i18n("Articulations"), 'articulation_prall',
-            i18n("Different kinds of articulations and other signs."))
+        Lqi.__init__(self, toolbox, 'articulation',
+            i18n("Articulations"), symbol='articulation_prall',
+            tooltip=i18n("Different kinds of articulations and other signs."))
             
         layout = QGridLayout(self)
         layout.setSpacing(0)
         row = 0
         cols = 5
 
-        self.shorthands = QCheckBox(i18n("Allow shorthands"), self)
+        self.shorthands = QCheckBox(i18n("Allow shorthands"))
         self.shorthands.setChecked(True)
         self.shorthands.setToolTip(i18n(
             "Use short notation for some articulations like staccato."))
@@ -85,20 +105,22 @@ class Articulations(Lqi):
         h.setToolTip(i18n("The direction to use for the articulations."))
         row += 1
 
+        self.titles = dict(ly.articulation.articulations(i18n))
         for title, group in ly.articulation.groups(i18n):
-            layout.addWidget(
-                QLabel('<u>%s</u>:' % title, self), row, 0, 1, cols)
+            layout.addWidget(QLabel('<u>{0}</u>:'.format(title)), row, 0, 1, cols)
             row += 1
             col = 0
             for sign, title in group:
-                b = QToolButton(self)
+                b = QToolButton(clicked=(lambda sign: lambda: self.writeSign(sign))(sign))
+                b.setContextMenuPolicy(Qt.CustomContextMenu)
+                b.customContextMenuRequested.connect((lambda button, sign:
+                    lambda pos: self.showContextMenu(sign, button.mapToGlobal(pos)))
+                    (b, sign))
                 b.setAutoRaise(True)
                 # load and convert the icon to the default text color
-                toolbox.addSymbol(b, 'articulation_%s' % sign, 22)
+                toolbox.addSymbol(b, 'articulation_' + sign, 22)
                 b.setIconSize(QSize(22, 22))
-                b.setToolTip('%s (\\%s)' % (title, sign))
-                QObject.connect(b, SIGNAL("clicked()"),
-                    lambda sign = sign: self.writeSign(sign))
+                b.setToolTip('<b>{0}</b> (\\{1})'.format(title, sign))
                 layout.addWidget(b, row, col)
                 col += 1
                 if col >= cols:
@@ -108,10 +130,10 @@ class Articulations(Lqi):
                 row += 1
 
         # help text
-        l = QLabel("<p><i>%s</i></p><p><i>%s</i></p>" % (
+        l = QLabel("<p><i>{0}</i></p><p><i>{1}</i></p>".format(
             i18n("Click an articulation sign to add it to your document."),
             i18n("If you select some music first, the articulation will "
-              "be added to all notes in the selection.")), self)
+              "be added to all notes in the selection.")))
         l.setWordWrap(True)
         layout.addWidget(l, row, 0, 4, cols)
 
@@ -129,5 +151,27 @@ class Articulations(Lqi):
         doc = self.mainwin.currentDocument()
         doc.manipulator().addArticulation(art)
         doc.view.setFocus()
+    
+    def showContextMenu(self, name, pos):
+        menu = KMenu(self.mainwin)
+        menu.aboutToHide.connect(menu.deleteLater)
+        a = menu.addAction(KIcon("accessories-character-map"),
+            i18n("Configure Keyboard Shortcut (%1)", self.shortcutText(name) or i18n("None")))
+        a.triggered.connect(lambda: self.editShortcut(name))
+        menu.popup(pos)
         
+    def editShortcut(self, name):
+        title = self.titles[name]
+        icon = self.toolbox.symbolIcon('articulation_' + name, 22)
+        super(Articulations, self).editShortcut(name, title, icon)
+        self.mainwin.currentDocument().view.setFocus()
+        
+    def actionTriggered(self, name):
+        self.writeSign(name)
+        
+    def populateAction(self, name, action):
+        if name in self.titles:
+            action.setText(self.titles[name])
+            self.toolbox.addSymbol(action, 'articulation_' + name)
+
 
