@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 Code that manages the progress bar at the bottom of
 the Frescobaldi main window
@@ -24,30 +26,31 @@ the Frescobaldi main window
 
 import weakref, time
 
-from PyQt4.QtCore import QObject, QTimer, SIGNAL
+from PyQt4.QtCore import QTimer
 
 _ticks = 10     # ticks per second
+
 class ProgressBarManager(object):
     
     def __init__(self, jobmanager, progressbar):
         self.bar = progressbar
         self.man = jobmanager
         self.times = weakref.WeakKeyDictionary() # don't keep real references
-        self.runtimes = weakref.WeakKeyDictionary()
         self.timer = QTimer()
         self.hideTimer = QTimer()
         
         self.timer.setInterval(1000 / _ticks)
-        QObject.connect(self.timer, SIGNAL("timeout()"), self.timeout)
+        self.timer.timeout.connect(self.timeout)
         self.hideTimer.setInterval(3000)
         self.hideTimer.setSingleShot(True)
-        QObject.connect(self.hideTimer, SIGNAL("timeout()"), self.bar.hide)
+        self.hideTimer.timeout.connect(self.bar.hide)
         self.man.jobStarted.connect(self.start)
         self.man.jobFinished.connect(self.stop)
         
-    def start(self, doc):
-        """ Call this when a job on doc started. """
-        lastruntime = self.runtimes.get(doc, 0.0)
+    def start(self, job):
+        """ Call this when a job has started. """
+        doc = job.document
+        lastruntime = doc.metainfo["build time"]
         if lastruntime == 0.0:
             lastruntime = 3.0 + doc.lines() / 20 # very arbitrary estimate...
         self.times[doc] = time.time()
@@ -60,12 +63,13 @@ class ProgressBarManager(object):
             self.timer.start()
         self.bar.setMaximum(self.bar.maximum() + int(_ticks * lastruntime))
                 
-    def stop(self, doc, success):
-        """ Call this when a job on doc stopped. """
+    def stop(self, job, success):
+        """ Call this when a job has stopped. """
+        doc = job.document
         starttime = self.times.get(doc, 0.0)
         if starttime and success:
             runtime = time.time() - starttime
-            self.runtimes[doc] = runtime
+            doc.metainfo["build time"] = runtime
         
         if self.man.count() == 0:
             self.timer.stop()
@@ -78,4 +82,4 @@ class ProgressBarManager(object):
     def timeout(self):
         self.bar.setValue(self.bar.value() + 1)
         
-        
\ No newline at end of file
+
