@@ -17,25 +17,30 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """ Code to run LilyPond and display its output in a LogWidget """
 
 import math, os, re, shutil, sys, tempfile, time
+from itertools import count, repeat
 
-from PyQt4.QtCore import (
-    QObject, QProcess, QSize, QTimer, QUrl, QVariant, Qt, SIGNAL)
+from PyQt4.QtCore import QProcess, QSize, QTimer, QUrl, Qt
 from PyQt4.QtGui import (
-    QBrush, QColor, QFont, QFrame, QStackedWidget, QTextBrowser,
-    QTextCharFormat, QTextCursor, QToolBar, QVBoxLayout, QWidget)
+    QBrush, QCheckBox, QColor, QFont, QFrame, QLabel, QListWidget,
+    QListWidgetItem, QStackedWidget, QTextBrowser, QTextCharFormat, QTextCursor,
+    QToolBar, QVBoxLayout, QWidget)
 from PyKDE4.kdecore import KGlobal, KPluginLoader, KProcess, KUrl, i18n
 from PyKDE4.kdeui import (
-    KApplication, KDialog, KGuiItem, KIcon, KMenu, KMessageBox, KStandardGuiItem)
+    KApplication, KDialog, KIcon, KMenu, KMessageBox, KStandardGuiItem)
 from PyKDE4.kio import KEncodingFileDialog
 
-from signals import Signal
+from signals import Signal, SignalProxy
 
 from kateshell.app import resolvetabs_text
 from frescobaldi_app.actions import openPDF
-import frescobaldi_app.mainapp
+from frescobaldi_app.mainapp import (
+    automaticLilyPondCommand, lilyPondCommand, lilyPondVersion, updatedFiles)
+
 
 def config(group):
     return KGlobal.config().group(group)
@@ -44,191 +49,328 @@ def config(group):
 _ly_message_re = re.compile(r"^((.*?):(\d+)(?::(\d+))?)(?=:)", re.M)
 
 
-class Ly2PDF(object):
+class BasicLilyPondJob(object):
     """
-    An object of this class performs one LilyPond run on a LilyPond file
-    given in filename, with output to the LogWidget log.
+    Performs one job to run LilyPond.
     
-    It emits the signal done(success, self) when the jobs has finished.
-    The updatedFiles method then can return a function that returns the
-    updated files of a given type.
+    Set different parameters and then call start() to start.
+    The Signal done(success, self) is emitted when the job has finished.
+    The SignalProxy output(msg, type, newline=False) is emitted when there is output
+    (stderr and stdout).  This proxy is called like Log and LogWidget.
+    Don't use (Basic)LilyPondJob for more than one run.
     """
-    preview = False
-    def __init__(self, lyfile, log):
-        self.done = Signal(fireonce=True)
-        # save this so the log knows if we built a PDF with point and click:
-        log.preview = self.preview
-        self.log = log
+    
+    # these are default values for attributes that can be set:
+    
+    command = "lilypond"        # lilypond command to run
+    arguments = ["--pdf"]       # arguments
+    include = []                # directories to add to include path
+    lyfile = None               # lilypond document filename to run on
+    
+    preview = False             # run with point and click
+    verbose = False             # run with --verbose
+    delfiles = True             # delete intermediate files
+    
+    startTime = 0.0             # time.time() this job started
+    
+    done = Signal(fireOnce=True)
+    output = SignalProxy()
+    
+    def __init__(self):
+        pass
         
-        self.lyfile = lyfile
-        self.basename = os.path.splitext(lyfile)[0]
-        self.directory, self.lyfile_arg = os.path.split(lyfile)
-
-        self.p = KProcess()
-        self.p.setOutputChannelMode(KProcess.MergedChannels)
-        self.p.setWorkingDirectory(self.directory)
-        cmd = [config("commands").readEntry("lilypond", QVariant("lilypond")).toString()]
-        if config("preferences").readEntry("verbose lilypond output",
-                                           QVariant(False)).toBool():
-            cmd.append("--verbose")
-        cmd.append("--pdf")
-        cmd.append(self.preview and "-dpoint-and-click" or "-dno-point-and-click")
-        if config("preferences").readEntry("delete intermediate files",
-                                           QVariant(True)).toBool():
-            cmd.append("-ddelete-intermediate-files")
-        cmd.append(self.lyfile_arg)
+    def start(self):
+        """ Starts the process. """
+        # save some values
+        self._directory, self._basename = os.path.split(self.lyfile)
         
-        self.p.setProgram(cmd)
-        QObject.connect(self.p, SIGNAL("finished(int, QProcess::ExitStatus)"),
-                        self.finished)
-        QObject.connect(self.p, SIGNAL("error(QProcess::ProcessError)"), self.error)
-        QObject.connect(self.p, SIGNAL("readyRead()"), self.readOutput)
+        # construct the full LilyPond command.
+        cmd = [self.command]
+        self.verbose and cmd.append("--verbose")
+        cmd.append("-dpoint-and-click=" + scmbool(self.preview))
+        cmd.append("-ddelete-intermediate-files=" + scmbool(self.delfiles))
+        for path in self.include:
+            cmd.append("--include")
+            cmd.append(path)
+        cmd.extend(self.arguments)
+        cmd.append(self._basename)
         
-        self.log.clear()
-        mode = unicode(self.preview and i18n("preview mode") or i18n("publish mode"))
-        self.log.writeLine(i18n("LilyPond [%1] starting (%2)...", self.lyfile_arg, mode))
-        self.startTime = time.time()
-        self.p.start()
+        # create KProcess instance that does the work
+        p = self._p = KProcess()
+        p.setOutputChannelMode(KProcess.MergedChannels)
+        p.setWorkingDirectory(self._directory)
+        p.setProgram(cmd)
+        p.finished.connect(self._finished)
+        p.error.connect(self._error)
+        p.readyRead.connect(self._readOutput)
         
-    def finished(self, exitCode, exitStatus):
-        if exitCode:
-            self.log.writeMsg(i18n("LilyPond [%1] exited with return code %2.",
-                self.lyfile_arg, exitCode), "msgerr")
-        elif exitStatus:
-            self.log.writeMsg(i18n("LilyPond [%1] exited with exit status %2.",
-                self.lyfile_arg, exitStatus), "msgerr")
+        mode = i18n("preview mode") if self.preview else i18n("publish mode")
+        version = lilyPondVersion(self.command)
+        if version:
+            self.output.writeLine(i18n("LilyPond %1 [%2] starting (%3)...",
+                format(version), self._basename, mode))
         else:
-            # We finished successfully, show elapsed time...
-            seconds = time.time() - self.startTime
-            if seconds < 60:
-                elapsed = '%.1f"' % seconds
-            else:
-                elapsed = "%i'%i\"" % divmod(seconds, 60)
-            self.log.writeMsg(i18n("LilyPond [%1] finished (%2).",
-                self.lyfile_arg, elapsed), "msgok")
-        self.bye(not (exitCode or exitStatus))
-    
-    def error(self, errCode):
-        """ Called when QProcess encounters an error """
-        if errCode == QProcess.FailedToStart:
-            self.log.writeMsg(i18n(
-                "Could not start LilyPond. Please check path and permissions."),
-                "msgerr")
-        elif errCode == QProcess.ReadError:
-            self.log.writeMsg(i18n("Could not read from the LilyPond process."),
-                "msgerr")
-        elif self.p.state() == QProcess.NotRunning:
-            self.log.writeMsg(i18n("An unknown error occured."), "msgerr")
-        if self.p.state() == QProcess.NotRunning:
-            self.bye(False)
+            self.output.writeLine(i18n("LilyPond [%1] starting (%2)...",
+                self._basename, mode))
+        
+        self.startTime = time.time()
+        p.start()
         
-    def bye(self, success):
-        # otherwise we delete ourselves during our event handler, causing crash
-        QTimer.singleShot(0, lambda: self.done(success, self))
-
     def abort(self):
         """ Abort the LilyPond job """
-        self.p.terminate()
+        self._p.terminate()
 
     def kill(self):
         """
         Immediately kill the job, and disconnect it's output signals, etc.
         Emits the done(False) signal.
         """
-        QObject.disconnect(self.p,
-            SIGNAL("finished(int, QProcess::ExitStatus)"), self.finished)
-        QObject.disconnect(self.p,
-            SIGNAL("error(QProcess::ProcessError)"), self.error)
-        QObject.disconnect(self.p, SIGNAL("readyRead()"), self.readOutput)
-        self.p.kill()
-        self.p.waitForFinished(2000)
+        self._p.finished.disconnect(self._finished)
+        self._p.error.disconnect(self._error)
+        self._p.readyRead.disconnect(self._readOutput)
+        self._p.kill()
+        self._p.waitForFinished(2000)
         self.done(False, self)
         
-    def readOutput(self):
+    def _finished(self, exitCode, exitStatus):
+        if exitCode:
+            self.output.writeMsg(i18n("LilyPond [%1] exited with return code %2.",
+                self._basename, exitCode), "msgerr")
+        elif exitStatus:
+            self.output.writeMsg(i18n("LilyPond [%1] exited with exit status %2.",
+                self._basename, exitStatus), "msgerr")
+        else:
+            # We finished successfully, show elapsed time...
+            minutes, seconds = divmod(time.time() - self.startTime, 60)
+            f = "{0:.0f}'{1:.0f}\"" if minutes else '{1:.1f}"'
+            self.output.writeMsg(i18n("LilyPond [%1] finished (%2).",
+                self._basename, f.format(minutes, seconds)), "msgok")
+        
+        # otherwise we delete ourselves during our event handler, causing crash
+        QTimer.singleShot(0, lambda: self.done(not (exitCode or exitStatus), self))
+    
+    def _error(self, errCode):
+        """ Called when QProcess encounters an error """
+        if errCode == QProcess.FailedToStart:
+            self.output.writeMsg(i18n(
+                "Could not start LilyPond. Please check path and permissions."),
+                "msgerr")
+        elif errCode == QProcess.ReadError:
+            self.output.writeMsg(i18n("Could not read from the LilyPond process."),
+                "msgerr")
+        elif self._p.state() == QProcess.NotRunning:
+            self.output.writeMsg(i18n("An unknown error occured."), "msgerr")
+        if self._p.state() == QProcess.NotRunning:
+            # otherwise we delete ourselves during our event handler, causing crash
+            QTimer.singleShot(0, lambda: self.done(False, self))
+        
+    def _readOutput(self):
         encoding = sys.getfilesystemencoding() or 'utf-8'
-        text = str(self.p.readAllStandardOutput()).decode(encoding, 'replace')
-        parts = _ly_message_re.split(text)
+        text = str(self._p.readAllStandardOutput()).decode(encoding, 'replace')
+        parts = iter(_ly_message_re.split(text))
         # parts has an odd length(1, 6, 11 etc)
         # message, <url, path, line, col, message> etc.
-        self.log.write(parts.pop(0))
-        while len(parts[:5]) == 5:
-            url, path, line, col, msg = parts[:5]
-            path = os.path.join(self.directory, path)
+        self.output.write(next(parts))
+        for url, path, line, col, msg in zip(*repeat(parts, 5)):
+            path = os.path.join(self._directory, path)
             line = int(line or "1") or 1
             col = int(col or "0")
-            self.log.writeFileRef(url, path, line, col)
-            self.log.write(msg)
-            del parts[:5]
+            self.output.writeFileRef(url, path, line, col)
+            self.output.write(msg)
     
     def updatedFiles(self):
         """
         Returns a function that can list updated files based on extension.
         """
-        return frescobaldi_app.mainapp.updatedFiles(self.lyfile,
-            math.floor(self.startTime))
-        
+        return updatedFiles(self.lyfile, math.floor(self.startTime))
 
-class LyDoc2PDF(Ly2PDF):
+
+class LilyPondJob(BasicLilyPondJob):
     """
-    Runs LilyPond on the given Document.
+    A LilyPondJob with default settings from Frescobaldi config.
     """
-    def __init__(self, doc, log, preview):
-        self.preview = preview
-        if doc.needsLocalFileManager():
+    def __init__(self):
+        super(LilyPondJob, self).__init__()
+        self.command = lilyPondCommand()
+        self.arguments = ["--pdf"]
+        self.verbose = config("preferences").readEntry("verbose lilypond output", False)
+        self.delfiles = config("preferences").readEntry("delete intermediate files", True)
+        self.include = config("preferences").readPathEntry("lilypond include path", [])
+
+
+class DocumentJob(LilyPondJob):
+    """
+    A job to be run on a Document instance.
+    The document is available in the document attribute.
+    """
+    document = None
+    
+    def __init__(self, doc=None):
+        self.document = doc
+        super(DocumentJob, self).__init__()
+        
+    def start(self):
+        if self.document.needsLocalFileManager():
             # handle nonlocal or unnamed documents
-            lyfile = doc.localFileManager(True).makeLocalFile()
+            lyfile = self.document.localFileManager(True).makeLocalFile()
         else:
-            lyfile = doc.localPath()
-            lvars = doc.variables()
-            ly = (lvars.get(preview and 'master-preview' or 'master-publish')
+            # look for %%master directives
+            lyfile = self.document.localPath()
+            lvars = self.document.variables()
+            ly = (lvars.get(self.preview and 'master-preview' or 'master-publish')
                   or lvars.get('master'))
             if ly:
                 lyfile = os.path.join(os.path.dirname(lyfile), ly)
-        super(LyDoc2PDF, self).__init__(lyfile, log)
-        doc.closed.connect(self.kill)
-        
+        self.lyfile = lyfile
+        self.document.closed.connect(self.kill)
+        super(DocumentJob, self).start()
 
-class JobManager(object):
+
+class RunLilyPondDialog(KDialog):
     """
-    Manages LilyPond jobs.
-    
-    Create new jobs with the createJob method.
-    (Stop jobs by calling abort() on the job).
-    Emits:
-    jobStarted(Document)
-    jobFinished(Document, success, job)
+    A dialog where a DocumentJob can be configured before it's started.
     """
     def __init__(self, mainwin):
         self.mainwin = mainwin
-        self.jobs = {}
-        self.jobStarted = Signal()
-        self.jobFinished = Signal()
+        KDialog.__init__(self, mainwin)
+        self.setCaption(i18n("Run LilyPond"))
+        self.setButtons(KDialog.ButtonCode(
+            KDialog.Help | KDialog.Ok | KDialog.Cancel ))
+        self.setButtonText(KDialog.Ok, i18n("Run LilyPond"))
+        self.setButtonIcon(KDialog.Ok, KIcon("run-lilypond"))
+        self.setHelp("running")
         
-    def job(self, doc):
-        return self.jobs.get(doc)
-    
-    def count(self):
-        return len(self.jobs)
+        layout = QVBoxLayout(self.mainWidget())
         
-    def docs(self):
-        return self.jobs.keys()
+        layout.addWidget(QLabel(i18n(
+            "Select which LilyPond version you want to run:")))
+            
+        self.lilypond = QListWidget()
+        self.lilypond.setIconSize(QSize(22, 22))
+        self.lilypond.setSpacing(4)
+        layout.addWidget(self.lilypond)
         
-    def createJob(self, doc, log, preview):
-        if doc in self.jobs:
-            return
-        self.jobs[doc] = LyDoc2PDF(doc, log, preview)
-        self.jobStarted(doc)
-        def finished(success, job):
-            del self.jobs[doc]
-            self.jobFinished(doc, success, job)
-        self.jobs[doc].done.connect(finished)
-        return self.jobs[doc]
+        self.preview = QCheckBox(i18n(
+            "Run LilyPond in preview mode (with Point and Click)"))
+        layout.addWidget(self.preview)
+        
+        self.verbose = QCheckBox(i18n("Run LilyPond with verbose output"))
+        layout.addWidget(self.verbose)
+        
+    def configureJob(self, job, doc=None):
+        """Configure a job, belonging to document.
+        
+        If the document is not given, it is expected to live in the document
+        attribute of the job. If there is already a job running, we just display,
+        but disable the Run button, until the old job finishes.
+        """
+        doc = doc or job.document
+        
+        # populate the dialog based on remembered settings for this document
+        self.lilypond.clear()
+        
+        # find the configured lilypond versions
+        conf = config("lilypond")
+        paths = conf.readEntry("paths", ["lilypond"]) or ["lilypond"]
+        default = conf.readEntry("default", "lilypond")
+        
+        import ly.version
+        
+        # get all versions
+        ver = dict((path, lilyPondVersion(path)) for path in paths)
+        
+        # default
+        if default not in paths:
+            default = paths[0]
+            
+        # Sort on version
+        paths.sort(key=ver.get)
+        versions = [format(ver.get(p)) for p in paths]
+        
+        # Determine automatic version (lowest possible)
+        autopath = None
+        docVersion = doc.lilyPondVersion()
+        if docVersion:
+            autopath = automaticLilyPondCommand(docVersion)
+        
+        def addItem(version, path, icon, title, tooltip):
+            item = QListWidgetItem(self.lilypond)
+            item.setIcon(KIcon(icon))
+            item.setText("{0}\n{1}: {2}".format(title, i18n("Command"), path))
+            item.setToolTip(tooltip)
+            version or item.setFlags(Qt.NoItemFlags)
+        
+        # Add all available LilyPond versions:
+        for path in paths:
+            if ver[path]:
+                title = i18n("LilyPond %1", format(ver[path]))
+                tooltip = i18n("Use LilyPond version %1", format(ver[path]))
+                addenda, tips = [], []
+                if path == default:
+                    addenda.append(i18n("default"))
+                    tips.append(i18n("Default LilyPond Version."))
+                if path == autopath:
+                    addenda.append(i18n("automatic"))
+                    tips.append(i18n("Automatic LilyPond Version (determined from document)."))
+                if addenda:
+                    title += " [{0}]".format(", ".join(addenda))
+                    tooltip += "\n{0}".format("\n".join(tips))
+                addItem(format(ver[path]), path, "run-lilypond", title,
+                    tooltip + "\n" + i18n("Path: %1",
+                        ly.version.LilyPondInstance(path).command() or path))
+            else:
+                addItem("", path, "dialog-error",
+                    i18n("LilyPond (version unknown)"),
+                    i18n("Use LilyPond (version unknown)\nPath: %1",
+                        ly.version.LilyPondInstance(path).command() or path))
+        
+        # Copy the settings from the document:
+        self.preview.setChecked(doc.metainfo["custom preview"])
+        self.verbose.setChecked(doc.metainfo["custom verbose"])
+        
+        try:
+            self.lilypond.setCurrentRow(versions.index(
+                doc.metainfo["custom lilypond version"]))
+        except ValueError:
+            cmd = autopath if autopath and conf.readEntry("automatic version",
+                False) else default
+            self.lilypond.setCurrentRow(paths.index(cmd))
+            
+        # Focus our listbox:
+        self.lilypond.setFocus()
+        
+        # Disable the Run button if a job is running for this document
+        oldjob = self.mainwin.jobManager().job(doc)
+        self.enableButtonOk(not oldjob)
+        if oldjob:
+            enable = lambda: self.enableButtonOk(True)
+            oldjob.done.connect(enable)
+        
+        # Wait for user interaction:
+        result = self.exec_()
+        
+        # If a job was running, don't listen to it anymore
+        if oldjob:
+            oldjob.done.disconnect(enable)
+        
+        if not result:
+            return False # cancelled
+        
+        # Save the settings in the document's metainfo and configure job:
+        doc.metainfo["custom preview"] = job.preview = self.preview.isChecked()
+        doc.metainfo["custom verbose"] = job.verbose = self.verbose.isChecked()
+        index = self.lilypond.currentRow()
+        doc.metainfo["custom lilypond version"] = versions[index]
+        job.command = paths[index]
+        return True
 
 
 class LogWidget(QFrame):
+    
+    preview = False   # this is used by the ActionManager
+    
     def __init__(self, parent=None):
         QFrame.__init__(self, parent)
-        self.preview = False # this is used by Ly2PDF and the ActionManager
         layout = QVBoxLayout(self)
         layout.setContentsMargins(0, 0, 0, 0)
         layout.setSpacing(0)
@@ -295,19 +437,16 @@ class Log(LogWidget):
         self.tool = tool
         self.doc = doc
         self.anchors = {}
-        self.anchorgen = anchorgen().next
+        self.anchorgen = anchorgen()
         LogWidget.__init__(self, tool.widget)
-        QObject.connect(self.textBrowser, SIGNAL("anchorClicked(QUrl)"),
-            self.anchorClicked)
+        self.textBrowser.anchorClicked.connect(self.anchorClicked)
         # context menu:
         self.textBrowser.setContextMenuPolicy(Qt.CustomContextMenu)
-        QObject.connect(self.textBrowser,
-            SIGNAL("customContextMenuRequested(QPoint)"),
-            self.showContextMenu)
+        self.textBrowser.customContextMenuRequested.connect(self.showContextMenu)
     
     def clear(self):
         self.anchors.clear()
-        self.anchorgen = anchorgen().next
+        self.anchorgen = anchorgen()
         super(Log, self).clear()
         
     def show(self):
@@ -316,7 +455,7 @@ class Log(LogWidget):
         self.tool.show()
 
     def writeFileRef(self, text, path, line, column, tooltip=None, format='url'):
-        anchor = self.anchorgen()
+        anchor = next(self.anchorgen)
         self.anchors[anchor] = FileRef(self.doc.app, path, line, column)
         f = self.formats[format]
         f.setAnchorHref(anchor)
@@ -331,16 +470,17 @@ class Log(LogWidget):
     
     def showContextMenu(self, pos):
         m = KMenu(self.textBrowser)
+        m.aboutToHide.connect(m.deleteLater)
         m.addTitle(i18n("LilyPond Log"))
         self.addContextMenuActions(m)
-        m.exec_(self.textBrowser.mapToGlobal(pos))
+        m.popup(self.textBrowser.mapToGlobal(pos))
         
     def addContextMenuActions(self, menu):
         a = menu.addAction(KIcon("edit-copy"), i18n("&Copy"))
-        QObject.connect(a, SIGNAL("triggered()"), self.copyLog)
+        a.triggered.connect(self.copyLog)
         g = KStandardGuiItem.saveAs()
         a = menu.addAction(g.icon(), g.text())
-        QObject.connect(a, SIGNAL("triggered()"), self.saveLogAs)
+        a.triggered.connect(self.saveLogAs)
 
     def copyLog(self):
         text = (self.textBrowser.textCursor().selection().toPlainText()
@@ -355,39 +495,18 @@ class Log(LogWidget):
             i18n("Save LilyPond Log as"),
             KEncodingFileDialog.Saving, self.textBrowser)
         dlg.setSelection(fileName)
-        
-        # because KEncodingFileDialog.setConfirmOverwrite() is not available
-        # in KDE 4.1, we try to use it, but fall back to our old own
-        # implementation.
-        checkOverwrite = False
-        try:
-            dlg.setConfirmOverwrite(True)
-        except AttributeError:
-            checkOverwrite = True
-        result = dlg.exec_()
-        if not result:
+        dlg.setConfirmOverwrite(True)
+        if not dlg.exec_():
             return # Cancelled
-        
-        # now confirm overwrite if the dialog didn't handle that (KDE < 4.2)
-        fileName = unicode(dlg.selectedFile())
-        if (checkOverwrite and os.path.exists(fileName)
-            and KMessageBox.warningContinueCancel(self.textBrowser,
-                i18n("A file named \"%1\" already exists. "
-                        "Are you sure you want to overwrite it?",
-                        os.path.basename(fileName)),
-                i18n("Overwrite File?"), KGuiItem(i18n("&Overwrite"))) ==
-                KMessageBox.Cancel):
-            return
-        
-        encoding = str(dlg.selectedEncoding())
-        text = unicode(self.textBrowser.textCursor().selection().toPlainText()
+        encoding = dlg.selectedEncoding()
+        fileName = dlg.selectedFile()
+        text = (self.textBrowser.textCursor().selection().toPlainText()
                 or self.textBrowser.toPlainText())
         try:
-            f = open(fileName, 'w')
-            f.write(text.encode(encoding, 'replace'))
-            f.write('\n')
-            f.close()
-        except (OSError, IOError), e:
+            with open(fileName, 'w') as f:
+                f.write(text.encode(encoding, 'replace'))
+                f.write('\n')
+        except (OSError, IOError) as e:
             KMessageBox.error(self.textBrowser,
                 i18n("Could not save LilyPond log:\n\n%1", unicode(e)))
 
@@ -478,7 +597,7 @@ class LocalFileManager(object):
     def __init__(self, doc):
         self.doc = doc
         self.directory = tempfile.mkdtemp()
-        path = unicode(self.doc.url().path())
+        path = self.doc.url().path()
         self.filename = path and os.path.basename(path) or "music.ly"
 
     def __del__(self):
@@ -499,7 +618,8 @@ class LocalFileManager(object):
         lyfile = self.path()
         # Save the file to our local storage
         # TODO: error handling
-        file(lyfile, 'w').write(self.doc.text().encode(self.doc.encoding() or 'utf-8'))
+        with open(lyfile, 'w') as f:
+            f.write(self.doc.text().encode(self.doc.encoding() or 'utf-8'))
         return lyfile
  
 
@@ -521,10 +641,15 @@ class BackgroundJob(object):
         
     def run(self, text, fileName='output.ly'):
         lyfile = os.path.join(self.directory(), fileName)
-        file(lyfile, 'w').write(text.encode('utf-8'))
+        with open(lyfile, 'w') as f:
+            f.write(text.encode('utf-8'))
+        self.log.clear()
         # ... and run LilyPond.
-        self.job = Ly2PDF(lyfile, self.log)
-        self.job.done.connect(self.finished)
+        job = self.job = LilyPondJob()
+        job.lyfile = lyfile
+        job.output.connect(self.log)
+        job.done.connect(self.finished)
+        job.start()
     
     def finished(self):
         """
@@ -632,7 +757,7 @@ class LilyPreviewDialog(KDialog):
         self.setMainWidget(self.preview)
         self.setMinimumSize(QSize(400, 300))
         self.loadSettings()
-        QObject.connect(self, SIGNAL("finished()"), self.slotFinished)
+        self.finished.connect(self.slotFinished)
         
     def loadSettings(self):
         self.restoreDialogSize(config("preview dialog"))
@@ -678,6 +803,13 @@ def anchorgen(num = 0):
     Generates an infinite row of anchor names, named
     "anchor0", "anchor1", etc.
     """
-    while True:
-        yield "anchor%d" % num
-        num += 1
+    for num in count(num):
+        yield "anchor{0}".format(num)
+
+
+def scmbool(value):
+    """
+    Returns the Scheme notation for the boolean value.
+    """
+    return "#t" if value else "#f"
+
