@@ -17,16 +17,17 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 Score Wizard
 """
 
-import os, re, sip, string, sys, types
+import os, re, sip, sys, types
 import ly, ly.dom
-from rational import Rational
+from fractions import Fraction
 
-from PyQt4.QtCore import (
-    QObject, QSize, QSizeF, QString, QStringList, QUrl, QVariant, Qt, SIGNAL)
+from PyQt4.QtCore import QSize, QSizeF, QUrl, Qt
 from PyQt4.QtGui import (
     QCheckBox, QComboBox, QGridLayout, QGroupBox, QHBoxLayout, QLabel,
     QListWidget, QListWidgetItem, QSplitter, QStackedWidget, QTextBrowser,
@@ -36,7 +37,7 @@ from PyKDE4.kdeui import (
     KCompletion, KHBox, KIcon, KLineEdit, KPageDialog, KPushButton,
     KStandardGuiItem, KVBox)
 
-from kateshell.app import lazymethod
+from kateshell.app import cacheresult
 from frescobaldi_app.mainapp import SymbolManager
 from frescobaldi_app.widgets import TapButton
 from frescobaldi_app.version import defaultVersion
@@ -47,13 +48,6 @@ def config(group=None):
         c = c.group(group)
     return c
 
-def onSignal(obj, signalName):
-    """ decorator to easily add connect a Qt signal to a Python slot."""
-    def decorator(func):
-        QObject.connect(obj, SIGNAL(signalName), func)
-        return func
-    return decorator
-
 
 class ScoreWizard(KPageDialog):
     def __init__(self, mainwin):
@@ -73,16 +67,18 @@ class ScoreWizard(KPageDialog):
         self.settings = Settings(self)
         self.loadCompletions()
         self.restoreDialogSize(config("dialogsize"))
-        @onSignal(self, "defaultClicked()")
-        def default():
-            self.titles.default()
-            self.parts.default()
-            self.settings.default()
-        @onSignal(self, "tryClicked()")
-        def previewscore():
-            self.previewDialog().showPreview()
+        self.defaultClicked.connect(self.default)
+        self.tryClicked.connect(self.previewScore)
+    
+    def default(self):
+        self.titles.default()
+        self.parts.default()
+        self.settings.default()
+    
+    def previewScore(self):
+        self.previewDialog().showPreview()
         
-    @lazymethod
+    @cacheresult
     def previewDialog(self):
         from frescobaldi_app.scorewiz import preview
         return preview.PreviewDialog(self)
@@ -109,7 +105,7 @@ class ScoreWizard(KPageDialog):
         for name, widget in self.completableWidgets.iteritems():
             c = widget.completionObject()
             c.setOrder(KCompletion.Sorted)
-            c.setItems(conf.readEntry(name, QVariant(QStringList())).toStringList())
+            c.setItems(conf.readEntry(name, []))
 
     def done(self, result):
         self.saveDialogSize(config("dialogsize"))
@@ -147,23 +143,20 @@ class Titles(QWidget):
         t.setContentsMargins(2, 2, 2, 2)
         t.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         t.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-        @onSignal(t.document().documentLayout(), "documentSizeChanged(QSizeF)")
-        def resize(size):
-            t.setMinimumSize(size.toSize() + QSize(4, 4))
+        t.setMinimumSize(QSize(350, 350))
+        #t.document().documentLayout().documentSizeChanged.connect(
+            #lambda size: t.setMinimumSize(size.toSize() + QSize(4, 4)))
 
         headers = ly.headers(i18n)
         msg = i18n("Click to enter a value.")
-        html = string.Template(titles_html % (
-                i18n("bottom of first page"),
-                i18n("bottom of last page"))
-            ).substitute(dict(
-                (k, "<a title='%s' href='%s'>%s</a>" % (msg, k, v))
-                for k, v in headers))
-        t.setHtml(html)
+        t.setHtml(titles_html.format(
+            copyrightmsg = i18n("bottom of first page"),
+            taglinemsg = i18n("bottom of last page"),
+            **dict((k, "<a title='{0}' href='{1}'>{2}</a>".format(msg, k, v))
+                    for k, v in headers)))
         l.addWidget(t)
-        @onSignal(t, "anchorClicked(QUrl)")
-        def focusEntry(qurl):
-            self.findChild(KLineEdit, qurl.toString()).setFocus()
+        t.anchorClicked.connect(lambda qurl:
+            self.findChild(KLineEdit, qurl.toString()).setFocus())
 
         g = QGridLayout()
         g.setVerticalSpacing(1)
@@ -189,7 +182,7 @@ class Titles(QWidget):
     def headers(self):
         """ Iterate over the user-entered headers (name, value) """
         for h in ly.headerNames:
-            yield h, unicode(self.findChild(KLineEdit, h).text())
+            yield h, self.findChild(KLineEdit, h).text()
 
 
 class Parts(QSplitter):
@@ -203,7 +196,7 @@ class Parts(QSplitter):
         # The part types overview widget.
         v = KVBox()
         self.addWidget(v)
-        QLabel('<b>%s</b>' % i18n("Available parts:"), v)
+        QLabel('<b>{0}</b>'.format(i18n("Available parts:")), v)
         allParts = QTreeWidget(v)
         addButton = KPushButton(KStandardGuiItem.add(), v)
         addButton.setToolTip(i18n("Add selected part to your score."))
@@ -211,15 +204,15 @@ class Parts(QSplitter):
         # The listbox with selected parts
         v = KVBox()
         self.addWidget(v)
-        QLabel('<b>%s</b>' % i18n("Score:"), v)
+        QLabel('<b>{0}</b>'.format(i18n("Score:")), v)
         score = QListWidget(v)
         self.score = score  # so the partList method can find us
         h = KHBox(v)
         removeButton = KPushButton(KStandardGuiItem.remove(), h)
         upButton = QToolButton(h)
-        upButton.setIcon(KIcon("go-up"))        # TODO: check icon
+        upButton.setIcon(KIcon("go-up"))
         downButton = QToolButton(h)
-        downButton.setIcon(KIcon("go-down"))    # TODO: check icon
+        downButton.setIcon(KIcon("go-down"))
 
         # The StackedWidget with settings
         partSettings = QStackedWidget()
@@ -262,22 +255,22 @@ class Parts(QSplitter):
                     parent.enableButton(KPageDialog.Try, False)
                 sip.delete(self.w)
                 sip.delete(self) # TODO: check if necessary
-
-        @onSignal(allParts, "itemDoubleClicked(QTreeWidgetItem*, int)")
+        
+        @allParts.itemDoubleClicked.connect
         def addPart(item, col):
             if hasattr(item, "partClass"):
                 PartItem(item.partClass)
-
-        @onSignal(allParts, "itemClicked(QTreeWidgetItem*, int)")
+        
+        @allParts.itemClicked.connect
         def toggleExpand(item, col):
             item.setExpanded(not item.isExpanded())
 
-        @onSignal(addButton, "clicked()")
+        @addButton.clicked.connect
         def addSelectedParts():
             for item in allParts.selectedItems():
                 PartItem(item.partClass)
 
-        @onSignal(removeButton, "clicked()")
+        @removeButton.clicked.connect
         def removeSelectedParts():
             for item in score.selectedItems():
                 item.remove()
@@ -295,7 +288,7 @@ class Parts(QSplitter):
                     i.setSelected(True)
             return decorator
             
-        @onSignal(upButton, "clicked()")
+        @upButton.clicked.connect
         @keepSel
         def moveUp():
             """ Move selected parts up. """
@@ -304,7 +297,7 @@ class Parts(QSplitter):
                     item = score.takeItem(row)
                     score.insertItem(row - 1, item)
 
-        @onSignal(downButton, "clicked()")
+        @downButton.clicked.connect
         @keepSel
         def moveDown():
             """ Move selected parts down. """
@@ -313,7 +306,7 @@ class Parts(QSplitter):
                     item = score.takeItem(row)
                     score.insertItem(row + 1, item)
 
-        @onSignal(score, "currentItemChanged(QListWidgetItem*, QListWidgetItem*)")
+        @score.currentItemChanged.connect
         def showItem(cur, prev):
             if cur:
                 cur.showSettingsWidget()
@@ -394,7 +387,7 @@ class Settings(SymbolManager, QWidget):
         self.pickup = QComboBox(h)
         self.pickup.addItem(i18n("None"))
         self.pickup.insertSeparator(1)
-        durs = [('note_%s' % d.replace('.', 'd'), d) for d in durations]
+        durs = [('note_' + d.replace('.', 'd'), d) for d in durations]
         for icon, text in durs:
             self.addItemSymbol(self.pickup, self.pickup.count(), icon)
             self.pickup.addItem(text)
@@ -450,20 +443,9 @@ class Settings(SymbolManager, QWidget):
         self.lylang.addItems([l.title() for l in self.languageNames])
         h.setToolTip(i18n(
             "The LilyPond language you want to use for the pitch names."))
-        @onSignal(self.lylang, "currentIndexChanged(const QString&)")
-        def slotLanguageChanged(lang):
-            """ Change the LilyPond language, affects key names """
-            lang = unicode(lang).lower()    # can be QString
-            if lang not in self.languageNames:
-                lang = 'nederlands'
-            index = self.key.currentIndex()
-            if index == -1:
-                index = 0
-            self.key.clear()
-            self.key.addItems(ly.keyNames[lang])
-            self.key.setCurrentIndex(index)
-        slotLanguageChanged('') # init with default
-
+        self.lylang.currentIndexChanged.connect(self.slotLanguageChanged)
+        self.slotLanguageChanged(0) # init with default
+        
         h = KHBox()
         v.addWidget(h)
         l = QLabel(i18n("Version:"), h)
@@ -472,7 +454,7 @@ class Settings(SymbolManager, QWidget):
         l.setBuddy(self.lyversion)
         version = defaultVersion()
         if version:
-            self.lyversion.addItem(version)
+            self.lyversion.addItem(str(version))
         self.lyversion.addItems(('2.10.0', '2.11.0'))
         h.setToolTip(i18n(
             "The LilyPond version you will be using for this document."))
@@ -502,6 +484,11 @@ class Settings(SymbolManager, QWidget):
             "score. The MIDI output also uses the metronome setting."))
         v.addWidget(self.metro)
 
+        self.book = QCheckBox(i18n("Wrap score in \\book block"))
+        self.book.setToolTip(i18n(
+            "If checked, wraps the \\score block inside a \\book block."))
+        v.addWidget(self.book)
+
         # paper size:
         h = KHBox()
         v.addWidget(h)
@@ -512,9 +499,7 @@ class Settings(SymbolManager, QWidget):
         self.paperLandscape = QCheckBox(i18n("Landscape"), h)
         self.paper.addItem(i18n("Default"))
         self.paper.addItems(ly.paperSizes)
-        @onSignal(self.paper, "activated(int)")
-        def checkLandscape(i):
-            self.paperLandscape.setEnabled(bool(i))
+        self.paper.activated.connect(lambda i: self.paperLandscape.setEnabled(bool(i)))
 
         # Instrument names
         instr.setCheckable(True)
@@ -548,9 +533,8 @@ class Settings(SymbolManager, QWidget):
         h.setToolTip(i18n("Which language to use for the instrument names."))
 
         langs = KGlobal.dirs().findAllResources("locale", "*/LC_MESSAGES/frescobaldi.mo")
-        self.instrLanguages = list(sorted(set(unicode(lang).split('/')[-3] for lang in langs)))
-        self.instrLang.addItems([KGlobal.locale().languageCodeToName(lang)
-                for lang in self.instrLanguages])
+        self.instrLanguages = list(sorted(set(lang.split('/')[-3] for lang in langs)))
+        self.instrLang.addItems(map(KGlobal.locale().languageCodeToName, self.instrLanguages))
         
         self.default()
         self.loadConfig()
@@ -558,38 +542,40 @@ class Settings(SymbolManager, QWidget):
     def saveConfig(self):
         conf = config()
         conf.writeEntry('language', self.getLanguage() or 'default')
-        conf.writeEntry('typographical', QVariant(self.typq.isChecked()))
-        conf.writeEntry('remove tagline', QVariant(self.tagl.isChecked()))
-        conf.writeEntry('remove barnumbers', QVariant(self.barnum.isChecked()))
-        conf.writeEntry('midi', QVariant(self.midi.isChecked()))
-        conf.writeEntry('metronome mark', QVariant(self.metro.isChecked()))
+        conf.writeEntry('typographical', self.typq.isChecked())
+        conf.writeEntry('remove tagline', self.tagl.isChecked())
+        conf.writeEntry('remove barnumbers', self.barnum.isChecked())
+        conf.writeEntry('midi', self.midi.isChecked())
+        conf.writeEntry('metronome mark', self.metro.isChecked())
+        conf.writeEntry('wrap in book', self.book.isChecked())
         if self.paper.currentIndex() > 0:
             conf.writeEntry('paper size', ly.paperSizes[self.paper.currentIndex() - 1])
-        conf.writeEntry('paper landscape', QVariant(self.paperLandscape.isChecked()))
+        conf.writeEntry('paper landscape', self.paperLandscape.isChecked())
         g = config('instrument names')
-        g.writeEntry('show', QVariant(self.instr.isChecked()))
+        g.writeEntry('show', self.instr.isChecked())
         g.writeEntry('first', ['long', 'short'][self.instrFirst.currentIndex()])
         g.writeEntry('other', ['long', 'short', 'none'][self.instrOther.currentIndex()])
         g.writeEntry('lang', (['default', 'english'] + self.instrLanguages)[self.instrLang.currentIndex()])
 
     def loadConfig(self):
         conf = config()
-        self.setLanguage(conf.readEntry('language', QVariant('default')).toString())
-        self.typq.setChecked(conf.readEntry('typographical', QVariant(True)).toBool())
-        self.tagl.setChecked(conf.readEntry('remove tagline', QVariant(False)).toBool())
-        self.barnum.setChecked(conf.readEntry('remove barnumbers', QVariant(False)).toBool())
-        self.midi.setChecked(conf.readEntry('midi', QVariant(True)).toBool())
-        self.metro.setChecked(conf.readEntry('metronome mark', QVariant(False)).toBool())
+        self.setLanguage(conf.readEntry('language', 'default'))
+        self.typq.setChecked(conf.readEntry('typographical', True))
+        self.tagl.setChecked(conf.readEntry('remove tagline', False))
+        self.barnum.setChecked(conf.readEntry('remove barnumbers', False))
+        self.midi.setChecked(conf.readEntry('midi', True))
+        self.metro.setChecked(conf.readEntry('metronome mark', False))
+        self.book.setChecked(conf.readEntry('wrap in book', False))
 
-        psize = conf.readEntry('paper size', QVariant('')).toString()
+        psize = conf.readEntry('paper size', "")
         if psize in ly.paperSizes:
             self.paper.setCurrentIndex(ly.paperSizes.index(psize) + 1)
-        self.paperLandscape.setChecked(conf.readEntry('paper landscape', QVariant(False)).toBool())
+        self.paperLandscape.setChecked(conf.readEntry('paper landscape', False))
         self.paperLandscape.setEnabled(psize in ly.paperSizes)
 
         g = config('instrument names')
         def readconf(entry, itemlist, defaultIndex):
-            item = g.readEntry(entry, QVariant(itemlist[defaultIndex])).toString()
+            item = g.readEntry(entry, itemlist[defaultIndex])
             if item in itemlist:
                 return itemlist.index(item)
             else:
@@ -602,7 +588,7 @@ class Settings(SymbolManager, QWidget):
         self.instrFirst.setCurrentIndex(first)
         self.instrOther.setCurrentIndex(other)
         self.instrLang.setCurrentIndex(lang)
-        self.instr.setChecked(g.readEntry('show', QVariant(True)).toBool())
+        self.instr.setChecked(g.readEntry('show', True))
 
     def default(self):
         """ Set various items to their default state """
@@ -619,6 +605,7 @@ class Settings(SymbolManager, QWidget):
         self.barnum.setChecked(False)
         self.midi.setChecked(True)
         self.metro.setChecked(False)
+        self.book.setChecked(False)
         self.paper.setCurrentIndex(0)
         self.paperLandscape.setEnabled(False)
         self.instrFirst.setCurrentIndex(0)
@@ -628,7 +615,7 @@ class Settings(SymbolManager, QWidget):
 
     def getLanguage(self):
         """ Return the configured LilyPond pitch language, '' for default. """
-        if self.lylang.currentIndex():
+        if self.lylang.currentIndex() >= 2:
             return self.languageNames[self.lylang.currentIndex() - 2]
         else:
             return ''
@@ -639,6 +626,16 @@ class Settings(SymbolManager, QWidget):
             self.lylang.setCurrentIndex(0)
         else:
             self.lylang.setCurrentIndex(self.languageNames.index(lang) + 2)
+    
+    def slotLanguageChanged(self, index):
+        """ Change the LilyPond language, affects key names """
+        lang = index < 2 and "nederlands" or self.languageNames[index - 2]
+        key = self.key.currentIndex()
+        if key == -1:
+            key = 0
+        self.key.clear()
+        self.key.addItems(ly.keyNames[lang])
+        self.key.setCurrentIndex(key)
 
 
 class Builder(object):
@@ -652,24 +649,37 @@ class Builder(object):
 
     Parts may interact with:
     
+    methods:
     include             to request filenames to be included
     
     addCodeBlock        to add arbitrary strings to the output (e.g. functions)
 
-    lilypondVersion     a tuple like (2, 11, 64) describing the LilyPond the
-                        document is built for.
-
     getInstrumentNames  to translate instrument names
 
     setInstrumentNames  to translate and set instrument names for a node
     
+    setMidiInstrument   to set the Midi instrument for a node
+    
+    getMidiTempo        to get the configured tempo e.g."(ly:make-moment 100 4)"
+    
+    setMidiTempo        to write the configured tempo in the given context
+    
+    properties:
+    lilyPondVersion     a tuple like (2, 11, 64) describing the LilyPond the
+                        document is built for.
+
     midi                property is True if MIDI output is requested
     
-    setMidiInstrument   to set the Midi instrument for a node
+    book                True if the main \\score { } block is to be wrapped
+                        inside a \\book { } block. A part may set this to True
+                        if it is needed (e.g. when alternate output files are to
+                        be created).
+    
     """
     def __init__(self, wizard):
         self.wizard = wizard
         self.midi = wizard.settings.midi.isChecked()
+        self.book = wizard.settings.book.isChecked()
         
     def ly(self, doc=None):
         """ Return LilyPond formatted output. """
@@ -695,13 +705,13 @@ class Builder(object):
         self.translate = lambda s: s    # english (untranslated)
         i = s.instrLang.currentIndex()
         if i == 0:                      # default (translated)
-            self.translate = lambda s: unicode(i18n(s))
+            self.translate = i18n
         elif i >= 2:                    # other translation
             try:
                 import gettext
-                self.translate = gettext.GNUTranslations(open(unicode(
+                self.translate = gettext.GNUTranslations(open(
                   KGlobal.dirs().findResource("locale", s.instrLanguages[i-2] +
-                    "/LC_MESSAGES/frescobaldi.mo")))).ugettext
+                    "/LC_MESSAGES/frescobaldi.mo"))).ugettext
             except IOError:
                 pass
         
@@ -712,10 +722,10 @@ class Builder(object):
         self.codeBlocks = []
         
         # version:
-        version = unicode(s.lyversion.currentText())
+        version = s.lyversion.currentText()
         ly.dom.Version(version, doc)
         ly.dom.BlankLine(doc)
-        self.lilypondVersion = tuple(map(int, re.findall('\\d+', version)))
+        self.lilyPondVersion = tuple(map(int, re.findall('\\d+', version)))
 
         # header:
         h = ly.dom.Header()
@@ -731,7 +741,7 @@ class Builder(object):
 
         # paper size:
         if s.paper.currentIndex():
-            ly.dom.Scheme('(set-paper-size "%s"%s)' % (
+            ly.dom.Scheme('(set-paper-size "{0}"{1})'.format(
                     s.paper.currentText(),
                     s.paperLandscape.isChecked() and " 'landscape" or ""),
                 ly.dom.Paper(doc)).after = 1
@@ -748,7 +758,7 @@ class Builder(object):
         # pitch language:
         language = s.getLanguage()
         if language:
-            self.include("%s.ly" % language)
+            self.include("{0}.ly".format(language))
 
         # add code blocks, if any:
         for code in self.codeBlocks[::-1]:
@@ -774,7 +784,7 @@ class Builder(object):
         g = ly.dom.Seq(globalAssignment)
 
         # First find out if we need to define a tempoMark section.
-        tempoText = unicode(s.tempoInd.text())
+        tempoText = s.tempoInd.text()
         metro = s.metro.isChecked()
         dur = durations[s.metroDur.currentIndex()]
         val = s.metroVal.currentText()
@@ -797,8 +807,7 @@ class Builder(object):
             if metro:
                 # Constuct a tempo indication with metronome mark
                 ly.dom.QuotedString(tempoText + " ", m)
-                ly.dom.Line(r'\small \general-align #Y #DOWN \note #"%s" #1 = %s' %
-                    (dur, val), m)
+                ly.dom.Line(r'\small \general-align #Y #DOWN \note #"{0}" #1 = {1}'.format(dur, val), m)
             else:
                 # Constuct a tempo indication without metronome mark
                 ly.dom.QuotedString(tempoText, m)
@@ -812,14 +821,14 @@ class Builder(object):
 
         # key signature
         note, alter = ly.keys[s.key.currentIndex()]
-        alter = Rational(alter, 2)
+        alter = Fraction(alter, 2)
         mode = ly.modes()[s.mode.currentIndex()][0]
         ly.dom.KeySignature(note, alter, mode, g).after = 1
         # time signature
-        match = re.search('(\\d+).*?(\\d+)', unicode(s.time.currentText()))
+        match = re.search('(\\d+).*?(\\d+)', s.time.currentText())
         if match:
             if s.time.currentText() in ('2/2', '4/4'):
-                if self.lilypondVersion >= (2, 11, 44):
+                if self.lilyPondVersion >= (2, 11, 44):
                     ly.dom.Line(r"\numericTimeSignature", g)
                 else:
                     ly.dom.Line(r"\override Staff.TimeSignature #'style = #'()", g)
@@ -883,12 +892,15 @@ class Builder(object):
                 for n in part.nodes:
                     sim.append(n)
 
-        # clean up the parts:
-        for part in partList:
-            part.cleanup()
-        
         # put the score in the document
-        doc.append(score)
+        if self.book:
+            book = ly.dom.Book()
+            book.append(score)
+            doc.append(book)
+        else:
+            doc.append(score)
+        
+        # Add some layout stuff
         lay = ly.dom.Layout(score)
         if s.barnum.isChecked():
             ly.dom.Line('\\remove "Bar_number_engraver"',
@@ -896,11 +908,19 @@ class Builder(object):
         if self.midi:
             mid = ly.dom.Midi(score)
             if tempoText or not metro:
-                base, mul = midiDurations[s.metroDur.currentIndex()]
-                val = int(val) * mul
-                ly.dom.Context('Score', mid)['tempoWholesPerMinute'] = \
-                    ly.dom.Scheme("(ly:make-moment %s %s)" % (val, base))
+                self.setMidiTempo(ly.dom.Context('Score', mid))
+        
+        # Add possible aftermath:
+        for part in partList:
+            if part.aftermath:
+                ly.dom.BlankLine(doc)
+                for n in part.aftermath:
+                    doc.append(n)
 
+        # clean up the parts:
+        for part in partList:
+            part.cleanup()
+        
     ##
     # The following methods are to be used by the parts.
     ##
@@ -979,6 +999,22 @@ class Builder(object):
         """
         if self.midi:
             node.getWith()['midiInstrument'] = midiInstrument
+    
+    def getMidiTempo(self):
+        """
+        Returns the configured tempo as a string e.g. "(ly:make-moment 100 4)"
+        """
+        s = self.wizard.settings
+        base, mul = midiDurations[s.metroDur.currentIndex()]
+        val = int(s.metroVal.currentText()) * mul
+        return "(ly:make-moment {0} {1})".format(val, base)
+        
+    def setMidiTempo(self, node):
+        """
+        Writes the configured tempo to the 'tempoWholesPerMinute' variable
+        of the given context (should be subclass of ly.dom.HandleVars).
+        """
+        node['tempoWholesPerMinute'] = ly.dom.Scheme(self.getMidiTempo())
 
 
 class PartBase(object):
@@ -996,10 +1032,11 @@ class PartBase(object):
 
     def cleanup(self):
         """
-        Delete previously built assignments and nodes
+        Delete previously built assignments, nodes and aftermath
         """
         self.assignments = []
         self.nodes = []
+        self.aftermath = []
         
     def run(self, builder):
         """
@@ -1037,7 +1074,7 @@ class PartBase(object):
         Reimplement this method to add widgets with settings
         to the given layout.
         """
-        layout.addWidget(QLabel('(%s)' % i18n("No settings available.")))
+        layout.addWidget(QLabel('({0})'.format(i18n("No settings available."))))
 
     def build(self, builder):
         """
@@ -1045,45 +1082,49 @@ class PartBase(object):
         self.assignments and self.nodes.
         builder is a Builder instance providing access to users settings.
         You must implement this method in your part subclasses.
+        
+        You may also add ly.dom.Node objects to the self.aftermath list. These
+        are reproduces below the main score and can be used to create additional
+        score sections or markups, etc.
         """
         pass
 
 
-titles_html = r"""
-<html><head><style type='text/css'>
-body {
+titles_html = r"""<html><head><style type='text/css'>
+body {{
   background-color: #fefefe;
   color: black;
-}
-a {
+}}
+a {{
   text-decoration: none;
   color: black;
-}
+}}
 </style></head>
-<body><table width='100%%' style='font-family:serif;'>
-<tr><td colspan=3 align=center>$dedication</td></tr>
-<tr><td colspan=3 align=center style='font-size:20pt;'><b>$title</b></td></tr>
-<tr><td colspan=3 align=center style='font-size:12pt;'><b>$subtitle</b></td></tr>
-<tr><td colspan=3 align=center><b>$subsubtitle</b></td></tr>
+<body><table width='100%' style='font-family:serif;'>
+<tr><td colspan=3 align=center>{dedication}</td></tr>
+<tr><td colspan=3 align=center style='font-size:20pt;'><b>{title}</b></td></tr>
+<tr><td colspan=3 align=center style='font-size:12pt;'><b>{subtitle}</b></td></tr>
+<tr><td colspan=3 align=center><b>{subsubtitle}</b></td></tr>
 <tr>
-    <td align=left width='25%%'>$poet</td>
-    <td align=center><b>$instrument</b></td>
-    <td align=right width='25%%'>$composer</td>
+    <td align=left width='25%'>{poet}</td>
+    <td align=center><b>{instrument}</b></td>
+    <td align=right width='25%'>{composer}</td>
 </tr>
 <tr>
-    <td align=left>$meter</td>
+    <td align=left>{meter}</td>
     <td> </td>
-    <td align=right>$arranger</td>
+    <td align=right>{arranger}</td>
 </tr>
 <tr>
-    <td align=left>$piece</td>
+    <td align=left>{piece}</td>
     <td> </td>
-    <td align=right>$opus</td>
+    <td align=right>{opus}</td>
 </tr>
 <tr><td colspan=3 align=center><img src='scorewiz.png'></td></tr>
-<tr><td colspan=3 align=center>$copyright <i>(%s)</i></td></tr>
-<tr><td colspan=3 align=center>$tagline <i>(%s)</i></td></tr>
-</table></body></html>"""
+<tr><td colspan=3 align=center>{copyright} <i>({copyrightmsg})</i></td></tr>
+<tr><td colspan=3 align=center>{tagline} <i>({taglinemsg})</i></td></tr>
+</table></body></html>
+"""
 
 durations = ['16', '16.', '8', '8.', '4', '4.', '2', '2.', '1', '1.']
 midiDurations = ((16,1),(32,3),(8,1),(16,3),(4,1),(8,3),(2,1),(4,3),(1,1),(2,3))
