@@ -17,14 +17,18 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 Part types for the Score Wizard (scorewiz/__init__.py).
 In separate file to ease maintenance.
 """
 
-from rational import Rational
+from fractions import Fraction
+from collections import defaultdict
+from itertools import cycle, islice, chain, repeat
 
-from PyQt4.QtCore import QObject, QRegExp, Qt, SIGNAL
+from PyQt4.QtCore import QRegExp, Qt
 from PyQt4.QtGui import (
     QCheckBox, QComboBox, QGroupBox, QLabel, QRadioButton, QRegExpValidator,
     QSpinBox, QVBoxLayout)
@@ -89,7 +93,7 @@ class Part(frescobaldi_app.scorewiz.PartBase):
         Identifier('global', s).after = 1
         if transposition is not None:
             toct, tnote, talter = transposition
-            Pitch(toct, tnote, Rational(talter, 2), Transposition(s))
+            Pitch(toct, tnote, Fraction(talter, 2), Transposition(s))
         LineComment(i18n("Music follows here."), s)
         BlankLine(s)
         return s, ref
@@ -165,8 +169,7 @@ class TablaturePart(SingleVoicePart):
             i18n("Tablature"),
             i18n("Both")))
         if self.tunings:
-            QObject.connect(self.staffType, SIGNAL("activated(int)"),
-                self.slotTabEnable)
+            self.staffType.activated.connect(self.slotTabEnable)
             self.widgetsTuning(layout)
             self.slotTabEnable(0)
 
@@ -238,7 +241,7 @@ class TablaturePart(SingleVoicePart):
             else:
                 for num, ref in zip(order, refs):
                     s = Seq(TabVoice(parent=sim))
-                    Text('\\voice%s' % ly.nums(num), s)
+                    Text('\\voice' + ly.nums(num), s)
                     Identifier(ref, s)
         
         if staffType == 0:
@@ -292,7 +295,7 @@ class VocalPart(Part):
         l = LyricMode()
         if verse:
             name = name + ly.nums(verse)
-            Line('\\set stanza = "%d."' % verse, l)
+            Line('\\set stanza = "{0}."'.format(verse), l)
         a, ref = self.assign(name)
         a.append(l)
         LineComment(i18n("Lyrics follow here."), l)
@@ -379,7 +382,7 @@ class KeyboardPart(Part):
         self.nodes.append(p)
 
     def widgets(self, layout):
-        l = QLabel('%s <i>(%s)</i>' % (
+        l = QLabel('{0} <i>({1})</i>'.format(
             i18n("Adjust how many separate voices you want on each staff."),
             i18n("This is primarily useful when you write polyphonic music "
             "like a fuge.")))
@@ -415,8 +418,8 @@ class Chords(Part):
         Identifier('global', s).after = 1
         i = self.chordStyle.currentIndex()
         if i > 0:
-            Line('\\%sChords' %
-                ('german', 'semiGerman', 'italian', 'french')[i-1], s)
+            Line('\\{0}Chords'.format(
+                ('german', 'semiGerman', 'italian', 'french')[i-1]), s)
         LineComment(i18n("Chords follow here."), s)
         BlankLine(s)
         self.nodes.append(p)
@@ -553,8 +556,8 @@ class Banjo(TablaturePart):
             super(Banjo, self).setTunings(tab)
         else:
             tab.getWith()['stringTunings'] = Scheme(
-                '(four-string-banjo %s)' %
-                    self.tunings[self.tuningSel.currentIndex()][1])
+                '(four-string-banjo {0})'.format(
+                    self.tunings[self.tuningSel.currentIndex()][1]))
 
 
 class ClassicalGuitar(TablaturePart):
@@ -887,9 +890,9 @@ class LeadSheet(VocalPart, Chords):
         self.nodes.append(p)
 
     def widgets(self, layout):
-        l = QLabel('<i>%s</i>' % i18n(
+        l = QLabel('<i>{0}</i>'.format(i18n(
             "The Lead Sheet provides a staff with chord names above "
-            "and lyrics below it. A second staff is optional."))
+            "and lyrics below it. A second staff is optional.")))
         l.setWordWrap(True)
         layout.addWidget(l)
         self.chords = QGroupBox(i18n("Chord names"))
@@ -911,7 +914,7 @@ class Choir(VocalPart):
     _name = ki18n("Choir")
 
     def widgets(self, layout):
-        l = QLabel('<p>%s <i>(%s)</i></p>' % (
+        l = QLabel('<p>{0} <i>({1})</i></p>'.format(
             i18n("Please select the voices for the choir. "
             "Use the letters S, A, T, or B. A hyphen denotes a new staff."),
             i18n("Hint: For a double choir you can use two choir parts.")))
@@ -933,204 +936,252 @@ class Choir(VocalPart):
             'SS-A-T-B', 'S-A-TT-B', 'SS-A-TT-B',
             'S-S-A-T-T-B', 'S-S-A-A-T-T-B-B',
             ))
-        g = QGroupBox(i18n("Lyrics"))
-        layout.addWidget(g)
-        group = QVBoxLayout()
-        g.setLayout(group)
-        self.lyrAllSame = QRadioButton(i18n("All voices same lyrics"))
-        self.lyrAllSame.setChecked(True)
-        self.lyrAllSame.setToolTip(i18n(
-            "One set of the same lyrics is placed between all staves."))
-        group.addWidget(self.lyrAllSame)
-        self.lyrEachSame = QRadioButton(i18n("Every voice same lyrics"))
-        self.lyrEachSame.setToolTip(i18n(
-            "Every voice gets its own lyrics, using the same text as the "
-            "other voices."))
-        group.addWidget(self.lyrEachSame)
-        self.lyrEachDiff = QRadioButton(i18n("Every voice different lyrics"))
-        self.lyrEachDiff.setToolTip(i18n(
-            "Every voice gets a different set of lyrics."))
-        group.addWidget(self.lyrEachDiff)
-        self.stanzaWidget(group)
+        self.stanzaWidget(layout)
+        h = KHBox()
+        layout.addWidget(h)
+        l = QLabel(i18n("Lyrics:"), h)
+        self.lyrics = QComboBox(h)
+        l.setBuddy(self.lyrics)
+        for index, (text, tooltip) in enumerate((
+         (i18n("All voices same lyrics"),
+          i18n("A set of the same lyrics is placed between all staves.")),
+         (i18n("Every voice same lyrics"),
+          i18n("Every voice gets its own lyrics, using the same text as the"
+               " other voices.")),
+         (i18n("Every voice different lyrics"),
+          i18n("Every voice gets a different set of lyrics.")),
+         (i18n("Distribute stanzas"),
+          i18n("One set of stanzas is distributed across the staves.")))):
+            self.lyrics.addItem(text)
+            self.lyrics.setItemData(index, tooltip, Qt.ToolTipRole)
+        self.lyrics.setCurrentIndex(0)
         self.ambitusWidget(layout)
         self.pianoReduction = QCheckBox(i18n("Piano reduction"))
         self.pianoReduction.setToolTip(i18n(
             "Adds an automatically generated piano reduction."))
         layout.addWidget(self.pianoReduction)
+        self.rehearsalMidi = QCheckBox(i18n("Rehearsal MIDI files"))
+        self.rehearsalMidi.setToolTip(i18n(
+            "Creates a rehearsal MIDI file for every voice, "
+            "even if no MIDI output is generated for the main score."))
+        layout.addWidget(self.rehearsalMidi)
 
-    partInfo = {
-        'S': ('soprano', 1, SopranoVoice.instrumentNames),
-        'A': ('alto', 0, AltoVoice.instrumentNames),
-        'T': ('tenor', 0, TenorVoice.instrumentNames),
-        'B': ('bass', -1, BassVoice.instrumentNames),
+    identifiers = {
+        'S': 'soprano',
+        'A': 'alto',
+        'T': 'tenor',
+        'B': 'bass',
     }
-
+    
+    octaves = {
+        'S': SopranoVoice.octave,
+        'A': AltoVoice.octave,
+        'T': TenorVoice.octave,
+        'B': BassVoice.octave,
+    }
+    
+    instrumentNames = {
+        'S': SopranoVoice.instrumentNames,
+        'A': AltoVoice.instrumentNames,
+        'T': TenorVoice.instrumentNames,
+        'B': BassVoice.instrumentNames,
+    }
+            
     def build(self, builder):
         # normalize voicing
-        staves = unicode(self.voicing.currentText()).upper()
+        staves = self.voicing.currentText().upper()
         # remove unwanted characters
         staves = re.sub(r'[^SATB-]+', '', staves)
         # remove double hyphens, and from begin and end
         staves = re.sub('-+', '-', staves).strip('-')
         if not staves:
             return
+        
         splitStaves = staves.split('-')
+        numStaves = len(splitStaves)
+        staffCIDs = defaultdict(int)    # number same-name staff Context-IDs
+        voiceCounter = defaultdict(int) # dict to number same voice types
+        maxNumVoices = max(map(len, splitStaves)) # largest number of voices
+        numStanzas = self.stanzas.value()
+        lyrics = defaultdict(list)      # lyrics grouped by stanza number
+        pianoReduction = defaultdict(list)
+        rehearsalMidis = []
+        
         p = ChoirStaff()
         choir = Sim(p)
+        self.nodes.append(p)
+        
         # print main instrumentName if there are more choirs, and we
         # have more than one staff.
-        if len(splitStaves) > 1 and self.num:
+        if numStaves > 1 and self.num:
             builder.setInstrumentNames(p, I18N_NOOP("Choir|Ch."), self.num)
-        count = dict.fromkeys('SATB', 0)  # dict with count of parts.
-        maxLen = max(map(len, splitStaves))
-        toGo = max(2, len(splitStaves)) # nr staves with same lyrics below + 1
-        lyr, staffNames = [], []
-        pianoReduction = dict((key, []) for key in 'SATB')
-        for staff in splitStaves:
-            toGo -= 1
+        
+        # get the preferred way of adding lyrics
+        lyrAllSame, lyrEachSame, lyrEachDiff, lyrSpread = (
+            self.lyrics.currentIndex() == i for i in range(4))
+        lyrEach = lyrEachSame or lyrEachDiff
+        
+        # stanzas to print (0 = don't print stanza number):
+        if numStanzas == 1:
+            allStanzas = [0]
+        else:
+            allStanzas = list(range(1, numStanzas + 1))
+        
+        # Which stanzas to print where:
+        if lyrSpread and numStanzas > 1 and numStaves > 2:
+            spaces = numStaves - 1
+            count, rest = divmod(max(numStanzas, spaces), spaces)
+            stanzaSource = cycle(allStanzas)
+            stanzaGroups = (islice(stanzaSource, num) for num in chain(
+                repeat(count + 1, rest), repeat(count, numStaves - rest)))
+        else:
+            stanzaGroups = repeat(allStanzas, numStaves)
+        
+        # a function to set staff affinity (in LilyPond 2.13.4 and above):
+        if builder.lilyPondVersion >= (2, 13, 4):
+            def setStaffAffinity(context, affinity):
+                Line("\\override VerticalAxisGroup "
+                     "#'staff-affinity = #" + affinity, context.getWith())
+        else:
+            def setStaffAffinity(lyricsContext, affinity):
+                pass
+        
+        # a function to make a column markup:
+        if builder.lilyPondVersion >= (2, 11, 57):
+            columnCommand = 'center-column'
+        else:
+            columnCommand = 'center-align'
+        def makeColumnMarkup(names):
+            node = Markup()
+            column = MarkupEnclosed(columnCommand, node)
+            for name in names:
+                QuotedString(name, column)
+            return node
+        
+        stavesLeft = numStaves
+        for staff, stanzas in zip(splitStaves, stanzaGroups):
+            # are we in the last staff?
+            stavesLeft -= 1
+            # the number of voices in this staff
+            numVoices = len(staff)
             # sort the letters in order SATB
             staff = ''.join(i * staff.count(i) for i in 'SATB')
             # Create the staff for the voices
             s = Staff(parent=choir)
             builder.setMidiInstrument(s, self.midiInstrument)
-            # Build lists of the voices and their instrument names
-            instrNames, voices = [], []
-            for part in staff:
-                if staves.count(part) > 1:
-                    count[part] += 1
-                name, octave, instrName = self.partInfo[part]
-                instrNames.append((instrName, count[part]))
-                voices.append((part, name, count[part], octave))
-            if len(staff) == 1:
-                # There is only one voice in the staff. Just set the instrument
-                # name directly in the staff.
-                name, num = instrNames[0]
-                builder.setInstrumentNames(s, name, num)
-                # if *all* staves have only one voice, addlyrics is used.
-                # In that case, don't remove the braces.
-                mus = maxLen == 1 and Seq(s) or Seqr(s)
+            
+            # Build a list of the voices in this staff.
+            # Each entry is a tuple(name, num).
+            # name is one of 'S', 'A', 'T', or 'B'
+            # num is an integer: 0 when a voice occurs only once, or >= 1 when
+            # there are more voices of the same type (e.g. Soprano I and II)
+            voices = []
+            for voice in staff:
+                if staves.count(voice) > 1:
+                    voiceCounter[voice] += 1
+                voices.append((voice, voiceCounter[voice]))
+            
+            # Add the instrument names to the staff:
+            if numVoices == 1:
+                voice, num = voices[0]
+                builder.setInstrumentNames(s, self.instrumentNames[voice], num)
             else:
-                # There are more instrument names for the staff, stack them in
-                # a markup column.
-                def mkup(names):
-                    n = Markup()
-                    if builder.lilypondVersion >= (2, 11, 57):
-                        col = 'center-column'
-                    else:
-                        col = 'center-align'
-                    col = MarkupEnclosed(col, n)
-                    for name in names:
-                        QuotedString(name, col)
-                    return n
-                builder.setInstrumentNames(s, map(mkup, zip(
-                    *[builder.getInstrumentNames(name, num)
-                        for name, num in instrNames])))
-                mus = Simr(s)
+                # stack instrument names (long and short) in a markup column.
+                builder.setInstrumentNames(s,
+                  map(makeColumnMarkup, zip(*[builder.getInstrumentNames(
+                  self.instrumentNames[voice], num) for voice, num in voices])))
+            
+            # Make the { } or << >> holder for this staff's children.
+            # If *all* staves have only one voice, addlyrics is used.
+            # In that case, don't remove the braces.
+            staffMusic = (Seq if lyrEach and maxNumVoices == 1 else
+                          Seqr if numVoices == 1 else Simr)(s)
+            
             # Set the clef for this staff:
             if 'B' in staff:
-                Clef('bass', mus)
+                Clef('bass', staffMusic)
             elif 'T' in staff:
-                Clef('treble_8', mus)
+                Clef('treble_8', staffMusic)
 
-            stanzas = self.stanzas.value()
-            stanzas = stanzas == 1 and [0] or range(1, stanzas + 1)
-
-            # Add the voices
-            if len(staff) == 1:
-                part, name, num, octave = voices[0]
-                mname = name + (num and ly.nums(num) or '')
-                if self.lyrEachDiff.isChecked():
-                    lyrName = mname + 'Verse'
-                else:
-                    lyrName = 'verse'
-                if maxLen == 1:
-                    # if all staves have only one voice, use \addlyrics...
-                    stub, ref = self.assignMusic(mname, octave)
-                    Identifier(ref, mus)
-                    if not (self.lyrAllSame.isChecked() and not toGo):
-                        for verse in stanzas:
-                            lyr.append((AddLyrics(s), lyrName, verse))
-                else:
-                    # otherwise create explicit Voice and Lyrics contexts.
-                    vname = name + str(num or '')
-                    v = Seqr(Voice(vname, parent=mus))
-                    stub, ref = self.assignMusic(mname, octave)
-                    Identifier(ref, v)
-                    if not (self.lyrAllSame.isChecked() and not toGo):
-                        for verse in stanzas:
-                            lyr.append((LyricsTo(vname, Lyrics(parent=choir)),
-                                lyrName, verse))
-                pianoReduction[part].append(ref)
-                if self.ambitus.isChecked():
-                    Line('\\consists "Ambitus_engraver"', s.getWith())
+            # Determine voice order (\voiceOne, \voiceTwo etc.)
+            if numVoices == 1:
+                order = (0,)
+            elif numVoices == 2:
+                order = 1, 2
+            elif staff in ('SSA', 'TTB'):
+                order = 1, 3, 2
+            elif staff in ('SAA', 'TBB'):
+                order = 1, 2, 4
+            elif staff in ('SSAA', 'TTBB'):
+                order = 1, 3, 2, 4
             else:
-                # There is more than one voice in the staff.
-                # Determine their order (\voiceOne, \voiceTwo etc.)
-                if len(staff) == 2:
-                    order = 1, 2
-                elif staff in ('SSA', 'TTB'):
-                    order = 1, 3, 2
-                elif staff in ('SAA', 'TBB'):
-                    order = 1, 2, 4
-                elif staff in ('SSAA', 'TTBB'):
-                    order = 1, 3, 2, 4
-                else:
-                    order = range(1, len(staff) + 1)
-                # What name would the staff get if we need to refer to it?
-                staffName, snum = staff, 1
-                # if a name (like 's' or 'sa') is already in use in this part,
-                # just add a number ('ss2' or 'sa2', etc.)
-                while staffName in staffNames:
-                    snum += 1
-                    staffName = staff + str(snum)
-                staffNames.append(staffName)
-                # We want the staff name (actually context-id) in lower case.
-                staffName = staffName.lower()
-                # Create the voices and their lyrics.
-                for (part, name, num, octave), vnum in zip(voices, order):
-                    mname = name + (num and ly.nums(num) or '')
-                    vname = name + str(num or '')
-                    v = Voice(vname, parent=mus)
-                    # Add ambitus to voice, move to the right if necessary
-                    if self.ambitus.isChecked():
-                        Line('\\consists "Ambitus_engraver"', v.getWith())
-                        if vnum > 1:
-                            Line("\\override Ambitus #'X-offset = #%s" %
-                                 ((vnum - 1) * 2.0), v.getWith())
-                    v = Seq(v)
-                    Text('\\voice' + ly.nums(vnum), v)
-                    stub, ref = self.assignMusic(mname, octave)
-                    Identifier(ref, v)
-                    pianoReduction[part].append(ref)
-                    if self.lyrAllSame.isChecked() and toGo and vnum == 1:
-                        lyrName = 'verse'
-                        above = False
-                    elif self.lyrEachSame.isChecked():
-                        lyrName = 'verse'
-                        above = vnum & 1
-                    elif self.lyrEachDiff.isChecked():
-                        lyrName = mname + 'Verse'
-                        above = vnum & 1
-                    else:
-                        continue
-                    # Create the lyrics. If they should be above the staff,
-                    # give the staff a suitable name, and use alignAboveContext
-                    # to align the Lyrics above the staff.
-                    if above:
-                        s.cid = Reference(staffName)
+                order = range(1, numVoices + 1)
+            
+            # What name would the staff get if we need to refer to it?
+            # If a name (like 's' or 'sa') is already in use in this part,
+            # just add a number ('ss2' or 'sa2', etc.)
+            staffCIDs[staff] += 1
+            cid = Reference(staff.lower() +
+                str(staffCIDs[staff] if staffCIDs[staff] > 1 else ""))
+            
+            # Create voices and their lyrics:
+            for (voice, num), voiceNum in zip(voices, order):
+                name = self.identifiers[voice]
+                if num:
+                    name += ly.nums(num)
+                stub, ref = self.assignMusic(name, self.octaves[voice])
+                lyrName = name + 'Verse' if lyrEachDiff else 'verse'
+            
+                # Use \addlyrics if all staves have exactly one voice.
+                if lyrEach and maxNumVoices == 1:
                     for verse in stanzas:
-                        l = Lyrics(parent=choir)
-                        if above:
-                            l.getWith()['alignAboveContext'] = s.cid
-                        lyr.append((LyricsTo(vname, l), lyrName, verse))
+                        lyrics[verse].append((AddLyrics(s), lyrName))
+                    Identifier(ref, staffMusic)
+                else:
+                    voiceName = self.identifiers[voice] + str(num or '')
+                    v = Voice(voiceName, parent=staffMusic)
+                    voiceMusic = Seqr(v)
+                    if voiceNum:
+                        Text('\\voice' + ly.nums(voiceNum), voiceMusic)
+                    Identifier(ref, voiceMusic)
+                    
+                    if stanzas and (lyrEach or (voiceNum <= 1 and
+                                    (stavesLeft or numStaves == 1))):
+                        # Create the lyrics. If they should be above the staff,
+                        # give the staff a suitable name, and use alignAbove-
+                        # Context to align the Lyrics above the staff.
+                        above = voiceNum & 1 if lyrEach else False
+                        if above and s.cid is None:
+                            s.cid = cid
 
+                        for verse in stanzas:
+                            l = Lyrics(parent=choir)
+                            if above:
+                                l.getWith()['alignAboveContext'] = cid
+                                setStaffAffinity(l, "DOWN")
+                            elif not lyrEach and stavesLeft:
+                                setStaffAffinity(l, "CENTER")
+                            lyrics[verse].append((LyricsTo(voiceName, l), lyrName))
+
+                # Add ambitus:
+                if self.ambitus.isChecked():
+                    ambitusContext = (s if numVoices == 1 else v).getWith()
+                    Line('\\consists "Ambitus_engraver"', ambitusContext)
+                    if voiceNum > 1:
+                        Line("\\override Ambitus #'X-offset = #{0}".format(
+                                 (voiceNum - 1) * 2.0), ambitusContext)
+            
+                pianoReduction[voice].append(ref)
+                rehearsalMidis.append((voice, num, ref, lyrName))
+            
         # Assign the lyrics, so their definitions come after the note defs.
+        # (These refs are used again below in the midi rehearsal routine.)
         refs = {}
-        for node, name, verse in lyr:
-            if (name, verse) not in refs:
-                refs[(name, verse)] = self.assignLyrics(name, verse)
-            Identifier(refs[(name, verse)], node)
-        self.nodes.append(p)
+        for verse in allStanzas:
+            for node, name in lyrics[verse]:
+                if (name, verse) not in refs:
+                    refs[(name, verse)] = self.assignLyrics(name, verse)
+                Identifier(refs[(name, verse)], node)
 
         # Create the piano reduction if desired
         if self.pianoReduction.isChecked():
@@ -1183,7 +1234,8 @@ class Choir(VocalPart):
 
             # Make the piano part somewhat smaller
             Line("fontSize = #-1", piano.getWith())
-            Line("\\override StaffSymbol #'staff-space = #(magstep -1)", piano.getWith())
+            Line("\\override StaffSymbol #'staff-space = #(magstep -1)",
+                piano.getWith())
             
             # Nice to add Mark engravers
             Line('\\consists "Mark_engraver"', rightStaff.getWith())
@@ -1193,6 +1245,64 @@ class Choir(VocalPart):
             if builder.midi:
                 Line('\\remove "Staff_performer"', rightStaff.getWith())
                 Line('\\remove "Staff_performer"', leftStaff.getWith())
+        
+        # Create MIDI files if desired
+        if self.rehearsalMidi.isChecked():
+            builder.book = True # force \book { } block
+
+            a, rehearsalMidi = self.assign('rehearsalMidi')
+            
+            func = SchemeList(a)
+            func.pre = '#(' # hack
+            Text('define-music-function', func)
+            Line('(parser location name midiInstrument lyrics) '
+                 '(string? string? ly:music?)', func)
+            choir = Sim(Command('unfoldRepeats', SchemeLily(func)))
+            
+            self.aftermath.append(Comment(i18n("Rehearsal MIDI files:")))
+            
+            for voice, num, ref, lyrName in rehearsalMidis:
+                # Append voice to the rehearsalMidi function
+                name = self.identifiers[voice] + str(num or '')
+                seq = Seq(Voice(name, parent=Staff(name, parent=choir)))
+                Text('s1*0\\f', seq) # add one dynamic
+                Identifier(ref, seq) # add the reference to the voice
+                
+                # Append score to the aftermath (stuff put below the main score)
+                if self.num:
+                    suffix = "choir{0}-{1}".format(self.num, name)
+                else:
+                    suffix = name
+                self.aftermath.append(
+                    Line('#(define output-suffix "{0}")'.format(suffix)))
+                book = Book()
+                self.aftermath.append(book)
+                self.aftermath.append(BlankLine())
+                score = Score(book)
+                
+                # TODO: make configurable
+                if voice in ('SA'):
+                    midiInstrument = "soprano sax"
+                else:
+                    midiInstrument = "tenor sax"
+
+                cmd = Command(rehearsalMidi, score)
+                QuotedString(name, cmd)
+                QuotedString(midiInstrument, cmd)
+                Identifier(refs[(lyrName, stanzas[0])], cmd)
+                Midi(score)
+            
+            Text("\\context Staff = $name", choir)
+            seq = Seq(choir)
+            Line("\\set Score.midiMinimumVolume = #0.5", seq)
+            Line("\\set Score.midiMaximumVolume = #0.5", seq)
+            Line("\\set Score.tempoWholesPerMinute = #" + builder.getMidiTempo(), seq)
+            Line("\\set Staff.midiMinimumVolume = #0.8", seq)
+            Line("\\set Staff.midiMaximumVolume = #1.0", seq)
+            Line("\\set Staff.midiInstrument = $midiInstrument", seq)
+            lyr = Lyrics(parent=choir)
+            lyr.getWith()['alignBelowContext'] = Text('$name')
+            Text("\\lyricsto $name $lyrics", lyr)
 
 
 class Piano(KeyboardPart):
@@ -1296,8 +1406,8 @@ class Drums(Part):
         if self.drumVoices.value() > 1:
             for i in range(1, self.drumVoices.value()+1):
                 q = Seq(DrumVoice(parent=s))
-                Text('\\voice%s' % ly.nums(i), q)
-                ref = self.assignDrums('drum%s' % ly.nums(i))
+                Text('\\voice' + ly.nums(i), q)
+                ref = self.assignDrums('drum' + ly.nums(i))
                 Identifier(ref, q)
         else:
             ref = self.assignDrums('drum')
@@ -1306,13 +1416,13 @@ class Drums(Part):
         i = self.drumStyle.currentIndex()
         if i > 0:
             v = ('drums', 'timbales', 'congas', 'bongos', 'percussion')[i]
-            p.getWith()['drumStyleTable'] = Scheme('%s-style' % v)
+            p.getWith()['drumStyleTable'] = Scheme(v + '-style')
             v = (5, 2, 2, 2, 1)[i]
-            Line("\\override StaffSymbol #'line-count = #%i" % v, p.getWith())
+            Line("\\override StaffSymbol #'line-count = #{0}".format(v), p.getWith())
         if self.drumStems.isChecked():
             Line("\\override Stem #'stencil = ##f", p.getWith())
-            Line("\\override Stem #'length = #3  %% %s"
-                % i18n("keep some distance."), p.getWith())
+            Line("\\override Stem #'length = #3  % " + i18n("keep some distance."),
+                p.getWith())
         self.nodes.append(p)
 
     def widgets(self, layout):
