@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 Preview dialog for the Score Wizard (scorewiz/__init__.py).
 In separate file to ease maintenance.
@@ -60,16 +62,16 @@ class PreviewDialog(LilyPreviewDialog):
         
         def addItems(stub, generator):
             for dur, dots in durs:
-                node = generator.next()
+                node = next(generator)
                 node.append(ly.dom.Duration(dur, dots))
                 stub.append(node)
             
-        lyrics = lyricsGen(len(durs)).next
+        lyrics = lyricsGen(len(durs))
         # iter over all the Assignments to add example notes etc.
         for a in doc.findChildren(ly.dom.Assignment, 1):
             stub = a[-1]
             if isinstance(stub, ly.dom.LyricMode):
-                stub.append(lyrics())
+                stub.append(next(lyrics))
             elif isinstance(stub, ly.dom.Relative):
                 addItems(stub[-1], pitchGen(keysig))
             elif isinstance(stub, ly.dom.ChordMode):
@@ -108,7 +110,7 @@ def lyricsGen(length):
 def figureGen():
     while True:
         for i in 5, 6, 3, 8, 7:
-            for s in "<%s>" % i, "\\skip", "\\skip":
+            for s in "<{0}>".format(i), "\\skip", "\\skip":
                 yield ly.dom.TextDur(s)
             
 def drumGen():
