@@ -17,30 +17,36 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """ Functions to run convert-ly and insert the LilyPond version """
 
 from subprocess import Popen, PIPE
 import ly.tokenize, ly.version
 
-from PyQt4.QtCore import QVariant
 from PyKDE4.kdecore import i18n, KGlobal
 from PyKDE4.kdeui import KMessageBox
 from PyKDE4.ktexteditor import KTextEditor
 
+from frescobaldi_app.mainapp import lilyPondCommand, convertLyCommand
+
 def defaultVersion():
     """
-    Returns the LilyPond version number string according to the user's
-    preference: the version of the currently installed LilyPond, the
-    version of the last rule in convert-ly, or a custom version.
+    Returns the LilyPond version according to the user's preference:
+    the version of the currently installed LilyPond, the version of the last
+    rule in convert-ly, or a custom version.
     """
+    lilypond = ly.version.LilyPondInstance(lilyPondCommand())
+    
     prefs = config("preferences")
-    pver = prefs.readEntry("default version", QVariant("lilypond")).toString()
+    pver = prefs.readEntry("default version", "lilypond")
+    
     version = ''
     if pver == "custom":
-        version = unicode(prefs.readEntry("custom version", QVariant("")).toString())
+        version = ly.version.Version.fromString(prefs.readEntry("custom version", ""))
     elif pver == "convert-ly":
-        version = ly.version.ConvertLyLastRuleVersion(command("convert-ly")).versionString
-    return version or ly.version.LilyPondVersion(command("lilypond")).versionString
+        version = lilypond.lastConvertLyRuleVersion()
+    return version or lilypond.version()
 
 def insertVersion(mainwin):
     """
@@ -58,7 +64,7 @@ def insertVersion(mainwin):
     else:
         version = defaultVersion()
         if version:
-            doc.doc.insertLine(0, '\\version "%s"' % version)
+            doc.doc.insertLine(0, '\\version "{0}"'.format(version))
         else:
             KMessageBox.sorry(mainwin,
                 i18n("Can't determine the version of LilyPond. "
@@ -70,8 +76,8 @@ def convertLy(mainwin):
     """
     doc = mainwin.currentDocument()
     text = doc.text()
-    docVersion = ly.version.getVersion(text) #or ly.guess.version(text)
-    lilyVersion = ly.version.LilyPondVersion(command("lilypond")).versionTuple
+    docVersion = ly.version.getVersion(text)
+    lilyVersion = ly.version.LilyPondInstance(lilyPondCommand()).version()
     
     if not docVersion:
         KMessageBox.sorry(mainwin, i18n(
@@ -90,12 +96,11 @@ def convertLy(mainwin):
         # read from stdin.
         try:
             out, err = Popen(
-                (command("convert-ly"), "-f", "%d.%d.%d" % docVersion, "-"),
+                (convertLyCommand(), "-f", str(docVersion), "-"),
                 stdin=PIPE, stdout=PIPE, stderr=PIPE
                 ).communicate(text.encode('utf8'))
             if out:
-                doc.setText(u"%s\n\n%%{\n%s\n%%}\n" %
-                    (out.decode('utf8'), err.decode('utf8')))
+                doc.setText("{0}\n\n%{{\n{1}\n%}}\n".format(out.decode('utf8'), err.decode('utf8')))
                 KMessageBox.information(mainwin, i18n(
                  "The document has been processed with convert-ly. You'll find "
                  "the messages of convert-ly in a comment block at the end. "
@@ -106,7 +111,7 @@ def convertLy(mainwin):
                  "The document has been processed with convert-ly, but "
                  "remained unchanged. This is the message given by "
                  "convert-ly: %1", msg))
-        except OSError, e:
+        except OSError as e:
             msg = unicode(e)
             KMessageBox.error(mainwin, i18n("Could not start convert-ly: %1", msg))
 
@@ -114,5 +119,3 @@ def convertLy(mainwin):
 def config(group):
     return KGlobal.config().group(group)
 
-def command(cmd):
-    return unicode(config("commands").readEntry(cmd, QVariant(cmd)).toString())
