@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 Some special widgets
 """
@@ -24,13 +26,11 @@ Some special widgets
 import os
 from time import time
 
-from PyQt4.QtCore import (
-    QObject, QProcess, QRegExp, QString, QTimeLine, Qt, SIGNAL)
+from PyQt4.QtCore import QProcess, QRegExp, Qt
 from PyQt4.QtGui import (
-    QComboBox, QLabel, QLineEdit, QPainter, QPixmap, QPushButton, QSlider,
-    QSpinBox, QToolButton, QRegExpValidator, QWidget)
+    QComboBox, QLabel, QPushButton, QSlider, QSpinBox, QRegExpValidator)
 from PyKDE4.kdecore import i18n, KProcess
-from PyKDE4.kdeui import KApplication, KDialog, KLineEdit, KVBox
+from PyKDE4.kdeui import KDialog, KLineEdit, KVBox
 
 from frescobaldi_app.mainapp import SymbolManager
 
@@ -48,7 +48,7 @@ class TapButton(QPushButton):
         def tap():
             self.tapTime, t = time(), self.tapTime
             callback(int(60.0 / (self.tapTime - t)))
-        QObject.connect(self, SIGNAL("pressed()"), tap)
+        self.pressed.connect(tap)
         self.setToolTip(i18n("Click this button a few times to set the tempo."))
 
 
@@ -62,7 +62,7 @@ class ProcessButtonBase(object):
         super(ProcessButtonBase, self).__init__(*args)
         self.setCheckable(True)
         self._p = KProcess()
-        QObject.connect(self, SIGNAL("clicked()"), self.slotClicked)
+        self.clicked.connect(self.slotClicked)
     
     def slotClicked(self):
         if self.isRunning():
@@ -78,13 +78,13 @@ class ProcessButtonBase(object):
         """ Starts the process, calling the initializeProcess method first."""
         p = KProcess()    # create a new one (FIXME: really needed?)
         self._p = p
-        QObject.connect(self, SIGNAL("destroyed()"), p.terminate)
-        QObject.connect(p, SIGNAL("finished(int, QProcess::ExitStatus)"), self.slotFinished)
-        QObject.connect(p, SIGNAL("error(QProcess::ProcessError)"), self.slotError)
-        QObject.connect(p, SIGNAL("started()"), self.slotStarted)
-        QObject.connect(p, SIGNAL("readyRead()"), self.slotReadyRead)
-        QObject.connect(p, SIGNAL("readyReadStandardError()"), self.slotReadyReadStandardError)
-        QObject.connect(p, SIGNAL("readyReadStandardOutput()"), self.slotReadyReadStandardOutput)
+        self.destroyed.connect(p.terminate)
+        p.finished.connect(self.slotFinished)
+        p.error.connect(self.slotError)
+        p.started.connect(self.slotStarted)
+        p.readyRead.connect(self.slotReadyRead)
+        p.readyReadStandardError.connect(self.slotReadyReadStandardError)
+        p.readyReadStandardOutput.connect(self.slotReadyReadStandardOutput)
         self.initializeProcess(p)
         p.start()
         
@@ -187,10 +187,8 @@ class TempoControl(object):
         self.tapButton = TapButton(None, tap)
         
         # setup signals
-        QObject.connect(self.slider, SIGNAL("valueChanged(int)"),
-            self.spinbox.setValue)
-        QObject.connect(self.spinbox, SIGNAL("valueChanged(int)"),
-            self.slider.setValue)
+        self.slider.valueChanged.connect(self.spinbox.setValue)
+        self.spinbox.valueChanged.connect(self.slider.setValue)
         
         self.slider.setMinimumWidth(200)
         self.setTempo(100) # default
@@ -202,78 +200,6 @@ class TempoControl(object):
         self.spinbox.setValue(value)
 
 
-class ExecLineEdit(QLineEdit):
-    """
-    A QLineEdit to enter a filename or path.
-    The background changes to red if the entered path is not an
-    executable command.
-    """
-    def __init__(self, *args):
-        QLineEdit.__init__(self, *args)
-        QObject.connect(self, SIGNAL("textChanged(const QString&)"),
-            self._checkexec)
-
-    def _get(self, filename):
-        return unicode(filename)
-
-    def _checkexec(self, filename):
-        if not findexe(self._get(filename)):
-            self.setStyleSheet("QLineEdit { background-color: #FAA }")
-        else:
-            self.setStyleSheet("")
-
-
-class ExecArgsLineEdit(ExecLineEdit):
-    """
-    An ExecLineEdit that allows arguments in the command string.
-    """
-    def _get(self, filename):
-        if filename:
-            return unicode(filename).split()[0]
-        else:
-            return ''
-
-
-class StackFader(QWidget):
-    """
-    A widget that creates smooth transitions in a QStackedWidget.
-    """
-    def __init__(self, stackedWidget):
-        QWidget.__init__(self, stackedWidget)
-        self.curIndex = None
-        self.timeline = QTimeLine()
-        self.timeline.setDuration(333)
-        QObject.connect(self.timeline, SIGNAL("finished()"), self.hide)
-        QObject.connect(self.timeline, SIGNAL("valueChanged(qreal)"), self.animate)
-        QObject.connect(stackedWidget, SIGNAL("currentChanged(int)"), self.start)
-        self.hide()
-    
-    def start(self, index):
-        if self.curIndex is not None:
-            stack = self.parent()
-            old, new = stack.widget(self.curIndex), stack.widget(index)
-            if old and new:
-                self.old_pixmap = QPixmap(new.size())
-                old.render(self.old_pixmap)
-                self.pixmap_opacity = 1.0
-                self.resize(new.size())
-                self.timeline.start()
-                self.raise_()
-                self.show()
-        self.curIndex = index
-        
-    def paintEvent(self, ev):
-        painter = QPainter()
-        painter.begin(self)
-        painter.setOpacity(self.pixmap_opacity)
-        painter.drawPixmap(0, 0, self.old_pixmap)
-        painter.end()
-        
-    def animate(self, value):
-        self.pixmap_opacity = 1.0 - value
-        self.repaint()
-
-
 class ClefSelector(SymbolManager, QComboBox):
     """
     A ComboBox to select a clef.
@@ -300,7 +226,7 @@ class ClefSelector(SymbolManager, QComboBox):
             self.clefs.insert(0, ('', i18n("No Clef")))
         self.addItems([title for name, title in self.clefs])
         for index, (name, title) in enumerate(self.clefs):
-            self.addItemSymbol(self, index, 'clef_%s' % (name or 'none'))
+            self.addItemSymbol(self, index, 'clef_' + (name or 'none'))
     
     def clef(self):
         """
@@ -310,52 +236,3 @@ class ClefSelector(SymbolManager, QComboBox):
         return self.clefs[self.currentIndex()][0]
     
 
-# some handy "static" functions
-def promptText(parent, message, title = None, text="", rx=None, help=None):
-    """
-    Prompts for a text. Returns None on cancel, otherwise the input string.
-    rx = if given, regexp string that the input must validate against
-    help = if given, the docbook id in the help menu handbook.
-    """
-    d = KDialog(parent)
-    buttons = KDialog.Ok | KDialog.Cancel
-    if help:
-        buttons |= KDialog.Help
-        d.setHelp(help)
-    d.setButtons(KDialog.ButtonCode(buttons))
-    if title:
-        d.setCaption(title)
-    v = KVBox()
-    v.setSpacing(4)
-    d.setMainWidget(v)
-    QLabel(message, v)
-    edit = KLineEdit(v)
-    if rx:
-        edit.setValidator(QRegExpValidator(QRegExp(rx), edit))
-    edit.setText(text)
-    d.show()
-    edit.setFocus()
-    if d.exec_():
-        return unicode(edit.text())
-
-
-# utility functions used by above classes:
-def isexe(path):
-    """
-    Return path if it is an executable file, otherwise False
-    """
-    return os.access(path, os.X_OK) and path
-
-def findexe(filename):
-    """
-    Look up a filename in the system PATH, and return the full
-    path if it can be found. If the path is absolute, return it
-    unless it is not an executable file.
-    """
-    if os.path.isabs(os.path.expanduser(filename)):
-        return isexe(os.path.expanduser(filename))
-    for p in os.environ.get("PATH", os.defpath).split(os.pathsep):
-        if isexe(os.path.join(p, filename)):
-            return os.path.join(p, filename)
-    return False
-
