@@ -17,9 +17,10 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 
 import dbus
-from PyQt4.QtCore import QString
 from PyKDE4.kdecore import KUrl
 
 # interface for our object types (MainApp and Document)
@@ -36,8 +37,8 @@ def runningApp(servicePrefix, pid=None):
     bus = dbus.SessionBus(private=True)
     names = [n for n in bus.list_names() if n.startswith(servicePrefix)]
     if names:
-        if pid and "%s%s" % (servicePrefix, pid) in names:
-            name = "%s%s" % (servicePrefix, pid)
+        if pid and servicePrefix + pid in names:
+            name = servicePrefix + pid
         else:
             name = names[0]
         print "Found running instance:", name
@@ -77,13 +78,11 @@ class Proxy(object):
             meth = getattr(self.iface, attr)
             if callable(meth):
                 def proxy_func(*args):
-                    # convert args from QString or KUrl to unicode
+                    # convert args from KUrl to unicode
                     args = list(args)
                     for i in range(len(args)):
-                        if isinstance(args[i], QString):
-                            args[i] = unicode(args[i])
-                        elif isinstance(args[i], KUrl):
-                            args[i] = unicode(args[i].url())
+                        if isinstance(args[i], KUrl):
+                            args[i] = args[i].url()
                     # call the method
                     res = meth(*args)
                     # Return same proxy if the returned object is a reference
