@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 Exception dialog for unhandled Python exceptions
 (which are bugs in our program).
@@ -24,7 +26,6 @@ Exception dialog for unhandled Python exceptions
 
 import traceback
 
-from PyQt4.QtCore import QObject, SIGNAL
 from PyQt4.QtGui import QLabel, QTextBrowser, QTextCursor, QVBoxLayout
 from PyKDE4.kdecore import KGlobal, KToolInvocation, i18n
 from PyKDE4.kdeui import KDialog, KIcon
@@ -38,8 +39,8 @@ class ExceptionDialog(KDialog):
         self.tbfull = ''.join(traceback.format_exception(exctype, excvalue, exctb))
         
         l = QVBoxLayout(self.mainWidget())
-        l.addWidget(QLabel("<b>%s</b>" % i18n(
-            "An internal error has occurred:")))
+        l.addWidget(QLabel(u"<b>{0}</b>".format(i18n(
+            "An internal error has occurred:"))))
         b = QTextBrowser()
         l.addWidget(b)
         self.setCaption(i18n("Internal Error"))
@@ -49,7 +50,7 @@ class ExceptionDialog(KDialog):
             KDialog.User1 | KDialog.Close))
         self.setButtonIcon(KDialog.User1, KIcon("tools-report-bug"))
         self.setButtonText(KDialog.User1, i18n("Email Bug Report..."))
-        QObject.connect(self, SIGNAL("user1Clicked()"), self.reportBug)
+        self.user1Clicked.connect(self.reportBug)
         self.resize(600,300)
         self.exec_()
 
@@ -57,9 +58,9 @@ class ExceptionDialog(KDialog):
         self.accept()
         
         about = KGlobal.mainComponent().aboutData()
-        subject = "[%s %s] %s" % (
+        subject = u"[{0} {1}] {2}".format(
             about.programName(), about.version(), self.tbshort)
-        body = "%s %s\n\n%s\n%s\n\n" % (
+        body = u"{0} {1}\n\n{2}\n{3}\n\n".format(
             about.programName(), about.version(), self.tbfull,
             i18n("Optionally describe what you were doing below:"))
         to = about.bugAddress()
