@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """ Basic LilyPond information and utility functions """
 
 
@@ -36,21 +38,6 @@ class QuarterToneAlterationNotAvailable(Exception):
     pass
 
 
-
-
-def lazy(func):
-    """
-    A decorator that only performs the function call the first time,
-    caches the return value, and returns that next time.
-    The argments tuple should be hashable.
-    """
-    cache = {}
-    def loader(*args):
-        if args not in cache:
-            cache[args] = func(*args)
-        return cache[args]
-    return loader
-
 _nums = (
     'Zero', 'One', 'Two', 'Three', 'Four', 'Five', 'Six', 'Seven', 'Eight',
     'Nine', 'Ten', 'Eleven', 'Twelve', 'Thirteen', 'Fourteen', 'Fifteen',
