@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 r"""
 LilyPond DOM
 
@@ -35,7 +37,7 @@ Note: elements keep a weak reference to their parent.
 """
 
 import re, weakref
-from rational import Rational
+from fractions import Fraction
 
 import ly.pitch, ly.duration
 
@@ -315,13 +317,13 @@ class Printer(object):
         
     def quoteString(self, text):
         if self.typographicalQuotes:
-            text = re.sub(r'"(.*?)"', u'\u201C\\1\u201D', text)
-            text = re.sub(r"'(.*?)'", u'\u2018\\1\u2019', text)
-            text = text.replace("'", u'\u2018')
+            text = re.sub(r'"(.*?)"', '\u201C\\1\u201D', text)
+            text = re.sub(r"'(.*?)'", '\u2018\\1\u2019', text)
+            text = text.replace("'", '\u2018')
         # escape regular double quotes
         text = text.replace('"', '\\"')
         # quote the string
-        return '"%s"' % text
+        return '"{0}"'.format(text)
 
     def indentGen(self, node, startIndent = 0):
         """
@@ -366,7 +368,7 @@ class Named(object):
     name = ""
     
     def ly(self, printer):
-        return "\\%s %s" % (unicode(self.name), super(Named, self).ly(printer))
+        return "\\{0} {1}".format(unicode(self.name), super(Named, self).ly(printer))
         
         
 class HandleVars(object):
@@ -583,10 +585,8 @@ class BlockComment(Comment):
         
     def ly(self, printer):
         text = self.text.replace('%}', '')
-        if '\n' in text:
-            return "%%{\n%s\n%%}" % text
-        else:
-            return "%%{ %s %%}" % text
+        f = "%{{\n{0}\n%}}" if '\n' in text else "%{{ {0} %}}"
+        return f.format(text)
             
 
 class QuotedString(Text):
@@ -611,19 +611,19 @@ class Scheme(Text):
     isAtom = True
     
     def ly(self, printer):
-        return '#%s' % self.text
+        return '#' + self.text
 
 
 class Version(Line):
     """ a LilyPond version instruction """
     def ly(self, printer):
-        return r'\version "%s"' % self.text
+        return r'\version "{0}"'.format(self.text)
 
 
 class Include(Line):
     """ a LilyPond \\include statement """
     def ly(self, printer):
-        return r'\include "%s"' % self.text
+        return r'\include "{0}"'.format(self.text)
 
 
 class Assignment(Container):
@@ -652,7 +652,7 @@ class Assignment(Container):
             return self[0]
 
     def ly(self, printer):
-        return "%s = %s" % (
+        return "{0} = {1}".format(
             unicode(self.name), super(Assignment, self).ly(printer))
 
 
@@ -671,7 +671,7 @@ class Identifier(Leaf):
         self.name = name
         
     def ly(self, printer):
-        return "\\%s" % unicode(self.name)
+        return "\\" + unicode(self.name)
 
 
 class Statement(Named, Container):
@@ -733,6 +733,19 @@ class Seqr(Seq): may_remove_brackets = True
 class Simr(Sim): may_remove_brackets = True
     
 
+class SchemeLily(Enclosed):
+    """ A LilyPond expression between #{ #} (inside scheme) """
+    pre, post = "#{", "#}"
+
+
+class SchemeList(Enclosed):
+    """ A list of items enclosed in parentheses """
+    pre, post = "(", ")"
+    
+    def ly(self, printer):
+        return self.pre + super(Enclosed, self).ly(printer) + self.post
+
+
 class StatementEnclosed(Named, Enclosed):
     """
     Base class for LilyPond commands that have a single bracket-enclosed
@@ -786,7 +799,7 @@ class ContextName(Text):
     Used to print a context name, like \\Score.
     """
     def ly(self, printer):
-        return "\\%s" % self.text
+        return "\\" + self.text
 
 
 class Context(HandleVars, Section):
@@ -846,6 +859,7 @@ class CueVoice(ContextType): pass
 class Devnull(ContextType): pass
 class DrumStaff(ContextType): pass
 class DrumVoice(ContextType): pass
+class Dynamics(ContextType): pass
 class FiguredBass(ContextType): pass
 class FretBoards(ContextType): pass
 class Global(ContextType): pass
@@ -903,10 +917,10 @@ class ContextProperty(Leaf):
             # In \lyrics or \lyricmode: put spaces around dot.
             p = self.findParent(InputMode)
             if p and isinstance(p, LyricMode):
-                f = '%s . %s'
+                f = '{0} . {1}'
             else:
-                f = '%s.%s'
-            return f % (self.context, self.prop)
+                f = '{0}.{1}'
+            return f.format(self.context, self.prop)
         else:
             return self.prop
 
@@ -945,7 +959,7 @@ class LyricsTo(LyricMode):
         self.cid = cid
     
     def ly(self, printer):
-        res = ["\\%s" % self.name]
+        res = ["\\" + self.name]
         res.append(printer.quoteString(unicode(self.cid)))
         res.append(super(Named, self).ly(printer))
         return " ".join(res)
@@ -957,14 +971,14 @@ class Pitch(Leaf):
     octave is specified by an integer, zero for the octave containing middle C.
     note is a number from 0 to 6, with 0 corresponding to pitch C and 6
     corresponding to pitch B.
-    alter is the number of whole tones for alteration (can be int or Rational)
+    alter is the number of whole tones for alteration (can be int or Fraction)
     """
 
     def __init__(self, octave=0, note=0, alter=0, parent=None):
         super(Pitch, self).__init__(parent)
         self.octave = octave
         self.note = note
-        self.alter = Rational(alter)
+        self.alter = Fraction(alter)
 
     def ly(self, printer):
         """
@@ -983,18 +997,18 @@ class Duration(Leaf):
     A duration with duration (in logarithmical form): (-2 ... 8),
     where -2 = \\longa, -1 = \\breve, 0 = 1, 1 = 2, 2 = 4, 3 = 8, 4 = 16, etc,
     dots (number of dots),
-    factor (Rational giving the scaling of the duration).
+    factor (Fraction giving the scaling of the duration).
     """
     def __init__(self, dur, dots=0, factor=1, parent=None):
         super(Duration, self).__init__(parent)
         self.dur = dur # log
         self.dots = dots
-        self.factor = Rational(factor)
+        self.factor = Fraction(factor)
 
     def ly(self, printer):
         s = ly.duration.durations[self.dur + 3] + '.' * self.dots
         if self.factor != 1:
-            s += '*%s' % str(self.factor)
+            s += '*' + str(self.factor)
         return s
 
 
@@ -1008,7 +1022,7 @@ class Chord(Container):
         if len(pitches) == 1:
             s = pitches[0].ly(printer)
         else:
-            s = "<%s>" % ' '.join(p.ly(printer) for p in pitches)
+            s = "<{0}>".format(' '.join(p.ly(printer) for p in pitches))
         duration = self.findChild(Duration, 1)
         if duration:
             s += duration.ly(printer)
@@ -1044,12 +1058,12 @@ class KeySignature(Leaf):
     def __init__(self, note=0, alter=0, mode="major", parent=None):
         super(KeySignature, self).__init__(parent)
         self.note = note
-        self.alter = Rational(alter)
+        self.alter = Fraction(alter)
         self.mode = mode
 
     def ly(self, printer):
         pitch = ly.pitch.pitchWriter[printer.language](self.note, self.alter)
-        return "\\key %s \\%s" % (pitch, self.mode)
+        return "\\key {0} \\{1}".format(pitch, self.mode)
 
 
 class TimeSignature(Leaf):
@@ -1062,7 +1076,7 @@ class TimeSignature(Leaf):
         self.beat = beat
 
     def ly(self, printer):
-        return "\\time %i/%i" % (self.num, self.beat)
+        return "\\time {0}/{1}".format(self.num, self.beat)
 
 
 class Partial(Named, Duration):
@@ -1084,7 +1098,7 @@ class Tempo(Leaf):
         self.value = value
         
     def ly(self, printer):
-        return "\\tempo %s=%s" % (self.duration, self.value)
+        return "\\tempo {0}={1}".format(self.duration, self.value)
         
         
 class Clef(Leaf):
@@ -1096,8 +1110,8 @@ class Clef(Leaf):
         self.clef = clef
 
     def ly(self, printer):
-        f = self.clef.isalpha() and '%s' or '"%s"'
-        return "\\clef %s" % f % self.clef
+        clef = self.clef if self.clef.isalpha() else '"{0}"'.format(self.clef)
+        return "\\clef " + clef
 
 
 class VoiceSeparator(Leaf):
