@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """ LilyPond information and logic concerning durations """
 
 import ly.rx
@@ -111,10 +113,10 @@ def applyRhythm(text, rhythm):
             for i in durs:
                 yield i != old and i or ''
                 old = i
-    nextdur = durgen().next
+    durations = durgen()
     def repl(m):
         if m.group('chord'):
-            return m.group('chord') + nextdur()
+            return m.group('chord') + next(durations)
         return m.group()
     return ly.rx.chord_rest.sub(repl, text)
 
