@@ -17,11 +17,13 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """ Routines dealing with LilyPond fonts """
 
 import os, ly, xml.dom.minidom
 
-class SvgFont(object):
+class SvgFontInfo(object):
     """
     Can load a SVG font and provide information about the glyphs and
     their unicode values.
@@ -37,16 +39,8 @@ class SvgFont(object):
         doc.unlink()
         
     def glyph(self, glyphName):
-        return self.name2unicode.get(glyphName, u'')
+        return self.name2unicode.get(glyphName, '')
         
     def glyphs(self):
         return self.name2unicode.keys()
 
-@ly.lazy
-def emmentaler20(datadir):
-    """
-    Return the emmentaler-20 font, loaded and cached
-    """
-    font = os.path.join(datadir, "fonts", "svg", "emmentaler-20.svg")
-    return os.path.exists(font) and SvgFont(font) or None
-
