@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 Indent LilyPond input.
 
@@ -240,7 +242,7 @@ if __name__ == '__main__':
     options, args = op.parse_args()
     # TODO: encoding
     # TODO: error handling
-    infile = args and file(args[0]) or sys.stdin
+    infile = args and open(args[0]) or sys.stdin
     text = infile.read()
     text = indent(text,
         start=options.start_indent,
