@@ -17,8 +17,10 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 import re
-from rational import Rational
+from fractions import Fraction
 
 import ly
 
@@ -105,7 +107,7 @@ class Transposer(object):
     The scale is a list with the pitch height of the unaltered step (0 .. 6).
     The default scale is the normal scale: C, D, E, F, G, A, B.
     """
-    scale = (0, 1, 2, Rational(5, 2), Rational(7, 2), Rational(9, 2), Rational(11, 2))
+    scale = (0, 1, 2, Fraction(5, 2), Fraction(7, 2), Fraction(9, 2), Fraction(11, 2))
         
     def __init__(self, fromPitch, toPitch, scale = None):
         if scale is not None:
@@ -158,10 +160,8 @@ class PitchReader(object):
         self.names = list(names)
         self.accs = list(accs)
         self.replacements = replacements
-        
-        rx = "(%s)" % "|".join(names)
-        rx += "(%s)?$" % "|".join(acc for acc in accs if acc)
-        self.rx = re.compile(rx)
+        self.rx = re.compile("({0})({1})?$".format("|".join(names),
+            "|".join(acc for acc in accs if acc)))
 
     def __call__(self, text):
         for s, r in self.replacements:
@@ -172,7 +172,7 @@ class PitchReader(object):
             if m:
                 note = self.names.index(m.group(1))
                 if m.group(2):
-                    alter = Rational(self.accs.index(m.group(2)) - 4, 4)
+                    alter = Fraction(self.accs.index(m.group(2)) - 4, 4)
                 else:
                     alter = 0
                 return note, alter
