@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 This module defines a Tokenizer class to parse and tokenize LilyPond text.
 
@@ -89,7 +91,7 @@ def _make_re(classes):
     types. Reads the rx class attribute of the given classes.
     """
     return re.compile(
-        "|".join("(?P<%s>%s)" % (cls.__name__, cls.rx) for cls in classes),
+        "|".join("(?P<{0}>{1})".format(cls.__name__, cls.rx) for cls in classes),
         re.DOTALL)
 
 
@@ -375,7 +377,7 @@ class Tokenizer(object):
         pass
     
     class IncludeLanguageFile(IncludeFile):
-        rx = r'"(%s)\.ly"' % '|'.join(ly.pitch.pitchInfo.keys())
+        rx = r'"({0})\.ly"'.format('|'.join(ly.pitch.pitchInfo.keys()))
         def __init__(self, matchObj, tokenizer):
             tokenizer.language = self[1:-4]
             tokenizer.endArgument()
@@ -722,11 +724,11 @@ class Cursor(object):
             self.anchorLine = self.line
             self.anchorColumn = self.column + len(text)
 
-    def startEditing(self):
+    def __enter__(self):
         """ Called before edits are made. """
         pass
     
-    def endEditing(self):
+    def __exit__(self, *args):
         """ Called after edits have been done. """
         pass
     
@@ -806,21 +808,20 @@ class ChangeList(object):
 
     def applyToCursor(self, cursor):
         index = 0
-        cursor.startEditing()
-        for pos, end, text in self.changes():
-            if pos > index:
-                cursor.walk(self._text[index:pos])
-            if end > pos:
-                cursor.anchor(self._text[pos:end])
-                if text:
-                    cursor.replaceText(text)
-                    cursor.walk(text)
+        with cursor:
+            for pos, end, text in self.changes():
+                if pos > index:
+                    cursor.walk(self._text[index:pos])
+                if end > pos:
+                    cursor.anchor(self._text[pos:end])
+                    if text:
+                        cursor.replaceText(text)
+                        cursor.walk(text)
+                    else:
+                        cursor.removeText()
                 else:
-                    cursor.removeText()
-            else:
-                cursor.insertText(text)
-                cursor.walk(text)
-            index = end
-        cursor.endEditing()
+                    cursor.insertText(text)
+                    cursor.walk(text)
+                index = end
 
 
