@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """
 LilyPond reserved words for auto completion, and some regexps
 """
@@ -204,6 +206,7 @@ musiccommands = (
     'glissando',
     'grace',
     'graceSettings',
+    'halfopen',
     'harmonic',
     'hideNotes',
     'hideStaffSwitch',
@@ -359,6 +362,7 @@ musiccommands = (
     'slurSolid',
     'slurUp',
     'small',
+    'snappizzicato',
     'sostenutoOff',
     'sostenutoOn',
     'sp',
@@ -481,6 +485,7 @@ markupcommands_nargs = (
 (
     'doubleflat',
     'doublesharp',
+    'eyeglasses',
     'flat',
     'natural',
     'null',
@@ -562,6 +567,7 @@ markupcommands_nargs = (
     'underline',
     'upright',
     'vcenter',
+    'vspace',
     'verbatim-file',
     'whiteout',
     'wordwrap',
@@ -630,6 +636,7 @@ contexts = (
     'Devnull',
     'DrumStaff',
     'DrumVoice',
+    'Dynamics',
     'FiguredBass',
     'FretBoards',
     'Global',
@@ -683,7 +690,7 @@ engravers = (
     'Dynamic_align_engraver',
     'Dynamic_engraver',
     'Dynamic_performer',
-    'Engraver',
+    'Episema_engraver',
     'Extender_engraver',
     'Figured_bass_engraver',
     'Figured_bass_position_engraver',
@@ -723,7 +730,6 @@ engravers = (
     'Note_name_engraver',
     'Note_performer',
     'Note_spacing_engraver',
-    'Note_swallow_translator',
     'Ottava_spanner_engraver',
     'Output_property_engraver',
     'Page_turn_engraver',
@@ -741,13 +747,12 @@ engravers = (
     'Repeat_tie_engraver',
     'Rest_collision_engraver',
     'Rest_engraver',
-    'Rest_swallow_translator',
     'Rhythmic_column_engraver',
+    'Scheme_engraver',
     'Script_column_engraver',
     'Script_engraver',
     'Script_row_engraver',
     'Separating_line_group_engraver',
-    'Skip_event_swallow_translator',
     'Slash_repeat_engraver',
     'Slur_engraver',
     'Slur_performer',
@@ -776,7 +781,6 @@ engravers = (
     'Time_signature_engraver',
     'Time_signature_performer',
     'Timing_translator',
-    'Translator',
     'Trill_spanner_engraver',
     'Tuplet_engraver',
     'Tweak_engraver',
@@ -840,7 +844,7 @@ midi_instruments = (
     'contrabass',
     'tremolo strings',
     'pizzicato strings',
-    'orchestral strings', # should be: 'orchestral harp' (done in LP 2.13)
+    'orchestral harp', # till LilyPond 2.12 was this erroneously called: 'orchestral strings'
     'timpani',
     # (49-56 ensemble)
     'string ensemble 1',
@@ -998,6 +1002,7 @@ grobs = (
     'DynamicLineSpanner',
     'DynamicText',
     'DynamicTextSpanner',
+    'Episema',
     'Fingering',
     'FretBoard',
     'Glissando',
@@ -1056,6 +1061,7 @@ grobs = (
     'SostenutoPedalLineSpanner',
     'SpacingSpanner',
     'SpanBar',
+    'StaffGrouper',
     'StaffSpacing',
     'StaffSymbol',
     'StanzaNumber',
@@ -1107,6 +1113,7 @@ def schemeprops(grob = None):
 
 all_user_grob_properties = (
     'add-stem-support',
+    'after-last-staff-spacing',
     'after-line-breaking',
     'align-dir',
     'allow-loose-spacing',
@@ -1123,21 +1130,21 @@ all_user_grob_properties = (
     'avoid-slur',
     'axes',
     'bar-size',
-    'baseline-skip',
     'base-shortest-duration',
+    'baseline-skip',
+    'beam-thickness',
+    'beam-width',
     'beamed-stem-shorten',
     'beaming',
     'beamlet-default-length',
     'beamlet-max-length-proportion',
-    'beam-thickness',
-    'beam-width',
     'before-line-breaking',
     'between-cols',
+    'between-staff-spacing',
     'bound-details',
     'bound-padding',
     'bracket-flare',
     'bracket-visibility',
-    'breakable',
     'break-align-anchor',
     'break-align-anchor-alignment',
     'break-align-orders',
@@ -1145,6 +1152,8 @@ all_user_grob_properties = (
     'break-align-symbols',
     'break-overshoot',
     'break-visibility',
+    'breakable',
+    'c0-position',
     'circled-tip',
     'clip-edges',
     'collapse-height',
@@ -1154,9 +1163,12 @@ all_user_grob_properties = (
     'connect-to-neighbor',
     'control-points',
     'damping',
+    'dash-definition',
     'dash-fraction',
     'dash-period',
     'default-direction',
+    'default-next-staff-spacing',
+    'details',
     'digit-names',
     'direction',
     'dot-count',
@@ -1193,6 +1205,7 @@ all_user_grob_properties = (
     'gap',
     'gap-count',
     'glyph',
+    'glyph-name',
     'glyph-name-alist',
     'grow-direction',
     'hair-thickness',
@@ -1200,12 +1213,15 @@ all_user_grob_properties = (
     'head-direction',
     'height',
     'height-limit',
+    'hide-tied-accidental-after-break',
     'horizontal-shift',
     'horizontal-skylines',
     'ignore-collision',
     'implicit',
     'inspect-index',
     'inspect-quants',
+    'inter-loose-line-spacing',
+    'inter-staff-spacing',
     'keep-fixed-while-stretching',
     'keep-inside-line',
     'kern',
@@ -1240,11 +1256,13 @@ all_user_grob_properties = (
     'neutral-direction',
     'neutral-position',
     'next',
+    'next-staff-spacing',
     'no-alignment',
     'no-ledgers',
+    'no-stem-extend',
+    'non-affinity-spacing',
     'non-default',
     'non-musical',
-    'no-stem-extend',
     'note-names',
     'outside-staff-horizontal-padding',
     'outside-staff-padding',
@@ -1287,13 +1305,14 @@ all_user_grob_properties = (
     'spacing-increment',
     'springs-and-rods',
     'stacking-dir',
+    'staff-affinity',
     'staff-padding',
     'staff-position',
     'staff-space',
     'stem-attachment',
     'stem-end-position',
-    'stemlet-length',
     'stem-spacing-correction',
+    'stemlet-length',
     'stencil',
     'stencils',
     'strict-grace-spacing',
@@ -1302,10 +1321,10 @@ all_user_grob_properties = (
     'style',
     'text',
     'text-direction',
-    'thickness',
     'thick-thickness',
+    'thickness',
     'thin-kern',
-    'threshold',
+    'threshold', #removed in 2.14
     'tie-configuration',
     'to-barline',
     'toward-stem-shift',
@@ -1314,6 +1333,7 @@ all_user_grob_properties = (
     'used',
     'vertical-skylines',
     'when',
+    'whiteout',
     'width',
     'word-space',
     'X-extent',
@@ -1392,35 +1412,58 @@ headervars = (
     
 
 papervars = (
-    'after-title-space',
-    'annotate-spacing',
-    'before-title-space',
-    'between-system-padding',
-    'between-system-space',
-    'between-title-space',
-    'blank-after-score-page-force',
-    'blank-last-page-force',
-    'blank-page-force',
+    # page
+    'paper-height',
+    'paper-width',
+    'top-margin',
     'bottom-margin',
-    'first-page-number',
-    'foot-separation',
-    'force-assignment',
-    'head-separation',
+    'left-margin',
+    'right-margin',
+    'line-width',
+    
+    # vertical
+    'after-title-space', # 2.12
+    'after-title-spacing', # 2.14
+    'before-title-space', # 2.12
+    'before-title-spacing', # 2.14
+    'between-system-padding', # 2.12
+    'between-system-space', # 2.12
+    'between-scores-system-spacing', # 2.14
+    'between-system-spacing', # 2.14
+    'between-title-space', # 2.12
+    'between-title-spacing', # 2.14
+    'bottom-system-spacing', # 2.14
+    'top-title-spacing', # 2.14
+    'top-system-spacing', # 2.14
+    'page-top-space', # 2.12 only
+    
+    # horizontal
+    'binding-offset', # 2.14
     'horizontal-shift',
     'indent',
-    'input-encoding',
-    'left-margin',
-    'line-width',
-    #'max-systems-per-page', # (LP 2.13)
-    #'min-systems-per-page', # (LP 2.13)
-    'output-scale',
+    'short-indent',
+    'two-sided', # 2.14
+    'inner-margin', # 2.14
+    'outer-margin', # 2.14
+    
+    # debugging
+    'annotate-spacing',
+    
+    # other
+    'auto-first-page-number',
+    'blank-last-page-force',
+    'blank-page-force',
+    'check-consistency', # 2.14
+    'first-page-number',
+    'foot-separation', # not in 2.14?
+    'head-separation', # not in 2.14?
+    'max-systems-per-page', # 2.14
+    'min-systems-per-page', # 2.14
     'page-breaking-between-system-padding',
     'page-count',
     'page-limit-inter-system-space',
     'page-limit-inter-system-space-factor',
-    'page-top-space',
-    'paper-height',
-    'paper-width',
+    'page-spacing-weight', # 2.14
     'print-all-headers',
     'print-first-page-number',
     'print-page-number',
@@ -1428,10 +1471,10 @@ papervars = (
     'ragged-last',
     'ragged-last-bottom',
     'ragged-right',
-    'right-margin',
     'system-separator-markup',
-    #'systems-per-page', #(LP 2.13)
-    'top-margin',
+    'system-count',
+    'systems-per-page', # 2.14
+    
     # different markups
     'bookTitleMarkup',
     'evenFooterMarkup',
@@ -1441,6 +1484,12 @@ papervars = (
     'scoreTitleMarkup',
     'tocItemMarkup',
     'tocTitleMarkup',
+    
+    # undocumented?
+    #'blank-after-score-page-force',
+    #'force-assignment',
+    #'input-encoding',
+    #'output-scale',
 )
 
 
@@ -1452,157 +1501,165 @@ layoutvars = (
 
 
 contextproperties = (
-    'aDueText', 
-    'alignAboveContext', 
-    'alignBassFigureAccidentals', 
-    'alignBelowContext', 
-    'allowBeamBreak', 
-    'associatedVoice', 
-    'autoAccidentals', 
-    'autoBeamCheck', 
-    'autoBeaming', 
-    'autoBeamSettings', 
-    'autoCautionaries', 
-    'automaticBars', 
-    'barAlways', 
-    'barCheckSynchronize', 
-    'barNumberVisibility', 
-    'bassFigureFormatFunction', 
-    'bassStaffProperties', 
-    'beatGrouping', 
-    'beatLength', 
-    'chordChanges', 
-    'chordNameExceptions', 
-    'chordNameExceptionsFull', 
-    'chordNameExceptionsPartial', 
-    'chordNameFunction', 
-    'chordNameSeparator', 
-    'chordNoteNamer', 
-    'chordPrefixSpacer', 
-    'chordRootNamer', 
-    'clefGlyph', 
-    'clefOctavation', 
-    'clefPosition', 
-    'connectArpeggios', 
-    'countPercentRepeats', 
-    'createKeyOnClefChange', 
-    'createSpacing', 
-    'crescendoSpanner', 
-    'crescendoText', 
-    'currentBarNumber', 
-    'decrescendoSpanner', 
-    'decrescendoText', 
-    'defaultBarType', 
-    'doubleRepeatType', 
-    'doubleSlurs', 
-    'drumPitchTable', 
-    'drumStyleTable', 
-    'dynamicAbsoluteVolumeFunction', 
-    'explicitClefVisibility', 
-    'explicitKeySignatureVisibility', 
-    'extendersOverRests', 
-    'extraNatural', 
-    'figuredBassAlterationDirection', 
-    'figuredBassCenterContinuations', 
-    'figuredBassFormatter', 
-    'figuredBassPlusDirection', 
-    'fingeringOrientations', 
-    'firstClef', 
-    'followVoice', 
-    'fontSize', 
-    'forbidBreak', 
-    'forceClef', 
-    'gridInterval', 
-    'hairpinToBarline', 
-    'harmonicAccidentals', 
-    'highStringOne', 
-    'ignoreBarChecks', 
-    'ignoreFiguredBassRest', 
-    'ignoreMelismata', 
-    'implicitBassFigures', 
-    'implicitTimeSignatureVisibility', 
-    'instrumentCueName', 
-    'instrumentEqualizer', 
-    'instrumentName', 
-    'instrumentTransposition', 
-    'internalBarNumber', 
-    'keepAliveInterfaces', 
-    'keyAlterationOrder', 
-    'keySignature', 
-    'lyricMelismaAlignment', 
-    'majorSevenSymbol', 
-    'markFormatter', 
-    'maximumFretStretch', 
-    'measureLength', 
-    'measurePosition', 
-    'melismaBusyProperties', 
-    'metronomeMarkFormatter', 
-    'middleCClefPosition', 
-    'middleCOffset', 
-    'middleCPosition', 
-    'midiInstrument', 
-    'midiMaximumVolume', 
-    'midiMinimumVolume', 
-    'minimumFret', 
-    'minimumPageTurnLength', 
-    'minimumRepeatLengthForPageTurn', 
-    'noteToFretFunction', 
-    'ottavation', 
-    'output', 
-    'pedalSostenutoStrings', 
-    'pedalSostenutoStyle', 
-    'pedalSustainStrings', 
-    'pedalSustainStyle', 
-    'pedalUnaCordaStrings', 
-    'pedalUnaCordaStyle', 
-    'printKeyCancellation', 
-    'printOctaveNames', 
-    'printPartCombineTexts', 
-    'proportionalNotationDuration', 
-    'recordEventSequence', 
-    'rehearsalMark', 
-    'repeatCommands', 
-    'restNumberThreshold', 
-    'scriptDefinitions', 
-    'shapeNoteStyles', 
-    'shortInstrumentName', 
-    'shortVocalName', 
-    'skipBars', 
-    'skipTypesetting', 
-    'soloIIText', 
-    'soloText', 
-    'squashedPosition', 
-    'staffLineLayoutFunction', 
-    'stanza', 
-    'stemLeftBeamCount', 
-    'stemRightBeamCount', 
-    'stringNumberOrientations', 
-    'stringOneTopmost', 
-    'stringTunings', 
-    'strokeFingerOrientations', 
-    'subdivideBeams', 
-    'suggestAccidentals', 
-    'systemStartDelimiter', 
-    'systemStartDelimiterHierarchy', 
-    'tablatureFormat', 
-    'tempoUnitCount', 
-    'tempoUnitDuration', 
-    'tempoWholesPerMinute', 
-    'tieWaitForNote', 
-    'timeSignatureFraction', 
-    'timing', 
-    'tonic', 
-    'trebleStaffProperties', 
-    'tremoloFlags', 
-    'tupletFullLength', 
-    'tupletFullLengthNote', 
-    'tupletSpannerDuration', 
-    'useBassFigureExtenders', 
-    'verticallySpacedContexts', 
-    'vocalName', 
-    'voltaOnThisStaff', 
-    'voltaSpannerDuration', 
-    'whichBar', 
+    'aDueText',
+    'alignAboveContext',
+    'alignBassFigureAccidentals',
+    'alignBelowContext',
+    #'allowBeamBreak', #not in 2.12 and 2.14
+    'associatedVoice',
+    'autoAccidentals',
+    'autoBeamCheck',
+    'autoBeaming',
+    #'autoBeamSettings', # moved to beamSettings in 2.14
+    'autoCautionaries',
+    'automaticBars',
+    'barAlways',
+    'barCheckSynchronize',
+    'barNumberVisibility',
+    'bassFigureFormatFunction',
+    'bassStaffProperties',
+    'beamSettings',
+    'beatGrouping',
+    'beatLength',
+    'chordChanges',
+    'chordNameExceptions',
+    'chordNameExceptionsFull',
+    'chordNameExceptionsPartial',
+    'chordNameFunction',
+    'chordNameSeparator',
+    'chordNoteNamer',
+    'chordPrefixSpacer',
+    'chordRootNamer',
+    'clefGlyph',
+    'clefOctavation',
+    'clefPosition',
+    'completionBusy',
+    'connectArpeggios',
+    'countPercentRepeats',
+    'createKeyOnClefChange',
+    'createSpacing',
+    'crescendoSpanner',
+    'crescendoText',
+    'currentBarNumber',
+    'decrescendoSpanner',
+    'decrescendoText',
+    'defaultBarType',
+    'doubleRepeatType',
+    'doubleSlurs',
+    'drumPitchTable',
+    'drumStyleTable',
+    #'dynamicAbsoluteVolumeFunction', # not in 2.12 and 2.14
+    'explicitClefVisibility',
+    'explicitKeySignatureVisibility',
+    'extendersOverRests',
+    'extraNatural',
+    'figuredBassAlterationDirection',
+    'figuredBassCenterContinuations',
+    'figuredBassFormatter',
+    'figuredBassPlusDirection',
+    'fingeringOrientations',
+    'firstClef',
+    'followVoice',
+    'fontSize',
+    'forbidBreak',
+    'forceClef',
+    'fretLabels',
+    'gridInterval',
+    #'hairpinToBarline', # not in 2.12 and 2.14
+    'harmonicAccidentals',
+    'harmonicDots',
+    'highStringOne',
+    'ignoreBarChecks',
+    'ignoreFiguredBassRest',
+    'ignoreMelismata',
+    'implicitBassFigures',
+    'implicitTimeSignatureVisibility',
+    'instrumentCueName',
+    'instrumentEqualizer',
+    'instrumentName',
+    'instrumentTransposition',
+    'internalBarNumber',
+    'keepAliveInterfaces',
+    'keyAlterationOrder',
+    'keySignature',
+    'lyricMelismaAlignment',
+    'majorSevenSymbol',
+    'markFormatter',
+    'maximumFretStretch',
+    'measureLength',
+    'measurePosition',
+    'melismaBusyProperties',
+    'metronomeMarkFormatter',
+    'middleCClefPosition',
+    'middleCOffset',
+    'middleCPosition',
+    'midiInstrument',
+    'midiMaximumVolume',
+    'midiMinimumVolume',
+    'minimumFret',
+    'minimumPageTurnLength',
+    'minimumRepeatLengthForPageTurn',
+    'noChordSymbol', # since 2.14
+    'noteToFretFunction',
+    'ottavation',
+    'output',
+    'pedalSostenutoStrings',
+    'pedalSostenutoStyle',
+    'pedalSustainStrings',
+    'pedalSustainStyle',
+    'pedalUnaCordaStrings',
+    'pedalUnaCordaStyle',
+    'predefinedDiagramTable',
+    'printKeyCancellation',
+    'printOctaveNames',
+    'printPartCombineTexts',
+    'proportionalNotationDuration',
+    'recordEventSequence',
+    'rehearsalMark',
+    'repeatCommands',
+    'repeatCountVisibility',
+    'restNumberThreshold',
+    'shapeNoteStyles',
+    'shortInstrumentName',
+    'shortVocalName',
+    'skipBars',
+    'skipTypesetting',
+    'soloIIText',
+    'soloText',
+    'squashedPosition',
+    'staffLineLayoutFunction',
+    'stanza',
+    'stemLeftBeamCount',
+    'stemRightBeamCount',
+    'stringNumberOrientations',
+    'stringOneTopmost',
+    'stringTunings',
+    'strokeFingerOrientations',
+    'subdivideBeams',
+    'suggestAccidentals',
+    'systemStartDelimiter',
+    'systemStartDelimiterHierarchy',
+    'tablatureFormat',
+    'tempoHideNote',
+    'tempoText',
+    'tempoUnitCount',
+    'tempoUnitDuration',
+    'tempoWholesPerMinute',
+    'tieWaitForNote',
+    'timeSignatureFraction',
+    'timing',
+    'tonic',
+    'topLevelAlignment', # since 2.14
+    'trebleStaffProperties',
+    'tremoloFlags',
+    'tupletFullLength',
+    'tupletFullLengthNote',
+    'tupletSpannerDuration',
+    'useBassFigureExtenders',
+    'verticallySpacedContexts',
+    'vocalName',
+    'voltaSpannerDuration',
+    'whichBar',
 )
 
 
@@ -1625,6 +1682,8 @@ accidentalstyles = (
     'piano-cautionary',
     'neo-modern',
     'neo-modern-cautionary',
+    'neo-modern-voice',
+    'neo-modern-voice-cautionary',
     'dodecaphonic',
     'teaching',
     'no-reset',
@@ -1668,6 +1727,21 @@ break_visibility = (
     'center-invisible',
 )
 
+mark_formatters = (
+    'format-mark-alphabet',
+    'format-mark-barnumbers',
+    'format-mark-letters',
+    'format-mark-numbers',
+    'format-mark-box-alphabet',
+    'format-mark-box-barnumbers',
+    'format-mark-box-letters',
+    'format-mark-box-numbers',
+    'format-mark-circle-alphabet',
+    'format-mark-circle-barnumbers',
+    'format-mark-circle-letters',
+    'format-mark-circle-numbers',
+)
+
 
 set_context_re = re.compile(r'\\(un)?set\s+(' + '|'.join(contexts) + r')\s*.\s*$')
 context_re = re.compile(r'\b(' + '|'.join(contexts) + r')\s*\.\s*$')
